@@ -1,16 +1,50 @@
 
+from .utils.registry import register_function
 
 class omicverseConfig:
 
     def __init__(self,mode='cpu'):
         self.mode = mode
-        from .cylib._analytics_sender import send_analytics_full_silent
+        from .utils._analytics_sender import send_analytics_full_silent
         import datetime
         test_id_full = f"FULL-{datetime.datetime.now().strftime('%Y%m%d-%H%M%S')}"
         send_analytics_full_silent(test_id_full)
 
+    @register_function(
+        aliases=["GPU初始化", "gpu_init", "gpu_mode", "GPU模式", "rapids_init"],
+        category="utils",
+        description="Initialize GPU mode with RAPIDS for accelerated single-cell analysis",
+        examples=[
+            "# Initialize GPU mode with default settings",
+            "ov.settings.gpu_init()",
+            "# Custom GPU initialization",
+            "ov.settings.gpu_init(managed_memory=False, pool_allocator=True)",
+            "# Use specific GPU device", 
+            "ov.settings.gpu_init(devices=1)",
+            "# Check current mode",
+            "print(f'Current mode: {ov.settings.mode}')"
+        ],
+        related=["settings.cpu_init", "settings.cpu_gpu_mixed_init", "pp.anndata_to_GPU"]
+    )
     def gpu_init(self,managed_memory=True,pool_allocator=True,devices=0):
-        
+        r"""Initialize GPU mode with RAPIDS for accelerated single-cell analysis.
+
+        Arguments:
+            managed_memory: Enable NVIDIA Unified Memory for oversubscription. Default: True.
+            pool_allocator: Enable memory pool allocator for faster allocations. Default: True.
+            devices: GPU device IDs to register. Default: 0.
+
+        Returns:
+            None: Sets the mode to 'gpu' and configures RAPIDS environment.
+
+        Examples:
+            >>> import omicverse as ov
+            >>> # Initialize GPU mode with default settings
+            >>> ov.settings.gpu_init()
+            >>> # Custom GPU initialization
+            >>> ov.settings.gpu_init(managed_memory=False, pool_allocator=True)
+        """
+
         import scanpy as sc
         import cupy as cp
 
@@ -33,9 +67,61 @@ class omicverseConfig:
         print('CPU mode activated')
         self.mode = 'cpu'
     
+    @register_function(
+        aliases=["CPU-GPU混合模式", "cpu_gpu_mixed_init", "mixed_mode", "GPU混合模式", "mixed_init"],
+        category="utils",
+        description="Initialize CPU-GPU mixed mode for accelerated single-cell analysis",
+        examples=[
+            "# Initialize mixed mode for better performance", 
+            "ov.settings.cpu_gpu_mixed_init()",
+            "# Use mixed mode with preprocessing",
+            "ov.settings.cpu_gpu_mixed_init()",
+            "adata = ov.pp.qc(adata)  # Automatically uses mixed mode",
+            "# Check current mode",
+            "print(f'Current mode: {ov.settings.mode}')"
+        ],
+        related=["settings.gpu_init", "settings.cpu_init", "pp.qc", "pp.preprocess"]
+    )
     def cpu_gpu_mixed_init(self):
+        r"""Initialize CPU-GPU mixed mode for accelerated single-cell analysis.
+
+        Arguments:
+            None
+
+        Returns:
+            None: Sets the mode to 'cpu-gpu-mixed' and detects available GPU accelerators.
+
+        Examples:
+            >>> import omicverse as ov
+            >>> # Initialize mixed mode for better performance
+            >>> ov.settings.cpu_gpu_mixed_init()
+            >>> # Use mixed mode with preprocessing
+            >>> ov.settings.cpu_gpu_mixed_init()
+            >>> adata = ov.pp.qc(adata)  # Automatically uses mixed mode
+        """
         print('CPU-GPU mixed mode activated')
+        
+        # Detect available GPU accelerators for mixed mode
+        if torch is not None:
+            available_devices = []
+            if torch.cuda.is_available():
+                available_devices.append("CUDA")
+            if hasattr(torch.backends, 'mps') and torch.backends.mps.is_available():
+                available_devices.append("MPS")
+            if hasattr(torch.version, 'hip') and torch.version.hip is not None:
+                available_devices.append("ROCm")
+            if hasattr(torch, 'xpu') and torch.xpu.is_available():
+                available_devices.append("XPU")
+            
+            if available_devices:
+                print(f'Available GPU accelerators: {", ".join(available_devices)}')
+            else:
+                print('No GPU accelerators detected - fallback to CPU')
+        
         self.mode = 'cpu-gpu-mixed'
+    
+
+
 
 
 import subprocess
@@ -44,6 +130,277 @@ try:
 except ImportError:  # pragma: no cover - optional dependency
     torch = None
 
+def check_acceleration_packages():
+    """
+    Check which acceleration packages are installed and provide installation guidance.
+    
+    Returns
+    -------
+    dict
+        Dictionary containing installation status and recommendations
+    """
+    status = {
+        'torchdr': {'installed': False, 'recommended': False, 'device': 'cuda'},
+        'mlx': {'installed': False, 'recommended': False, 'device': 'mps'},
+        'torch': {'installed': False, 'recommended': False, 'device': 'any'}
+    }
+    
+    # Check PyTorch
+    try:
+        import torch
+        status['torch']['installed'] = True
+        status['torch']['recommended'] = True
+    except ImportError:
+        pass
+    
+    # Check TorchDR
+    try:
+        import torchdr
+        status['torchdr']['installed'] = True
+        status['torchdr']['recommended'] = True
+    except ImportError:
+        if torch is not None and torch.cuda.is_available():
+            status['torchdr']['recommended'] = True
+    
+    # Check MLX
+    try:
+        import mlx.core as mx
+        status['mlx']['installed'] = True
+        if mx.metal.is_available():
+            status['mlx']['recommended'] = True
+    except ImportError:
+        if torch is not None and hasattr(torch.backends, 'mps') and torch.backends.mps.is_available():
+            status['mlx']['recommended'] = True
+    
+    return status
+
+
+def print_acceleration_status(verbose=True):
+    """
+    Print the current acceleration package status and installation recommendations.
+    
+    Parameters
+    ----------
+    verbose : bool
+        Whether to print detailed information
+    """
+    status = check_acceleration_packages()
+    
+    print(f"{Colors.BLUE}🚀 Omicverse Acceleration Status:{Colors.ENDC}")
+    print("=" * 50)
+    
+    # PyTorch status
+    if status['torch']['installed']:
+        print(f"{Colors.GREEN}✅ PyTorch: Installed{Colors.ENDC}")
+        if verbose:
+            try:
+                import torch
+                print(f"   Version: {torch.__version__}")
+                if torch.cuda.is_available():
+                    print(f"   CUDA: Available ({torch.cuda.device_count()} device(s))")
+                if hasattr(torch.backends, 'mps') and torch.backends.mps.is_available():
+                    print(f"   MPS: Available (Apple Silicon)")
+            except:
+                pass
+    else:
+        print(f"{Colors.WARNING}⚠️ PyTorch: Not installed{Colors.ENDC}")
+        print(f"   {Colors.CYAN}Install with: pip install torch{Colors.ENDC}")
+    
+    # TorchDR status
+    if status['torchdr']['installed']:
+        print(f"{Colors.GREEN}✅ TorchDR: Installed{Colors.ENDC}")
+        if verbose:
+            try:
+                import torchdr
+                print(f"   Version: {torchdr.__version__}")
+            except:
+                pass
+    elif status['torchdr']['recommended']:
+        print(f"{Colors.WARNING}⚠️ TorchDR: Not installed (Recommended for CUDA){Colors.ENDC}")
+        print(f"   {Colors.CYAN}Install with: pip install torchdr{Colors.ENDC}")
+        print(f"   {Colors.CYAN}For GPU-accelerated dimensionality reduction{Colors.ENDC}")
+    else:
+        print(f"{Colors.FAIL}❌ TorchDR: Not installed{Colors.ENDC}")
+    
+    # MLX status
+    if status['mlx']['installed']:
+        print(f"{Colors.GREEN}✅ MLX: Installed{Colors.ENDC}")
+        if verbose:
+            try:
+                import mlx.core as mx
+                if mx.metal.is_available():
+                    print(f"   Metal: Available (Apple Silicon GPU)")
+                else:
+                    print(f"   Metal: Not available")
+            except:
+                pass
+    elif status['mlx']['recommended']:
+        print(f"{Colors.WARNING}⚠️ MLX: Not installed (Recommended for Apple Silicon){Colors.ENDC}")
+        print(f"   {Colors.CYAN}Install with: pip install mlx{Colors.ENDC}")
+        print(f"   {Colors.CYAN}For Apple Silicon GPU acceleration{Colors.ENDC}")
+    else:
+        print(f"{Colors.FAIL}❌ MLX: Not installed{Colors.ENDC}")
+    
+    print("=" * 50)
+    
+    # Recommendations
+    recommendations = []
+    if not status['torch']['installed']:
+        recommendations.append("Install PyTorch for basic GPU support")
+    if status['torchdr']['recommended'] and not status['torchdr']['installed']:
+        recommendations.append("Install TorchDR for CUDA-accelerated PCA")
+    if status['mlx']['recommended'] and not status['mlx']['installed']:
+        recommendations.append("Install MLX for Apple Silicon GPU acceleration")
+    
+    if recommendations:
+        print(f"{Colors.CYAN}💡 Recommendations:{Colors.ENDC}")
+        for i, rec in enumerate(recommendations, 1):
+            print(f"   {i}. {rec}")
+    else:
+        print(f"{Colors.GREEN}🎉 All recommended packages are installed!{Colors.ENDC}")
+
+
+def get_optimal_device(prefer_gpu=True, verbose=False):
+    """
+    Get the optimal PyTorch device based on available hardware.
+    Now includes acceleration package status checking.
+    
+    Priority order:
+    1. CUDA (NVIDIA GPUs) - requires TorchDR for optimal performance
+    2. MPS (Apple Silicon) - requires MLX for optimal performance
+    3. ROCm (AMD GPUs) 
+    4. XPU (Intel GPUs)
+    5. CPU (fallback)
+    
+    Parameters
+    ----------
+    prefer_gpu : bool
+        Whether to prefer GPU over CPU when available
+    verbose : bool
+        Whether to print device selection information and acceleration status
+        
+    Returns
+    -------
+    torch.device
+        The optimal device for computation
+    """
+    if torch is None:
+        if verbose:
+            print("PyTorch not available, using CPU")
+            print(f"{Colors.WARNING}💡 Install PyTorch for GPU support: pip install torch{Colors.ENDC}")
+        return "cpu"
+    
+    if not prefer_gpu:
+        if verbose:
+            print("GPU preference disabled, using CPU")
+        return torch.device("cpu")
+    
+    # Check acceleration packages if verbose
+    if verbose:
+        #print_acceleration_status(verbose=True)
+        print()  # Add spacing
+    
+    # Check devices in priority order
+    if torch.cuda.is_available():
+        device = torch.device("cuda")
+        if verbose:
+            print(f"Using CUDA device: {torch.cuda.get_device_name()}")
+            # Check if TorchDR is available for optimal performance
+            try:
+                import torchdr
+                print(f"{Colors.GREEN}✅ TorchDR available for GPU-accelerated PCA{Colors.ENDC}")
+            except ImportError:
+                print(f"{Colors.WARNING}⚠️ TorchDR not installed - install with: pip install torchdr{Colors.ENDC}")
+        return device
+    
+    if hasattr(torch.backends, 'mps') and torch.backends.mps.is_available():
+        device = torch.device("mps")
+        if verbose:
+            print("Using Apple Silicon MPS device (note: float32 required)")
+            # Check if MLX is available for optimal performance
+            try:
+                import mlx.core as mx
+                if mx.metal.is_available():
+                    print(f"{Colors.GREEN}✅ MLX available for Apple Silicon GPU acceleration{Colors.ENDC}")
+                else:
+                    print(f"{Colors.WARNING}⚠️ MLX Metal not available{Colors.ENDC}")
+            except ImportError:
+                print(f"{Colors.WARNING}⚠️ MLX not installed - install with: pip install mlx{Colors.ENDC}")
+        return device
+    
+    if hasattr(torch.version, 'hip') and torch.version.hip is not None:
+        # ROCm available
+        device = torch.device("cuda")  # ROCm uses cuda interface
+        if verbose:
+            print("Using AMD ROCm device")
+        return device
+    
+    if hasattr(torch, 'xpu') and torch.xpu.is_available():
+        device = torch.device("xpu")
+        if verbose:
+            print("Using Intel XPU device")
+        return device
+    
+    # Fallback to CPU
+    device = torch.device("cpu")
+    if verbose:
+        print("No GPU available, using CPU")
+        print(f"{Colors.CYAN}💡 For GPU acceleration, ensure PyTorch is installed with GPU support{Colors.ENDC}")
+    return device
+
+def prepare_data_for_device(X, device, verbose=False):
+    """
+    Prepare data for a specific device, handling device-specific requirements.
+    
+    Parameters
+    ----------
+    X : array-like
+        Input data (numpy array, sparse matrix, or other array-like)
+    device : torch.device
+        Target device
+    verbose : bool
+        Whether to print conversion information
+        
+    Returns
+    -------
+    X_prepared : array-like
+        Data prepared for the target device
+    """
+    import numpy as np
+    from scipy import sparse
+    
+    # Handle MPS float64 limitation
+    if hasattr(device, 'type') and device.type == 'mps':
+        # Handle numpy arrays
+        if hasattr(X, 'dtype') and X.dtype == np.float64:
+            if verbose:
+                print("   Converting float64 to float32 for MPS compatibility")
+            X = X.astype(np.float32)
+        
+        # Handle sparse matrices
+        elif sparse.issparse(X) and X.dtype == np.float64:
+            if verbose:
+                print("   Converting sparse matrix float64 to float32 for MPS compatibility")
+            X = X.astype(np.float32)
+        
+        # Handle AnnData objects
+        elif hasattr(X, 'X'):
+            if hasattr(X.X, 'dtype') and X.X.dtype == np.float64:
+                if verbose:
+                    print("   Converting AnnData.X float64 to float32 for MPS compatibility")
+                if sparse.issparse(X.X):
+                    X.X = X.X.astype(np.float32)
+                else:
+                    X.X = X.X.astype(np.float32)
+        
+        # Handle other array-like objects with dtype
+        elif hasattr(X, 'astype') and hasattr(X, 'dtype') and X.dtype == np.float64:
+            if verbose:
+                print("   Converting array float64 to float32 for MPS compatibility")
+            X = X.astype(np.float32)
+    
+    return X
+
 def check_reference_key(adata):
     if 'REFERENCE_MANU' not in adata.uns.keys():
         adata.uns['REFERENCE_MANU']={}
@@ -424,5 +781,27 @@ class Colors:
     BOLD = '\033[1m'        # Bold
     UNDERLINE = '\033[4m'   # Underline
 
+# Convenience function for users to check acceleration status
+def check_gpu_acceleration():
+    """
+    Convenience function to check GPU acceleration status and get installation recommendations.
+    
+    This function provides a user-friendly way to check which acceleration packages
+    are installed and get recommendations for optimal performance.
+    
+    Examples
+    --------
+    >>> import omicverse as ov
+    >>> ov.settings.check_gpu_acceleration()
+    
+    Returns
+    -------
+    dict
+        Dictionary containing installation status and recommendations
+    """
+    print_acceleration_status(verbose=True)
+    return check_acceleration_packages()
+
+
 settings = omicverseConfig()
         
\ No newline at end of file
