@@ -4,7 +4,7 @@ Different Expression analysis in Python
 
 import numpy as np
 import pandas as pd
-import scanpy as sc
+
 import statsmodels.api as sm
 import anndata as ad
 import matplotlib.patches as mpatches
@@ -12,14 +12,26 @@ import matplotlib.pyplot as plt
 import matplotlib
 from typing import Union,Tuple
 from ..utils import plot_boxplot
+from ..utils.registry import register_function
 from ..pl import volcano
 
-def Matrix_ID_mapping(data:pd.DataFrame,gene_ref_path:str)->pd.DataFrame:
+@register_function(
+    aliases=["基因ID映射", "gene_id_mapping", "id_mapping", "基因符号转换", "gene_symbol_mapping"],
+    category="bulk",
+    description="Map gene IDs to gene symbols using a reference table for bulk RNA-seq data",
+    examples=[
+        "ov.bulk.Matrix_ID_mapping(data, gene_ref_path='gene_reference.txt')",
+        "ov.bulk.Matrix_ID_mapping(data, gene_ref_path='gene_ref.tsv', keep_unmapped=False)"
+    ],
+    related=["bulk.deseq2_normalize", "utils.gene_symbol_to_ensembl", "pp.filter_genes"]
+)
+def Matrix_ID_mapping(data:pd.DataFrame,gene_ref_path:str,keep_unmapped:bool=True)->pd.DataFrame:
     r"""Map gene IDs in the input data to gene symbols using a reference table.
 
     Arguments:
         data: The input data containing gene IDs as index.
         gene_ref_path: The path to the reference table containing the mapping from gene IDs to gene symbols.
+        keep_unmapped: Whether to keep genes that are not found in the mapping table. If True, unmapped genes retain their original IDs. If False, unmapped genes are removed (original behavior). Default: True.
 
     Returns:
         data: The input data with gene IDs mapped to gene symbols.
@@ -27,17 +39,43 @@ def Matrix_ID_mapping(data:pd.DataFrame,gene_ref_path:str)->pd.DataFrame:
     """
     
     pair=pd.read_csv(gene_ref_path,sep='\t',index_col=0)
-    ret_gene=list(set(data.index.tolist()) & set(pair.index.tolist()))
-    data=data.loc[ret_gene]
-    #data=data_drop_duplicates_index(data)
-    new_index=[]
-    for i in ret_gene:
-        a=pair.loc[i,'symbol']
-        if str(a)=='nan':
-            new_index.append(i)
-        else:
-            new_index.append(a)
-    data.index=new_index
+    
+    if keep_unmapped:
+        # Keep all genes, map those that exist in the reference
+        all_genes = data.index.tolist()
+        mapped_genes = list(set(all_genes) & set(pair.index.tolist()))
+        unmapped_genes = list(set(all_genes) - set(pair.index.tolist()))
+        
+        new_index = []
+        
+        # Process mapped genes
+        for gene in all_genes:
+            if gene in pair.index:
+                symbol = pair.loc[gene, 'symbol']
+                if str(symbol) == 'nan':
+                    new_index.append(gene)  # Keep original ID if symbol is NaN
+                else:
+                    new_index.append(symbol)
+            else:
+                new_index.append(gene)  # Keep original ID for unmapped genes
+        
+        data.index = new_index
+        print(f"......Mapped {len(mapped_genes)} genes to symbols, kept {len(unmapped_genes)} unmapped genes with original IDs")
+    else:
+        # Original behavior: only keep genes that can be mapped
+        original_genes = data.index.tolist()
+        ret_gene=list(set(original_genes) & set(pair.index.tolist()))
+        data=data.loc[ret_gene]
+        new_index=[]
+        for i in ret_gene:
+            a=pair.loc[i,'symbol']
+            if str(a)=='nan':
+                new_index.append(i)
+            else:
+                new_index.append(a)
+        data.index=new_index
+        print(f"......Mapped {len(ret_gene)} genes to symbols, removed {len(original_genes) - len(ret_gene)} unmapped genes")
+    
     return data
 
 
@@ -157,6 +195,29 @@ def data_drop_duplicates_index(data:pd.DataFrame)->pd.DataFrame:
     data = data.loc[~data.index.duplicated(keep='first')]
     return data
 
+@register_function(
+    aliases=["差异表达分析", "DEG", "differential_expression", "差异基因分析", "pyDEG"],
+    category="bulk", 
+    description="Python implementation of differential expression analysis for bulk RNA-seq data",
+    examples=[
+        "# Initialize with raw count data",
+        "dds = ov.bulk.pyDEG(raw_count_data)",
+        "# Remove duplicate gene IDs",
+        "dds.drop_duplicates_index()",
+        "# Normalize using DESeq2 method",
+        "dds.normalize()",
+        "# Perform differential expression analysis",
+        "dds.deg_analysis(treatment_groups, control_groups, method='DEseq2')",
+        "# Set fold change thresholds",
+        "dds.foldchange_set(fc_threshold=2, pval_threshold=0.05)",
+        "# Visualize results",
+        "dds.plot_volcano(title='DEG Analysis')",
+        "dds.plot_boxplot(genes=['GENE1', 'GENE2'], treatment_groups, control_groups)",
+        "# Prepare ranking for GSEA",
+        "ranked_genes = dds.ranking2gsea()"
+    ],
+    related=["bulk.deseq2_normalize", "single.rank_genes_groups", "utils.volcano_plot"]
+)
 class pyDEG(object):
 
 
@@ -763,4 +824,4 @@ class pyDEG(object):
             
             
         else:  # This is where the "method" check (not pydeseq2 version check) ends
-            raise ValueError('The method is not supported.')
+            raise ValueError('The method is not supported.')
\ No newline at end of file
