@@ -1,538 +1,68 @@
 import scanpy as sc
 import numpy as np
 import pandas as pd
-
+from ..utils.registry import register_function
 
 sc_color=[
- '#1F577B', '#A56BA7', '#E0A7C8', '#E069A6', '#941456', '#FCBC10', '#EF7B77', '#279AD7','#F0EEF0',
- '#EAEFC5', '#7CBB5F','#368650','#A499CC','#5E4D9A','#78C2ED','#866017', '#9F987F','#E0DFED',
- '#01A0A7', '#75C8CC', '#F0D7BC', '#D5B26C', '#D5DA48', '#B6B812', '#9DC3C3', '#A89C92', '#FEE00C', '#FEF2A1']
+ '#1F577B', '#A56BA7', '#E0A7C8', '#E069A6', '#941456', 
+ '#FCBC10', '#EF7B77', '#279AD7','#F0EEF0',
+ '#EAEFC5', '#7CBB5F','#368650','#A499CC','#5E4D9A',
+ '#78C2ED','#866017', '#9F987F','#E0DFED',
+ '#01A0A7', '#75C8CC', '#F0D7BC', '#D5B26C', '#D5DA48',
+ '#B6B812', '#9DC3C3', '#A89C92', '#FEE00C', '#FEF2A1']
 
-red_color=['#F0C3C3','#E07370','#CB3E35','#A22E2A','#5A1713','#D3396D','#DBC3DC','#85539B','#5C2B80','#5C4694']
-green_color=['#91C79D','#8FC155','#56AB56','#2D5C33','#BBCD91','#6E944A','#A5C953','#3B4A25','#010000']
-orange_color=['#EFBD49','#D48F3E','#AC8A3E','#7D7237','#745228','#E1C085','#CEBC49','#EBE3A1','#6C6331','#8C9A48','#D7DE61']
-blue_color=['#347862','#6BBBA0','#81C0DD','#3E8CB1','#88C8D2','#52B3AD','#265B58','#B2B0D4','#5860A7','#312C6C']
-purple_color=['#823d86','#825b94','#bb98c6','#c69bc6','#a69ac9','#c5a6cc','#caadc4','#d1c3d4']
+red_color=['#F0C3C3','#E07370','#CB3E35','#A22E2A','#5A1713',
+           '#D3396D','#8B0000', '#A52A2A', '#CD5C5C', '#DC143C' ]
+
+green_color=['#91C79D','#8FC155','#56AB56','#2D5C33','#BBCD91',
+             '#6E944A','#A5C953','#3B4A25','#010000']
+
+orange_color=['#EFBD49','#D48F3E','#AC8A3E','#7D7237','#745228',
+              '#E1C085','#CEBC49','#EBE3A1','#6C6331','#8C9A48','#D7DE61']
+
+blue_color=['#1F577B', '#279AD7', '#78C2ED', '#01A0A7', '#75C8CC', '#9DC3C3',
+            '#3E8CB1', '#52B3AD', '#265B58', '#5860A7', '#312C6C', '#4CC9F0']
+
+purple_color=['#823d86','#825b94','#bb98c6','#c69bc6','#a69ac9',
+              '#c5a6cc','#caadc4','#d1c3d4']
 
 #more beautiful colors
 # 28-color palettes with distinct neighboring colors
-palette_28 = sc_color
-
+palette_28 = sc_color[:28]
 # 56-color palette with clear transitions
-palette_56 = ['#d60000',
- '#018700',
- '#b500ff',
- '#05acc6',
- '#97ff00',
- '#ffa52f',
- '#ff8ec8',
- '#79525e',
- '#00fdcf',
- '#afa5ff',
- '#93ac83',
- '#9a6900',
- '#366962',
- '#d3008c',
- '#fdf490',
- '#c86e66',
- '#9ee2ff',
- '#00c846',
- '#a877ac',
- '#b8ba01',
- '#f4bfb1',
- '#ff28fd',
- '#f2cdff',
- '#009e7c',
- '#ff6200',
- '#56642a',
- '#953f1f',
- '#90318e',
- '#ff3464',
- '#a0e491',
- '#8c9ab1',
- '#829026',
- '#ae083f',
- '#77c6ba',
- '#bc9157',
- '#e48eff',
- '#72b8ff',
- '#c6a5c1',
- '#ff9070',
- '#d3c37c',
- '#bceddb',
- '#6b8567',
- '#916e56',
- '#f9ff00',
- '#bac1df',
- '#ac567c',
- '#ffcd03',
- '#ff49b1',
- '#c15603',
- '#5d8c90',
- '#c144bc',
- '#00753f',
- '#ba6efd',
- '#00d493',
- '#00ff75',
- '#49a150',
- '#cc9790',
- '#00ebed',
- '#db7e01',
- '#f77589',
- '#b89500',
- '#c84248',
- '#00cff9',
- '#755726',
- '#85d401',
- '#ebffd4',
- '#a77b87',
- '#db72c8',
- '#cae256',
- '#8abf5d',
- '#a1216b',
- '#855b89',
- '#89bacf',
- '#ffbad6',
- '#b6cfaa',
- '#97414d',
- '#67aa00',
- '#fde1b1',
- '#ff3628',
- '#80793d',
- '#d6e8ff',
- '#a795c6',
- '#7ea59a',
- '#d182a3',
- '#54823b',
- '#e6a872',
- '#9cffff',
- '#da5480',
- '#05b3aa',
- '#ffaaf6',
- '#d1afef',
- '#da015d',
- '#ac1a13',
- '#60b385',
- '#d442fd',
- '#acaa59',
- '#fb9ca7',
- '#b3723b',
- '#f26952',
- '#aed1d4',
- '#9affc3',
- '#dbb333',
- '#eb01c3',
- '#9900c4',
- '#cfff9e',
- '#a55949',
- '#3b6d01',
- '#008579',
- '#959167',
- '#89dbb3',
- '#6d7400',
- '#aa5dca',
- '#07ef00',
- '#804f3d',
- '#d88052',
- '#ffc862',
- '#b8009e',
- '#99acdd',
- '#904f00',
- '#8c4470',
- '#4f6e52',
- '#ff8734',
- '#c68ecd',
- '#d4e29e',
- '#b1826d',
- '#9cfb75',
- '#56dd77',
- '#f90087',
- '#a1cdff',
- '#13cad1',
- '#118e54',
- '#d154a5',
- '#00dfc3',
- '#a3832f',
- '#77975b',
- '#baaa80',
- '#70a3af',
- '#d6fbff',
- '#e8013a',
- '#d84621',
- '#ff82ed',
- '#b63862',
- '#b6cd72',
- '#97626b',
- '#897490',
- '#00a316',
- '#00f4a1',
- '#bf90f2',
- '#89e4d8',
- '#a34d95',
- '#6e5d00',
- '#8cc68e',
- '#95aa2a',
- '#c672dd',
- '#b33b01',
- '#d69a36',
- '#dfacb6',
- '#009aa0',
- '#599000',
- '#97bca8',
- '#ac8ca8',
- '#dad4ff',
- '#547c72',
- '#00ba69',
- '#ffc38e',
- '#b800d4',
- '#dfcf5b',
- '#629a7b',
- '#bfedbc',
- '#c1bdfd',
- '#80d3dd',
- '#e2857e',
- '#f9eb4d',
- '#bf6d82',
- '#caff4f',
- '#ef72aa',
- '#ed67ff',
- '#9946ae',
- '#6d6942',
- '#e25660',
- '#dd662d',
- '#9cdb5d',
- '#e29ccf',
- '#b87500',
- '#c6002d',
- '#dfbcda',
- '#59b5df',
- '#ff59da',
- '#38c1a1',
- '#9e698c',
- '#acaac8',
- '#95622f',
- '#b55662',
- '#2b7e60',
- '#b1e400',
- '#eda590',
- '#95fde2',
- '#ff548e',
- '#bd6ea1',
- '#aa3b36',
- '#d8cf00',
- '#aa80cd',
- '#a08052',
- '#e100e8',
- '#c35b3d',
- '#b53a85',
- '#8c7700',
- '#dbbc95',
- '#529e93',
- '#afbc82',
- '#91b5b6',
- '#a75423',
- '#ffd4ef',
- '#79ae6b',
- '#5db54b',
- '#80fb9a',
- '#48ffef',
- '#979548',
- '#9387a7',
- '#31d400',
- '#6ee956',
- '#b6d4eb',
- '#705470',
- '#f2db8a',
- '#aad4c1',
- '#7ecdf2',
- '#89ba00',
- '#64b6ba',
- '#ffb500',
- '#c38285',
- '#caaa5e',
- '#647748',
- '#59e2ff',
- '#df4dcd',
- '#e9ff79',
- '#bc66b8',
- '#c395a5',
- '#64c674',
- '#d19570',
- '#70cf4f',
- '#aa6e66',
- '#9c60a5',
- '#00b800',
- '#e299b3',
- '#bc006b',
- '#b3e8ef',
- '#cdbfe4',
- '#77a342',
- '#856277',
- '#568e5b',
- '#9eafc4',
- '#e82fa0',
- '#247c2a',
- '#826723',
- '#bfbc4d',
- '#ddd3a5'][:56]
 
 # 112-color palette with distinct transitions
-palette_112 = ['#d60000',
- '#018700',
- '#b500ff',
- '#05acc6',
- '#97ff00',
- '#ffa52f',
- '#ff8ec8',
- '#79525e',
- '#00fdcf',
- '#afa5ff',
- '#93ac83',
- '#9a6900',
- '#366962',
- '#d3008c',
- '#fdf490',
- '#c86e66',
- '#9ee2ff',
- '#00c846',
- '#a877ac',
- '#b8ba01',
- '#f4bfb1',
- '#ff28fd',
- '#f2cdff',
- '#009e7c',
- '#ff6200',
- '#56642a',
- '#953f1f',
- '#90318e',
- '#ff3464',
- '#a0e491',
- '#8c9ab1',
- '#829026',
- '#ae083f',
- '#77c6ba',
- '#bc9157',
- '#e48eff',
- '#72b8ff',
- '#c6a5c1',
- '#ff9070',
- '#d3c37c',
- '#bceddb',
- '#6b8567',
- '#916e56',
- '#f9ff00',
- '#bac1df',
- '#ac567c',
- '#ffcd03',
- '#ff49b1',
- '#c15603',
- '#5d8c90',
- '#c144bc',
- '#00753f',
- '#ba6efd',
- '#00d493',
- '#00ff75',
- '#49a150',
- '#cc9790',
- '#00ebed',
- '#db7e01',
- '#f77589',
- '#b89500',
- '#c84248',
- '#00cff9',
- '#755726',
- '#85d401',
- '#ebffd4',
- '#a77b87',
- '#db72c8',
- '#cae256',
- '#8abf5d',
- '#a1216b',
- '#855b89',
- '#89bacf',
- '#ffbad6',
- '#b6cfaa',
- '#97414d',
- '#67aa00',
- '#fde1b1',
- '#ff3628',
- '#80793d',
- '#d6e8ff',
- '#a795c6',
- '#7ea59a',
- '#d182a3',
- '#54823b',
- '#e6a872',
- '#9cffff',
- '#da5480',
- '#05b3aa',
- '#ffaaf6',
- '#d1afef',
- '#da015d',
- '#ac1a13',
- '#60b385',
- '#d442fd',
- '#acaa59',
- '#fb9ca7',
- '#b3723b',
- '#f26952',
- '#aed1d4',
- '#9affc3',
- '#dbb333',
- '#eb01c3',
- '#9900c4',
- '#cfff9e',
- '#a55949',
- '#3b6d01',
- '#008579',
- '#959167',
- '#89dbb3',
- '#6d7400',
- '#aa5dca',
- '#07ef00',
- '#804f3d',
- '#d88052',
- '#ffc862',
- '#b8009e',
- '#99acdd',
- '#904f00',
- '#8c4470',
- '#4f6e52',
- '#ff8734',
- '#c68ecd',
- '#d4e29e',
- '#b1826d',
- '#9cfb75',
- '#56dd77',
- '#f90087',
- '#a1cdff',
- '#13cad1',
- '#118e54',
- '#d154a5',
- '#00dfc3',
- '#a3832f',
- '#77975b',
- '#baaa80',
- '#70a3af',
- '#d6fbff',
- '#e8013a',
- '#d84621',
- '#ff82ed',
- '#b63862',
- '#b6cd72',
- '#97626b',
- '#897490',
- '#00a316',
- '#00f4a1',
- '#bf90f2',
- '#89e4d8',
- '#a34d95',
- '#6e5d00',
- '#8cc68e',
- '#95aa2a',
- '#c672dd',
- '#b33b01',
- '#d69a36',
- '#dfacb6',
- '#009aa0',
- '#599000',
- '#97bca8',
- '#ac8ca8',
- '#dad4ff',
- '#547c72',
- '#00ba69',
- '#ffc38e',
- '#b800d4',
- '#dfcf5b',
- '#629a7b',
- '#bfedbc',
- '#c1bdfd',
- '#80d3dd',
- '#e2857e',
- '#f9eb4d',
- '#bf6d82',
- '#caff4f',
- '#ef72aa',
- '#ed67ff',
- '#9946ae',
- '#6d6942',
- '#e25660',
- '#dd662d',
- '#9cdb5d',
- '#e29ccf',
- '#b87500',
- '#c6002d',
- '#dfbcda',
- '#59b5df',
- '#ff59da',
- '#38c1a1',
- '#9e698c',
- '#acaac8',
- '#95622f',
- '#b55662',
- '#2b7e60',
- '#b1e400',
- '#eda590',
- '#95fde2',
- '#ff548e',
- '#bd6ea1',
- '#aa3b36',
- '#d8cf00',
- '#aa80cd',
- '#a08052',
- '#e100e8',
- '#c35b3d',
- '#b53a85',
- '#8c7700',
- '#dbbc95',
- '#529e93',
- '#afbc82',
- '#91b5b6',
- '#a75423',
- '#ffd4ef',
- '#79ae6b',
- '#5db54b',
- '#80fb9a',
- '#48ffef',
- '#979548',
- '#9387a7',
- '#31d400',
- '#6ee956',
- '#b6d4eb',
- '#705470',
- '#f2db8a',
- '#aad4c1',
- '#7ecdf2',
- '#89ba00',
- '#64b6ba',
- '#ffb500',
- '#c38285',
- '#caaa5e',
- '#647748',
- '#59e2ff',
- '#df4dcd',
- '#e9ff79',
- '#bc66b8',
- '#c395a5',
- '#64c674',
- '#d19570',
- '#70cf4f',
- '#aa6e66',
- '#9c60a5',
- '#00b800',
- '#e299b3',
- '#bc006b',
- '#b3e8ef',
- '#cdbfe4',
- '#77a342',
- '#856277',
- '#568e5b',
- '#9eafc4',
- '#e82fa0',
- '#247c2a',
- '#826723',
- '#bfbc4d',
- '#ddd3a5'][:112]
+cet_g_bw = [
+ '#d60000', '#8c3bff', '#018700', '#00acc6', '#97ff00', '#ff7ed1', '#6b004f', '#ffa52f', '#00009c', '#857067',
+ '#004942', '#4f2a00', '#00fdcf', '#bcb6ff', '#95b379', '#bf03b8', '#2466a1', '#280041', '#dbb3af', '#fdf490',
+ '#4f445b', '#a37c00', '#ff7066', '#3f806e', '#82000c', '#a37bb3', '#344d00', '#9ae4ff', '#eb0077', '#2d000a',
+ '#5d90ff', '#00c61f', '#5701aa', '#001d00', '#9a4600', '#959ea5', '#9a425b', '#001f31', '#c8c300', '#ffcfff',
+ '#00bd9a', '#3615ff', '#2d2424', '#df57ff', '#bde6bf', '#7e4497', '#524f3b', '#d86600', '#647438', '#c17287',
+ '#6e7489', '#809c03', '#bd8a64', '#623338', '#cacdda', '#6beb82', '#213f69', '#a17eff', '#fd03ca', '#75bcfd',
+ '#d8c382', '#cda3cd', '#6d4f00', '#006974', '#469e5d', '#93c6bf', '#f9ff00', '#bf5444', '#00643b', '#5b4fa8',
+ '#521f64', '#4f5eff', '#7e8e77', '#b808f9', '#8a91c3', '#b30034', '#87607e', '#9e0075', '#ffddc3', '#500800',
+ '#1a0800', '#4b89b5', '#00dfdf', '#c8fff9', '#2f3415', '#ff2646', '#ff97aa', '#03001a', '#c860b1', '#c3a136',
+ '#7c4f3a', '#f99e77', '#566464', '#d193ff', '#2d1f69', '#411a34', '#af9397', '#629e99', '#bcdd7b', '#ff5d93',
+ '#0f2823', '#b8bdac', '#743b64', '#0f000c', '#7e6ebc', '#9e6b3b', '#ff4600', '#7e0087', '#ffcd3d', '#2f3b42',
+ '#fda5ff', '#89013d', '#752b01', '#0a8995', '#050052', '#8ed631', '#52c372', '#465970', '#570121', '#a52101',
+ '#90934b', '#00421d', '#8000d1', '#2f263f', '#bf3883', '#f4ffd4', '#00d3ff', '#6900f7', '#9cbad1', '#79d8aa',
+ '#69565d', '#006905', '#36369c', '#018246', '#441d18', '#07a5ef', '#ff802f', '#a754b8', '#675982', '#72ffff',
+ '#d88701', '#bad3ff', '#8e362f', '#a7a080', '#007ce2', '#8e7e8e', '#994487', '#00f034', '#aeaac8', '#a06062',
+ '#4b3a77', '#6b8282', '#f0dde6', '#ffbad3', '#38a523', '#b3ffa8', '#0c1107', '#d6526e', '#959efd', '#7c7e00',
+ '#759eb8', '#db877e', '#111318', '#d482d4', '#9e00bf', '#dbefff', '#8eaa9a', '#706442', '#493b3d', '#084d5e',
+ '#9cb844', '#d8ddd4', '#caff6b', '#b364eb', '#465d33', '#009e7c', '#c14100', '#4fbcba', '#d88ab1', '#5b72b5',
+ '#4b4101', '#95825d', '#49748a', '#ff72ff', '#82691c', '#dbcfff', '#7e6bfd', '#627560', '#ffc191', '#595d00',
+ '#e408e6', '#b8b1b6', '#d32d41', '#314236', '#d8a362', '#5b8a33', '#2f1f00', '#97e6d6', '#2a6256', '#cd724d',
+ '#5d3d28', '#0059d8', '#ac93d6', '#6b1d93', '#b3015d', '#410046', '#9cffcf', '#e4489c', '#e2e246', '#dbe2a5',
+ '#002859', '#aa5b82', '#0000db', '#4b4d50', '#dabfd4', '#004d99', '#87649e', '#691d1c', '#8e52c4', '#b8dadf',
+ '#ddb3fd', '#7b4854', '#4b7200', '#440077', '#b15e00', '#91d185', '#54334b', '#69af85', '#aa93af', '#e65442',
+ '#8e8c89', '#70ac50', '#aa7c74', '#00343b', '#240f13', '#e6af00', '#79ccdb', '#18133a', '#9c5238', '#ba7b31',
+ '#b6ca93', '#310800', '#a39505', '#00daba', '#74a0dd', '#623b72', '#ffda8e', '#77b800', '#3f2f1c', '#578759',
+ '#2d0021', '#f4a1d4', '#da00aa', '#752849', '#bce400', '#c3c15d'
+]
+
+palette_112 = cet_g_bw[:112]
+palette_56 = cet_g_bw[:56]
 
 # Vibrant palette with clear distinctions (24 colors)
 vibrant_palette = [
@@ -558,13 +88,21 @@ pastel_palette = [
     '#FFD3B5', '#D4F0F0', '#B5EAD7', '#E2F0CB', '#C7CEEA', '#FFDAC1'
 ]
 
-ditto_color=[
-            "#E69F00", "#56B4E9", "#009E73", "#F0E442", "#0072B2",
-            "#D55E00", "#CC79A7", "#666666", "#AD7700", "#1C91D4",
-            "#007756", "#D5C711", "#005685", "#A04700", "#B14380",
-            "#4D4D4D", "#FFBE2D", "#80C7EF", "#00F6B3", "#F4EB71",
-            "#06A5FF", "#FF8320", "#D99BBD", "#8C8C8C"
-        ]
+def colormaps_palette(map_name:str)->list:
+    r"""Returns a colormap palette.
+
+    colormap's name could be found in `https://pratiman-91.github.io/colormaps/`
+    
+    Arguments:
+        map_name: Name of the colormap
+        
+    Returns:
+        List of hex color codes for the colormap
+    """
+    import colormaps
+    import matplotlib.colors as mcolors
+    cmap = getattr(colormaps, map_name)
+    return [mcolors.to_hex(i) for i in cmap.colors]
 
 def optim_palette(adata,basis,colors,palette=None,**kwargs):
     """
@@ -2575,6 +2113,26 @@ def get_forbidden():
 #from ._forbiddencity import get_forbidden
 Forbiddencity = get_forbidden()
 
+@register_function(
+    aliases=["禁城色彩", "ForbiddenCity", "forbidden_city", "中国传统色", "故宫色彩"],
+    category="pl",
+    description="Traditional Chinese color palette from Forbidden City with 384 colors",
+    examples=[
+        "# Initialize Forbidden City color palette",
+        "fb = ov.pl.ForbiddenCity()",
+        "# Get specific color by name",
+        "color = fb.get_color('凝夜紫')",
+        "# Get RGB values",
+        "rgb = fb.get_color_rgb('群青')",
+        "# Create custom colormap",
+        "colors = [fb.get_color_rgb('群青'), fb.get_color_rgb('半见'), fb.get_color_rgb('丹缽')]",
+        "cmap = fb.get_cmap_seg(colors)",
+        "# Use in plotting",
+        "ov.pl.embedding(adata, basis='X_umap', color='cell_type',",
+        "                palette=fb.red[:])"
+    ],
+    related=["pl.embedding", "pl.volcano", "pl.boxplot"]
+)
 class ForbiddenCity:
 
     green1=Forbidden_Cmap(range(1,19))
