@@ -1,4 +1,5 @@
 from ..utils._scatterplot import _embedding
+from ..utils.registry import register_function
 import collections.abc as cabc
 from copy import copy
 from numbers import Integral
@@ -44,6 +45,131 @@ from scanpy.plotting._utils import (
     VBound,
 )
 
+def mde(adata: AnnData, **kwargs):
+    r"""
+    Plot MDE embedding.
+
+    Arguments:
+        adata: Annotated data matrix.
+        **kwargs: Additional keyword arguments.
+
+    Returns:
+        fig: figure and axis
+        ax: axis
+    """
+    if 'X_mde' not in adata.obsm.keys():
+        raise ValueError('X_mde not found in adata.obsm. Please run ov.pp.mde first.')
+    return embedding(adata, basis='X_mde', **kwargs)
+
+def tsne(adata: AnnData, **kwargs):
+    r"""
+    Plot t-SNE embedding.
+
+    Arguments:
+        adata: Annotated data matrix.
+        **kwargs: Additional keyword arguments.
+
+    Returns:
+        fig: figure and axis
+        ax: axis
+    """
+    if 'X_tsne' not in adata.obsm.keys():
+        raise ValueError('X_tsne not found in adata.obsm. Please run ov.pp.tsne first.')
+    return embedding(adata, basis='X_tsne', **kwargs)
+
+def pca(adata: AnnData, **kwargs):
+    r"""
+    Plot PCA embedding.
+
+    Arguments:
+        adata: Annotated data matrix.
+        **kwargs: Additional keyword arguments.
+    """
+    if 'X_pca' not in adata.obsm.keys() or 'scaled|original|X_pca' not in adata.obsm.keys():
+        raise ValueError('X_pca not found in adata.obsm. Please run ov.pp.pca first.')
+    if 'scaled|original|X_pca' in adata.obsm.keys():
+        adata.obsm['X_pca'] = adata.obsm['scaled|original|X_pca']
+
+    return embedding(adata, basis='X_pca', **kwargs)
+
+def umap(adata: AnnData, **kwargs):
+    r"""
+    Plot UMAP embedding.
+
+    Arguments:
+        adata: Annotated data matrix.
+        color: Keys for annotations of observations/cells or variables/genes. (None)
+        gene_symbols: Key for field in `.var` that stores gene symbols. (None)
+        use_raw: Use `.raw` attribute of `adata` if present. (None)
+        sort_order: For continuous annotations used as color parameter, plot data points with higher values on top of others. (True)
+        edges: Show edges between cells. (False)
+        edges_width: Width of edges. (0.1)
+        edges_color: Color of edges. ('grey')
+        neighbors_key: Key to use for neighbors. (None)
+        arrows: Show arrows for velocity. (False)
+        arrows_kwds: Keyword arguments for arrow plots. (None)
+        groups: Groups to highlight. (None)
+        components: Components to plot. (None)
+        dimensions: Dimensions to plot. (None)
+        layer: Name of the layer to use for coloring. (None)
+        projection: Type of projection ('2d' or '3d'). ('2d')
+        scale_factor: Scaling factor for sizes. (None)
+        color_map: Colormap to use for continuous variables. (None)
+        cmap: Colormap to use for continuous variables. (None)
+        palette: Colors to use for categorical variables. (None)
+        na_color: Color to use for NaN values. ('lightgray')
+        na_in_legend: Include NaN values in legend. (True)
+        size: Size of the dots. (None)
+        frameon: Draw a frame around the plot. ('small')
+        legend_fontsize: Font size for legend. (None)
+        legend_fontweight: Font weight for legend. ('bold')
+        legend_loc: Location of legend. ('right margin')
+        legend_fontoutline: Outline width for legend text. (None)
+        colorbar_loc: Location of colorbar. ('right')
+        vmax: Maximum value for colorbar. (None)
+        vmin: Minimum value for colorbar. (None)
+        vcenter: Center value for colorbar. (None)
+        norm: Normalization for colorbar. (None)
+        add_outline: Add outline to points. (False)
+        outline_width: Width of outline. ((0.3, 0.05))
+        outline_color: Color of outline. (('black', 'white'))
+        ncols: Number of columns for subplots. (4)
+        hspace: Height spacing between subplots. (0.25)
+        wspace: Width spacing between subplots. (None)
+        title: Title for the plot. (None)
+        show: Show the plot. (None)
+        save: Save the plot. (None)
+        ax: Matplotlib axes object. (None)
+        return_fig: Return figure object. (None)
+        marker: Marker style. ('.')
+        **kwargs: Additional keyword arguments.
+
+    Returns:
+        fig: figure and axis
+        ax: axis
+    """
+    if 'X_umap' not in adata.obsm.keys():
+        raise ValueError('X_umap not found in adata.obsm. Please run ov.pp.umap first.')
+    return embedding(adata, basis='X_umap', **kwargs)
+
+@register_function(
+    aliases=["细胞嵌入图", "embedding", "scatter_plot", "降维可视化", "嵌入图"],
+    category="pl",
+    description="Scatter plot for user specified embedding basis (UMAP, t-SNE, PCA, etc)",
+    examples=[
+        "# Basic UMAP plot",
+        "ov.pl.embedding(adata, basis='X_umap', color='cell_type')",
+        "# Multiple genes visualization",
+        "ov.pl.embedding(adata, basis='X_umap', color=['CD3D', 'CD8A', 'CD4'])",
+        "# Customized embedding plot",
+        "ov.pl.embedding(adata, basis='X_tsne', color='leiden',",
+        "                palette=ov.pl.red_color, frameon='small')",
+        "# 3D embedding",
+        "ov.pl.embedding(adata, basis='X_umap', color='cell_type',",
+        "                projection='3d', ncols=2)"
+    ],
+    related=["pp.umap", "pp.tsne", "pp.pca", "pl.umap", "pl.tsne", "pl.pca"]
+)
 def embedding(
     adata: AnnData,
     basis: str,
@@ -170,6 +296,22 @@ def embedding(
 
 
 
+@register_function(
+    aliases=["细胞比例图", "cellproportion", "cell_proportion", "细胞组成", "比例图"],
+    category="pl",
+    description="Plot cell proportion of each cell type in each visual cluster",
+    examples=[
+        "# Basic cell proportion plot",
+        "ov.pl.cellproportion(adata, celltype_clusters='cell_type', groupby='leiden')",
+        "# Horizontal bar plot",
+        "ov.pl.cellproportion(adata, celltype_clusters='cell_type', groupby='leiden',",
+        "                     transpose=True, figsize=(6,4))",
+        "# Custom group order",
+        "ov.pl.cellproportion(adata, celltype_clusters='cell_type', groupby='leiden',",
+        "                     groupby_li=['0', '1', '2'], legend=True)"
+    ],
+    related=["pl.embedding", "tl.leiden"]
+)
 def cellproportion(adata:AnnData,celltype_clusters:str,groupby:str,
                        groupby_li=None,figsize:tuple=(4,6),
                        ticks_fontsize:int=12,labels_fontsize:int=12,ax=None,
@@ -278,6 +420,22 @@ def cellproportion(adata:AnnData,celltype_clusters:str,groupby:str,
     
 
 
+@register_function(
+    aliases=["细胞类型嵌入图", "embedding_celltype", "celltype_embedding", "嵌入细胞类型", "细胞类型可视化"],
+    category="pl",
+    description="Plot embedding with celltype color visualization",
+    examples=[
+        "# Basic celltype embedding plot",
+        "ov.pl.embedding_celltype(adata, celltype_key='cell_type')",
+        "# Custom parameters",
+        "ov.pl.embedding_celltype(adata, basis='tsne', figsize=(8,6),",
+        "                         celltype_key='leiden', title='Cell Types')",
+        "# Adjust range parameters",
+        "ov.pl.embedding_celltype(adata, celltype_range=(1,8),",
+        "                         embedding_range=(2,12))"
+    ],
+    related=["pl.embedding", "pl.cellproportion"]
+)
 def embedding_celltype(adata:AnnData,figsize:tuple=(6,4),basis:str='umap',
                             celltype_key:str='major_celltype',title:str=None,
                             celltype_range:tuple=(2,9),
@@ -309,12 +467,9 @@ def embedding_celltype(adata:AnnData,figsize:tuple=(6,4),basis:str='umap',
     """
 
     adata.obs[celltype_key]=adata.obs[celltype_key].astype('category')
-    if pd.__version__>="2.0.0":
-        cell_num_pd=pd.DataFrame(adata.obs[celltype_key].value_counts())
-        cell_num_pd[celltype_key]=cell_num_pd['count']
-    else:
-        cell_num_pd=pd.DataFrame(adata.obs[celltype_key].value_counts())
-        
+    cell_counts=adata.obs[celltype_key].value_counts()
+    cell_num_pd=pd.DataFrame({celltype_key: cell_counts.values}, index=cell_counts.index)
+    
     if '{}_colors'.format(celltype_key) in adata.uns.keys():
         cell_color_dict=dict(zip(adata.obs[celltype_key].cat.categories.tolist(),
                         adata.uns['{}_colors'.format(celltype_key)]))
@@ -383,6 +538,24 @@ def embedding_celltype(adata:AnnData,figsize:tuple=(6,4),basis:str='umap',
     return fig,[ax1,ax2]
 
 
+@register_function(
+    aliases=["凸包", "ConvexHull", "convex_hull", "凸包图", "轮廓图"],
+    category="pl",
+    description="Plot the ConvexHull for a cluster in embedding visualization",
+    examples=[
+        "# Basic ConvexHull plot",
+        "import matplotlib.pyplot as plt",
+        "fig, ax = plt.subplots()",
+        "ov.pl.embedding(adata, basis='X_umap', color='leiden', ax=ax, show=False)",
+        "ov.pl.ConvexHull(adata, basis='X_umap', cluster_key='leiden',",
+        "                 hull_cluster='0', ax=ax)",
+        "# Multiple clusters",
+        "for cluster in ['0', '1', '2']:",
+        "    ov.pl.ConvexHull(adata, basis='X_umap', cluster_key='leiden',",
+        "                     hull_cluster=cluster, ax=ax, alpha=0.3)"
+    ],
+    related=["pl.embedding", "pl.contour"]
+)
 def ConvexHull(adata:AnnData,basis:str,cluster_key:str,
                     hull_cluster:str,ax,color=None,alpha:float=0.2):
     r"""
@@ -431,6 +604,22 @@ def ConvexHull(adata:AnnData,basis:str,cluster_key:str,
     return ax
 
 
+@register_function(
+    aliases=["嵌入标签调整", "embedding_adjust", "adjust_labels", "标签调整", "嵌入图调整"],
+    category="pl",
+    description="Get locations of cluster median and adjust text labels accordingly on embedding",
+    examples=[
+        "# Basic label adjustment",
+        "ov.pl.embedding_adjust(adata, groupby='leiden')",
+        "# Exclude specific clusters",
+        "ov.pl.embedding_adjust(adata, groupby='cell_type', exclude=('Unknown',))",
+        "# Custom parameters",
+        "ov.pl.embedding_adjust(adata, groupby='leiden', basis='X_tsne',",
+        "                       adjust_kwargs={'avoid_text': True},",
+        "                       text_kwargs={'fontsize': 12, 'weight': 'bold'})"
+    ],
+    related=["pl.embedding", "pl.embedding_celltype"]
+)
 def embedding_adjust(
     adata, groupby, exclude=(), 
     basis='X_umap',ax=None, adjust_kwargs=None, text_kwargs=None
@@ -479,8 +668,26 @@ def embedding_adjust(
         texts = [ax.text(x=x, y=y, s=k, **text_kwargs) for k, (x, y) in medians.items()]
     from adjustText import adjust_text
     adjust_text(texts, **adjust_kwargs)
+    return texts
 
 
+@register_function(
+    aliases=["嵌入密度图", "embedding_density", "density_embedding", "密度嵌入图", "嵌入密度可视化"],
+    category="pl",
+    description="Create density visualization for specific clusters on embedding",
+    examples=[
+        "# Basic embedding density plot",
+        "ov.pl.embedding_density(adata, basis='X_umap', groupby='leiden',",
+        "                        target_clusters=['0', '1'])",
+        "# Multiple clusters",
+        "ov.pl.embedding_density(adata, basis='X_tsne', groupby='cell_type',",
+        "                        target_clusters=['T_cell', 'B_cell'])",
+        "# Custom visualization parameters",
+        "ov.pl.embedding_density(adata, basis='X_umap', groupby='leiden',",
+        "                        target_clusters=['2'], color='viridis')"
+    ],
+    related=["pl.embedding", "pl.calculate_gene_density", "pl.add_density_contour"]
+)
 def embedding_density(adata,basis,groupby,target_clusters,**kwargs):
     if 'X_' in basis:
         basis1=basis.split('_')[1]
@@ -496,6 +703,22 @@ def embedding_density(adata,basis,groupby,target_clusters,**kwargs):
                    **kwargs
                  )
 
+@register_function(
+    aliases=["柱点图", "bardotplot", "bar_dot_plot", "柱状点图", "条形点图"],
+    category="pl",
+    description="Create combined bar plot and dot plot for categorical data visualization",
+    examples=[
+        "# Basic bar dot plot",
+        "ov.pl.bardotplot(adata, groupby='cell_type', color='CD3D')",
+        "# Custom styling",
+        "ov.pl.bardotplot(adata, groupby='leiden', color='marker_gene',",
+        "                 figsize=(10,4), fontsize=14)",
+        "# Return values for further processing",
+        "values = ov.pl.bardotplot(adata, groupby='cluster', color='gene',",
+        "                          return_values=True)"
+    ],
+    related=["pl.violin", "pl.dotplot", "pl.boxplot"]
+)
 def bardotplot(adata,groupby,color,figsize=(8,3),return_values=False,
                fontsize=12,xlabel='',ylabel='',xticks_rotation=90,ax=None,
                bar_kwargs=None,scatter_kwargs=None):
@@ -599,6 +822,23 @@ import matplotlib.pyplot as plt
 import matplotlib.patches as mpatches
 from scipy.stats import kruskal
 
+@register_function(
+    aliases=["单组箱线图", "single_group_boxplot", "single_boxplot", "单个组箱图", "单组箱形图"],
+    category="pl",
+    description="Create boxplot for single group comparison with statistical analysis",
+    examples=[
+        "# Basic single group boxplot",
+        "ov.pl.single_group_boxplot(adata, groupby='cell_type', color='CD3D')",
+        "# Custom styling with color dictionary",
+        "colors = {'T_cell': 'red', 'B_cell': 'blue', 'NK_cell': 'green'}",
+        "ov.pl.single_group_boxplot(adata, groupby='cell_type', color='marker_gene',",
+        "                           type_color_dict=colors, title='Expression Comparison')",
+        "# Multiple comparisons",
+        "ov.pl.single_group_boxplot(adata, groupby='leiden', color='gene_expression',",
+        "                           ylabel='Expression Level')"
+    ],
+    related=["pl.boxplot", "pl.violin", "pl.bardotplot"]
+)
 def single_group_boxplot(adata,
                          groupby: str = '',
                          color: str = '',
@@ -781,6 +1021,25 @@ def single_group_boxplot(adata,
     if ax is None:
         return fig, ax
 
+@register_function(
+    aliases=["等高线图", "contour", "contour_plot", "轮廓线", "密度等高线"],
+    category="pl",
+    description="Add density contour lines for specific clusters on embedding plot",
+    examples=[
+        "# Basic contour plot",
+        "import matplotlib.pyplot as plt",
+        "fig, ax = plt.subplots()",
+        "ov.pl.embedding(adata, basis='X_umap', color='leiden', ax=ax, show=False)",
+        "ov.pl.contour(ax, adata, groupby='leiden', clusters=['0', '1'])",
+        "# Custom parameters",
+        "ov.pl.contour(ax, adata, groupby='cell_type', clusters=['T_cell'],",
+        "              basis='X_tsne', grid_density=150, contour_threshold=0.05)",
+        "# Multiple cluster contours",
+        "for cluster in ['0', '1', '2']:",
+        "    ov.pl.contour(ax, adata, groupby='leiden', clusters=[cluster])"
+    ],
+    related=["pl.embedding", "pl.ConvexHull", "pl.add_density_contour"]
+)
 def contour(ax,adata,groupby,clusters,basis='X_umap',
             grid_density=100,contour_threshold=0.1,
            **kwargs):
@@ -1408,4 +1667,50 @@ def dotplot_doublegroup(adata, gene, group1, group2, cmap='Reds',
     ax.grid(False)
     ax1.grid(False)
     ax1.axis(False)
-    plt.show()
\ No newline at end of file
+    plt.show()
+
+
+def add_arrow(ax,adata,basis,fontsize=12,
+              x_label=None,y_label=None,
+              arrow_scale=5,arrow_width=0.01):
+    r"""
+    Add arrow and label to the axis
+    Parameters
+    ----------
+    ax : matplotlib.axes.Axes
+        The axis to add the arrow and label to.
+    adata : AnnData
+        The AnnData object to add the arrow and label to.
+    basis : str
+        The basis to add the arrow and label to.
+    fontsize : int
+        The fontsize of the label.
+    arrow_scale : float
+        The scale of the arrow.
+    arrow_width : float
+        The width of the arrow.
+    x_label : str
+        The label of the x-axis.
+    y_label : str
+        The label of the y-axis.
+    """
+    if x_label is None:
+        x_label=basis+'1'
+    if y_label is None:
+        y_label=basis+'2'
+    
+    x_range=(adata.obsm[basis][:,0].max()-adata.obsm[basis][:,0].min())/6
+    y_range=(adata.obsm[basis][:,1].max()-adata.obsm[basis][:,1].min())/6
+    x_min=adata.obsm[basis][:,0].min()
+    y_min=adata.obsm[basis][:,1].min()
+    ax.arrow(x=x_min-x_range/5,y=y_min,dx=x_range+x_range/arrow_scale,dy=0, 
+            width=arrow_width, color="k", 
+                head_width=y_range*2/arrow_scale, head_length=x_range*2/arrow_scale, overhang=0.5)
+
+    ax.arrow(x=x_min,y=y_min-y_range/5,dx=0,dy=y_range+y_range/arrow_scale, 
+            width=arrow_width, color="k", 
+                head_width=x_range*2/arrow_scale, head_length=y_range*2/arrow_scale, overhang=0.5)
+    ax.text(x=x_min,y=y_min-y_range/2,s=x_label,fontsize=fontsize,multialignment='center',
+            verticalalignment='center')
+    ax.text(x=x_min-x_range/2,y=y_min,s=y_label,fontsize=fontsize,rotation='vertical',multialignment='center',
+           horizontalalignment='center')
\ No newline at end of file
