@@ -2,9 +2,10 @@ import numpy as np
 import matplotlib.pyplot as plt
 import pandas as pd
 import scipy.stats as st
-from typing import Sequence, Union, Optional, Literal
+from typing import Sequence, Union, Optional, Literal, List
 from matplotlib.axes import Axes
 from collections import OrderedDict
+from ..utils.registry import register_function
 
 try:
     from anndata import AnnData
@@ -33,6 +34,25 @@ except ImportError:
 # Type definitions to match scanpy
 DensityNorm = Literal["area", "count", "width"]
 
+@register_function(
+    aliases=["小提琴图", "violin", "violin_plot", "小提琴", "表达分布图"],
+    category="pl",
+    description="Enhanced violin plot for visualizing gene expression distribution across groups",
+    examples=[
+        "# Basic violin plot",
+        "ov.pl.violin(adata, keys=['CD3D', 'CD8A'], groupby='cell_type')",
+        "# Enhanced violin plot with statistics",
+        "ov.pl.violin(adata, keys=['GAPDH'], groupby='leiden',",
+        "             stripplot=True, statistical_tests=True)",
+        "# Multiple genes",
+        "ov.pl.violin(adata, keys=['CD3D', 'CD4', 'CD8A'], groupby='cell_type',",
+        "             custom_colors=['red', 'blue', 'green'], figsize=(8,6))",
+        "# With boxplot overlay",
+        "ov.pl.violin(adata, keys=['marker_gene'], groupby='cluster',",
+        "             show_boxplot=True, show_means=True)"
+    ],
+    related=["pl.embedding", "pl.dotplot"]
+)
 def violin(
     adata: AnnData,
     keys: Union[str, Sequence[str]],
@@ -63,7 +83,7 @@ def violin(
     background_color: str = 'white',
     spine_color: str = '#b4aea9',
     grid_lines: bool = True,
-    statistical_tests: bool = False,
+    statistical_tests: Union[bool, str, List[str]] = False,
     custom_colors: Optional[Sequence[str]] = None,
     figsize: Optional[tuple] = None,
     fontsize=13,
@@ -105,7 +125,7 @@ def violin(
         background_color: str. Background color of the plot. ('white')
         spine_color: str. Color of plot spines. ('#b4aea9')
         grid_lines: bool. Whether to show horizontal grid lines. (True)
-        statistical_tests: bool. Whether to perform and display statistical tests. (False)
+        statistical_tests: bool | str | List[str]. Statistical tests to perform. Options: False (no tests), True (auto-select), 'wilcox', 'ttest', 'anova', 'kruskal', 'mannwhitney', or list of methods. (False)
         custom_colors: Sequence[str] | None. Custom colors for groups. (None)
         figsize: tuple | None. Figure size (width, height). (None)
         fontsize: int. Font size for labels and ticks. (13)
@@ -332,26 +352,37 @@ def violin(
         )
     
     # Create single or multiple axis plots
-    if ax is None and figsize is not None:
-        fig, ax = plt.subplots(figsize=figsize)
-    elif ax is None:
-        fig, ax = plt.subplots(figsize=(10, 6))
-    
-    # Apply enhanced styling
-    if enhanced_style:
-        _apply_enhanced_styling(ax, background_color, spine_color, grid_lines)
+    if len(y_cols) > 1:
+        # Multiple keys: create subplots directly
+        if ax is not None:
+            print(f"{Colors.WARNING}Warning: ax parameter ignored when plotting multiple keys{Colors.ENDC}")
+        
+        # Use user-provided figsize or calculate appropriate size
+        if figsize is not None:
+            fig, axes = plt.subplots(1, len(y_cols), figsize=figsize)
+        else:
+            # Default: width scales with number of keys
+            fig, axes = plt.subplots(1, len(y_cols), figsize=(5*len(y_cols), 6))
+        
+        # Ensure axes is always a list
+        if len(y_cols) == 1:
+            axes = [axes]
+    else:
+        # Single key: use provided ax or create new one
+        if ax is None:
+            if figsize is not None:
+                fig, ax = plt.subplots(figsize=figsize)
+            else:
+                fig, ax = plt.subplots(figsize=(10, 6))
+        axes = [ax]
     
     # Create plots for each y variable
     for i, (y_col, y_label) in enumerate(zip(y_cols, ylabel)):
-        if len(y_cols) > 1:
-            # Create subplots for multiple keys
-            if i == 0:
-                fig, axes = plt.subplots(1, len(y_cols), figsize=figsize or (5*len(y_cols), 6))
-                if len(y_cols) == 1:
-                    axes = [axes]
-            current_ax = axes[i] if len(y_cols) > 1 else ax
-        else:
-            current_ax = ax
+        current_ax = axes[i]
+        
+        # Apply enhanced styling to each subplot
+        if enhanced_style:
+            _apply_enhanced_styling(current_ax, background_color, spine_color, grid_lines)
         
         # Prepare data for current y variable
         plot_data = _prepare_plot_data(obs_tidy, x_col, y_col, group_categories, order)
@@ -379,32 +410,36 @@ def violin(
         
         # Add statistical tests
         if statistical_tests:
-            _add_statistical_tests(current_ax, plot_data, group_categories)
+            _add_statistical_tests(current_ax, plot_data, group_categories, statistical_tests)
         
         # Customize axis
         _customize_axis(
             current_ax, group_categories, xlabel, y_label, groupby, 
             rotation, log, order
         )
+        
+        # Apply consistent styling to all subplots
         current_ax.spines['top'].set_visible(False)
         current_ax.spines['right'].set_visible(False)
         current_ax.spines['bottom'].set_visible(True)
         current_ax.spines['left'].set_visible(True)
         current_ax.spines['left'].set_position(('outward', 10))
         current_ax.spines['bottom'].set_position(('outward', 10))
-        if ticks_fontsize==None:
-            ticks_fontsize=fontsize-1
-    
-        current_ax.set_xticklabels(current_ax.get_xticklabels(),fontsize=ticks_fontsize,rotation=rotation)
-        current_ax.set_yticklabels(current_ax.get_yticklabels(),fontsize=ticks_fontsize)
-        current_ax.set_xlabel(groupby,fontsize=fontsize)
-        current_ax.set_ylabel(y_label,fontsize=fontsize)
-        #print(y_label)
-    
-    #plt.tight_layout()
+        
+        # Set font sizes
+        if ticks_fontsize is None:
+            ticks_fontsize = fontsize - 1
     
+        current_ax.set_xticklabels(current_ax.get_xticklabels(), fontsize=ticks_fontsize, rotation=rotation)
+        current_ax.set_yticklabels(current_ax.get_yticklabels(), fontsize=ticks_fontsize)
+        current_ax.set_xlabel(groupby, fontsize=fontsize)
+        current_ax.set_ylabel(y_label, fontsize=fontsize)
         current_ax.grid(False)
     
+    # Apply tight layout for better spacing
+    if len(y_cols) > 1:
+        plt.tight_layout()
+    
 
     
     # Save figure if requested
@@ -417,12 +452,16 @@ def violin(
         plt.show()
         return None
     
-    return ax
+    # Return appropriate object based on number of subplots
+    if len(y_cols) > 1:
+        return axes
+    else:
+        return axes[0]
 
 def _extract_data_from_adata(adata, keys, groupby, layer, use_raw):
     r"""
     Extract data from AnnData object for violin plotting.
-    
+
     Arguments:
         adata: AnnData object
         keys: list of str
@@ -431,24 +470,26 @@ def _extract_data_from_adata(adata, keys, groupby, layer, use_raw):
             Grouping variable
         layer: str, optional
             Layer to use for gene expression data
-        use_raw: bool
-            Whether to use raw data
-    
+        use_raw: bool, optional
+            Whether to use raw data. Defaults to True if `.raw` is present.
+
     Returns:
         dict: Dictionary containing extracted data
     """
-    # This is a simplified version - in real scanpy, this would handle
-    # raw data, layers, etc. more comprehensively
-    
+    # Default behavior: use raw if it exists and use_raw is not explicitly False
+    # This matches scanpy's behavior
+    if use_raw is None:
+        use_raw = hasattr(adata, 'raw') and adata.raw is not None
+
     data_dict = {}
-    
+
     # Handle groupby column
     if groupby is not None:
         if groupby in adata.obs.columns:
             data_dict[groupby] = adata.obs[groupby]
         else:
             raise ValueError(f"groupby '{groupby}' not found in adata.obs")
-    
+
     # Handle keys (variables)
     for key in keys:
         if key in adata.obs.columns:
@@ -457,21 +498,27 @@ def _extract_data_from_adata(adata, keys, groupby, layer, use_raw):
         elif key in adata.var_names:
             # Key is a gene/variable
             if use_raw and hasattr(adata, 'raw') and adata.raw is not None:
-                gene_idx = list(adata.raw.var_names).index(key)
-                data_dict[key] = adata.raw.X[:, gene_idx]
+                # Check if the key exists in raw.var_names
+                if key in adata.raw.var_names:
+                    gene_idx = list(adata.raw.var_names).index(key)
+                    data_dict[key] = adata.raw.X[:, gene_idx]
+                else:
+                    # Fall back to adata.X if key not in raw
+                    gene_idx = list(adata.var_names).index(key)
+                    data_dict[key] = adata.X[:, gene_idx]
             elif layer is not None and layer in adata.layers:
                 gene_idx = list(adata.var_names).index(key)
                 data_dict[key] = adata.layers[layer][:, gene_idx]
             else:
                 gene_idx = list(adata.var_names).index(key)
                 data_dict[key] = adata.X[:, gene_idx]
-                
+
             # Handle sparse matrices
             if hasattr(data_dict[key], 'toarray'):
                 data_dict[key] = data_dict[key].toarray().flatten()
         else:
             raise ValueError(f"Key '{key}' not found in adata.obs or adata.var_names")
-    
+
     return pd.DataFrame(data_dict)
 
 def _setup_colors(custom_colors, group_categories, adata, groupby):
@@ -619,34 +666,102 @@ def _add_mean_annotations(ax, plot_data, group_categories):
             ha='left'
         )
 
-def _add_statistical_tests(ax, plot_data, group_categories):
+def _add_statistical_tests(ax, plot_data, group_categories, statistical_tests):
     """Add statistical test results."""
     if len(group_categories) < 2:
         return
     
     try:
-        from scipy.stats import f_oneway, ttest_ind
+        from scipy.stats import (
+            f_oneway, ttest_ind, mannwhitneyu, kruskal, 
+            ranksums, wilcoxon
+        )
         
         data_arrays = [plot_data[cat] for cat in group_categories if len(plot_data[cat]) > 0]
         
-        if len(data_arrays) == 2:
-            # T-test for two groups
-            stat, p_val = ttest_ind(data_arrays[0], data_arrays[1])
-            test_text = f"t-test: p = {p_val:.2e}"
-        elif len(data_arrays) > 2:
-            # ANOVA for multiple groups
-            stat, p_val = f_oneway(*data_arrays)
-            test_text = f"ANOVA: F = {stat:.2f}, p = {p_val:.2e}"
+        if len(data_arrays) < 2:
+            return
+            
+        # Determine which tests to perform
+        if statistical_tests is True:
+            # Auto-select based on number of groups (backward compatibility)
+            if len(data_arrays) == 2:
+                tests_to_perform = ['ttest']
+            else:
+                tests_to_perform = ['anova']
+        elif isinstance(statistical_tests, str):
+            tests_to_perform = [statistical_tests]
+        elif isinstance(statistical_tests, list):
+            tests_to_perform = statistical_tests
         else:
             return
         
+        test_results = []
+        
+        for test_method in tests_to_perform:
+            test_method = test_method.lower()
+            
+            try:
+                if test_method in ['wilcox', 'wilcoxon'] and len(data_arrays) == 2:
+                    # Wilcoxon rank-sum test (Mann-Whitney U test)
+                    stat, p_val = mannwhitneyu(data_arrays[0], data_arrays[1], 
+                                              alternative='two-sided')
+                    test_results.append(f"Wilcox: p = {p_val:.2e}")
+                    
+                elif test_method == 'mannwhitney' and len(data_arrays) == 2:
+                    # Mann-Whitney U test (same as wilcox rank-sum)
+                    stat, p_val = mannwhitneyu(data_arrays[0], data_arrays[1], 
+                                              alternative='two-sided')
+                    test_results.append(f"Mann-Whitney: p = {p_val:.2e}")
+                    
+                elif test_method == 'ttest' and len(data_arrays) == 2:
+                    # T-test for two groups
+                    stat, p_val = ttest_ind(data_arrays[0], data_arrays[1])
+                    test_results.append(f"t-test: p = {p_val:.2e}")
+                    
+                elif test_method == 'anova' and len(data_arrays) >= 2:
+                    # ANOVA for multiple groups
+                    stat, p_val = f_oneway(*data_arrays)
+                    test_results.append(f"ANOVA: F = {stat:.2f}, p = {p_val:.2e}")
+                    
+                elif test_method == 'kruskal' and len(data_arrays) >= 2:
+                    # Kruskal-Wallis test (non-parametric alternative to ANOVA)
+                    stat, p_val = kruskal(*data_arrays)
+                    test_results.append(f"Kruskal-Wallis: H = {stat:.2f}, p = {p_val:.2e}")
+                    
+                elif test_method in ['wilcox', 'wilcoxon', 'mannwhitney'] and len(data_arrays) > 2:
+                    # Perform pairwise tests for multiple groups
+                    from itertools import combinations
+                    pairwise_results = []
+                    for i, (cat1, cat2) in enumerate(combinations(group_categories, 2)):
+                        if cat1 in plot_data and cat2 in plot_data:
+                            if len(plot_data[cat1]) > 0 and len(plot_data[cat2]) > 0:
+                                stat, p_val = mannwhitneyu(plot_data[cat1], plot_data[cat2], 
+                                                          alternative='two-sided')
+                                pairwise_results.append(f"{cat1}-{cat2}: p = {p_val:.2e}")
+                                # Only show first 3 pairwise comparisons to avoid clutter
+                                if i >= 2:
+                                    pairwise_results.append("...")
+                                    break
+                    if pairwise_results:
+                        test_results.append(f"Pairwise Wilcox: {'; '.join(pairwise_results)}")
+                        
+                else:
+                    print(f"Warning: Test method '{test_method}' not supported or not appropriate for {len(data_arrays)} groups")
+                    
+            except Exception as e:
+                print(f"Warning: Failed to perform {test_method} test: {str(e)}")
+        
         # Add test result text
-        ax.text(0.02, 0.98, test_text, transform=ax.transAxes, 
-               fontsize=10, verticalalignment='top',
-               bbox=dict(boxstyle="round", facecolor="white", alpha=0.8))
+        if test_results:
+            test_text = "\n".join(test_results)
+            ax.text(0.02, 0.98, test_text, transform=ax.transAxes, 
+                   fontsize=9, verticalalignment='top',
+                   bbox=dict(boxstyle="round", facecolor="white", alpha=0.9),
+                   family='monospace')
                
-    except ImportError:
-        print("Statistical tests require scipy")
+    except ImportError as e:
+        print(f"Statistical tests require scipy: {str(e)}")
 
 def _customize_axis(ax, group_categories, xlabel, ylabel, groupby, rotation, log, order):
     """Customize axis labels and ticks."""
