@@ -12,11 +12,40 @@ import scanpy as sc
 import time 
 
 from scipy.sparse import issparse, csr_matrix
+
+from ._qc import _is_rust_backend
 from ..utils import load_signatures_from_file,predefined_signatures
-from .._settings import settings,print_gpu_usage_color,EMOJI,add_reference
+from ..utils.registry import register_function
+from .._settings import settings,print_gpu_usage_color,EMOJI,Colors,add_reference
+
+
+from ._normalization import normalize_total,log1p
 from datetime import datetime
 
 
+# Helper functions for Rust anndata compatibility
+def _safe_copy(arr):
+    """Safely copy array data, compatible with both numpy and Rust backends."""
+    try:
+        # Try the standard copy method first
+        return arr.copy()
+    except AttributeError:
+        # For Rust backends that don't have copy method
+        return np.array(arr, copy=True)
+
+def _safe_to_df_copy(arr):
+    """Safely convert to DataFrame and copy, compatible with both numpy and Rust backends."""
+    try:
+        # Try the standard to_df().copy() first
+        return arr.to_df().copy()
+    except AttributeError:
+        # For Rust backends, convert to numpy first, then to DataFrame
+        if hasattr(arr, 'to_numpy'):
+            return pd.DataFrame(arr.to_numpy()).copy()
+        else:
+            return pd.DataFrame(np.array(arr)).copy()
+
+
 
 # Emoji map for UMAP status reporting
 
@@ -39,25 +68,33 @@ def identify_robust_genes(data: anndata.AnnData, percent_cells: float = 0.05) ->
 
     prior_n = data.shape[1]
 
+    from ._qc import _is_rust_backend
+    is_rust = _is_rust_backend(data)
+
     if issparse(data.X):
         data.var["n_cells"] = data.X.getnnz(axis=0)
         data._inplace_subset_var(data.var["n_cells"] > 0)
         data.var["percent_cells"] = (data.var["n_cells"] / data.shape[0]) * 100
         data.var["robust"] = data.var["percent_cells"] >= percent_cells
+    elif is_rust:
+        data.var["n_cells"] =data.X[:].getnnz(axis=0)
+        data.subset(var_indices=np.where(data.var["n_cells"]>0)[0])
+        data.var["percent_cells"] = (data.var["n_cells"] / data.shape[0]) * 100
+        data.var["robust"] = data.var["percent_cells"] >= percent_cells
     else:
         data.var["robust"] = True
 
     data.var["highly_variable_features"] = data.var["robust"]  
     # default all robust genes are "highly" variable
-    print(f"After filtration, {data.shape[1]}/{prior_n} genes are kept. \
-    Among {data.shape[1]} genes, {data.var['robust'].sum()} genes are robust.")
+    print(f"{Colors.BLUE}    After filtration, {data.shape[1]}/{prior_n} genes are kept.{Colors.ENDC}")
+    print(f"{Colors.BLUE}    Among {data.shape[1]} genes, {data.var['robust'].sum()} genes are robust.{Colors.ENDC}")
 
 def calc_mean_and_var(X: Union[csr_matrix, np.ndarray], axis: int) -> Tuple[np.ndarray, np.ndarray]:
     if issparse(X):
-        from ..cylib.fast_utils import calc_mean_and_var_sparse
+        #from ..cylib.fast_utils import calc_mean_and_var_sparse
         return calc_mean_and_var_sparse(X.shape[0], X.shape[1], X.data, X.indices, X.indptr, axis)
     else:
-        from ..cylib.fast_utils import calc_mean_and_var_dense
+        #from ..cylib.fast_utils import calc_mean_and_var_dense
         return calc_mean_and_var_dense(X.shape[0], X.shape[1], X, axis)
 def calc_stat_per_batch(X: Union[csr_matrix, np.ndarray], batch: \
     Union[pd.Categorical, np.ndarray, list]) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
@@ -70,10 +107,10 @@ def calc_stat_per_batch(X: Union[csr_matrix, np.ndarray], batch: \
         nbatch = codes.max() + 1 # assume cluster label starts from 0
 
     if issparse(X):
-        from ..cylib.fast_utils import calc_stat_per_batch_sparse
+        #from ..cylib.fast_utils import calc_stat_per_batch_sparse
         return calc_stat_per_batch_sparse(X.shape[0], X.shape[1], X.data, X.indices, X.indptr, nbatch, codes)
     else:
-        from ..cylib.fast_utils import calc_stat_per_batch_dense
+        #from ..cylib.fast_utils import calc_stat_per_batch_dense
         return calc_stat_per_batch_dense(X.shape[0], X.shape[1], X, nbatch, codes)
 
 def estimate_feature_statistics(data: anndata.AnnData, batch: str) -> None:
@@ -333,6 +370,12 @@ def corr2_coeff(a, b):
     """
     resol = 10 ** (-15)
 
+    # Convert sparse matrices to dense arrays if necessary
+    if issparse(a):
+        a = a.toarray()
+    if issparse(b):
+        b = b.toarray()
+
     a_ma = a - a.mean(1)[:, None]
     b_mb = b - b.mean(1)[:, None]
     ssa = (a_ma ** 2).sum(1)
@@ -358,9 +401,9 @@ def remove_cc_genes(adata:anndata.AnnData, organism:str='human', corr_threshold:
     cc_genes = list(set(cycling_genes['G1/S']) | set(cycling_genes['G2/M']))
     cc_genes = [ x for x in cc_genes if x in adata.var_names ]
     # Compute corr
-    cc_expression = adata[:, cc_genes].X.A.T
+    cc_expression = adata[:, cc_genes].X.toarray().T if issparse(adata[:, cc_genes].X) else adata[:, cc_genes].X.T
     hvgs = adata.var_names[adata.var['highly_variable_features']]
-    hvgs_expression = adata[:, hvgs].X.A.T
+    hvgs_expression = adata[:, hvgs].X.toarray().T if issparse(adata[:, hvgs].X) else adata[:, hvgs].X.T
     cc_corr = corr2_coeff(hvgs_expression, cc_expression)
 
     # Discard genes having the maximum correlation with one of the cc > corr_threshold
@@ -375,24 +418,101 @@ def remove_cc_genes(adata:anndata.AnnData, organism:str='human', corr_threshold:
 
 from sklearn.cluster import KMeans  
 
+@register_function(
+    aliases=["数据转GPU", "anndata_to_GPU", "to_gpu", "GPU转换", "move_to_gpu"],
+    category="preprocessing",
+    description="Migrate AnnData objects to GPU memory for accelerated processing with RAPIDS",
+    examples=[
+        "# Move data to GPU for processing",
+        "ov.pp.anndata_to_GPU(adata)",
+        "# After analysis, move back to CPU",
+        "ov.pp.anndata_to_CPU(adata)",
+        "# Use with GPU preprocessing pipeline",
+        "ov.settings.gpu_init()",
+        "ov.pp.anndata_to_GPU(adata)",
+        "adata = ov.pp.qc(adata)  # GPU-accelerated QC"
+    ],
+    related=["pp.anndata_to_CPU", "settings.gpu_init", "pp.qc", "pp.preprocess"]
+)
 def anndata_to_GPU(adata,**kwargs):
-    '''
-    Migrate the data of AnnData objects to the GPU for processing
-    '''
+    """Migrate AnnData objects to GPU memory for accelerated processing.
+
+    Arguments:
+        adata: AnnData object containing single-cell data.
+        **kwargs: Additional arguments passed to rapids_singlecell.get.anndata_to_GPU.
+
+    Returns:
+        None: The function modifies adata in place by moving data to GPU memory.
+
+    Examples:
+        >>> import omicverse as ov
+        >>> # Initialize GPU mode
+        >>> ov.settings.gpu_init()
+        >>> # Move data to GPU
+        >>> ov.pp.anndata_to_GPU(adata)
+        >>> # Perform GPU-accelerated analysis
+        >>> adata = ov.pp.qc(adata)
+        >>> # Move back to CPU when done
+        >>> ov.pp.anndata_to_CPU(adata)
+    """
     import rapids_singlecell as rsc
     rsc.get.anndata_to_GPU(adata,**kwargs)
     print('Data has been moved to GPU')
     print('Don`t forget to move it back to CPU after analysis is done')
     print('Use `ov.pp.anndata_to_CPU(adata)`')
 
+@register_function(
+    aliases=["数据转CPU", "anndata_to_CPU", "to_cpu", "CPU转换", "move_to_cpu"],
+    category="preprocessing",
+    description="Migrate AnnData objects from GPU back to CPU memory after analysis",
+    examples=[
+        "# Move data back to CPU after GPU processing",
+        "ov.pp.anndata_to_CPU(adata)",
+        "# Convert specific layer only",
+        "ov.pp.anndata_to_CPU(adata, layer='scaled')",
+        "# Convert with copy",
+        "ov.pp.anndata_to_CPU(adata, convert_all=True, copy=True)",
+        "# Complete GPU-CPU workflow",
+        "ov.settings.gpu_init()",
+        "ov.pp.anndata_to_GPU(adata)",
+        "adata = ov.pp.qc(adata)  # GPU processing",
+        "ov.pp.anndata_to_CPU(adata)  # Back to CPU"
+    ],
+    related=["pp.anndata_to_GPU", "settings.gpu_init", "pp.qc", "pp.preprocess"]
+)
 def anndata_to_CPU(adata,layer=None, convert_all=True, copy=False):
-    '''
-    Migrate the data of AnnData objects to the CPU for processing
-    '''
+    """Migrate AnnData objects from GPU back to CPU memory after analysis.
+
+    Arguments:
+        adata: AnnData object containing single-cell data on GPU.
+        layer: Specific layer to convert back to CPU. Default: None (all layers).
+        convert_all: Whether to convert all arrays to CPU. Default: True.
+        copy: Whether to create a copy during conversion. Default: False.
+
+    Returns:
+        None: The function modifies adata in place by moving data from GPU to CPU memory.
+
+    Examples:
+        >>> import omicverse as ov
+        >>> # After GPU processing, move back to CPU
+        >>> ov.pp.anndata_to_CPU(adata)
+        >>> # Convert only specific layer
+        >>> ov.pp.anndata_to_CPU(adata, layer='scaled', convert_all=False)
+    """
     import rapids_singlecell as rsc
     rsc.get.anndata_to_CPU(adata,layer=layer, convert_all=convert_all, copy=copy)
 
 
+@register_function(
+    aliases=["预处理", "preprocess", "preprocessing", "数据预处理"],
+    category="preprocessing",
+    description="Complete preprocessing pipeline including normalization, HVG selection, scaling, and PCA",
+    examples=[
+        "ov.pp.preprocess(adata, mode='shiftlog|pearson', n_HVGs=2000)",
+        "ov.pp.preprocess(adata, mode='pearson|pearson', target_sum=50e4)"
+    ],
+    related=["qc", "normalize", "scale", "pca", "highly_variable_genes"]
+)
 def preprocess(adata, mode='shiftlog|pearson', target_sum=50*1e4, n_HVGs=2000,
     organism='human', no_cc=False,batch_key=None,):
     """
@@ -419,30 +539,43 @@ def preprocess(adata, mode='shiftlog|pearson', target_sum=50*1e4, n_HVGs=2000,
     """
 
     # Log-normalization, HVGs identification
-    adata.layers['counts'] = adata.X.copy()
-    print('Begin robust gene identification')
+    from ._qc import _is_rust_backend
+    is_rust = _is_rust_backend(adata)
+    if is_rust:
+        adata.layers['counts'] = adata.X[:]
+    else:
+        adata.layers['counts'] = adata.X.copy()
+    
+    
+    print(f"{EMOJI['start']} [{datetime.now().strftime('%Y-%m-%d %H:%M:%S')}] Running preprocessing in '{settings.mode}' mode...")
+    print(f"{Colors.CYAN}Begin robust gene identification{Colors.ENDC}")
     identify_robust_genes(adata, percent_cells=0.05)
-    adata = adata[:, adata.var['robust']]
-    print('End of robust gene identification.')
+    if not is_rust:
+        adata = adata[:, adata.var['robust']]
+    else:
+        adata.subset(var_indices=np.where(adata.var['robust']==True)[0])
+    print(f"{EMOJI['done']} Robust gene identification completed successfully.")
     method_list = mode.split('|')
-    print(f'Begin size normalization: {method_list[0]} and HVGs selection {method_list[1]}')
+    print(f"{Colors.CYAN}Begin size normalization: {method_list[0]} and HVGs selection {method_list[1]}{Colors.ENDC}")
     if settings.mode == 'cpu' or settings.mode == 'cpu-gpu-mixed':
         data_load_start = time.time()
         if method_list[0] == 'shiftlog': # Size normalization + scanpy batch aware HVGs selection
-            sc.pp.normalize_total(
+            normalize_total(
                 adata,
                 target_sum=target_sum,
                 exclude_highly_expressed=True,
                 max_fraction=0.2,
             )
-            sc.pp.log1p(adata)
+            log1p(adata)
             
         elif method_list[0] == 'pearson':
             # Perason residuals workflow
-            sc.experimental.pp.normalize_pearson_residuals(adata)
+            from .experimental import normalize_pearson_residuals
+            normalize_pearson_residuals(adata)
 
         if method_list[1] == 'pearson': # Size normalization + scanpy batch aware HVGs selection
-            sc.experimental.pp.highly_variable_genes(
+            from .experimental import highly_variable_genes
+            highly_variable_genes(
                 adata,
                 flavor="pearson_residuals",
                 layer='counts',
@@ -452,7 +585,7 @@ def preprocess(adata, mode='shiftlog|pearson', target_sum=50*1e4, n_HVGs=2000,
             if no_cc:
                 remove_cc_genes(adata, organism=organism, corr_threshold=0.1)
         elif method_list[1] == 'seurat':
-            sc.pp.highly_variable_genes(
+            highly_variable_genes(
                 adata,
                 flavor="seurat_v3",
                 layer='counts',
@@ -462,7 +595,7 @@ def preprocess(adata, mode='shiftlog|pearson', target_sum=50*1e4, n_HVGs=2000,
             if no_cc:
                 remove_cc_genes(adata, organism=organism, corr_threshold=0.1)
         data_load_end = time.time()
-        print(f'Time to analyze data in cpu: {data_load_end - data_load_start} seconds.')
+        print(f"{Colors.BLUE}    Time to analyze data in cpu: {data_load_end - data_load_start:.2f} seconds.{Colors.ENDC}")
     else:
         import rapids_singlecell as rsc
         data_load_start = time.time()
@@ -489,14 +622,25 @@ def preprocess(adata, mode='shiftlog|pearson', target_sum=50*1e4, n_HVGs=2000,
                 batch_key=batch_key,
             )
         data_load_end = time.time()
-        print(f'Time to analyze data in gpu: {data_load_end - data_load_start} seconds.')
+        print(f"{Colors.BLUE}    Time to analyze data in gpu: {data_load_end - data_load_start:.2f} seconds.{Colors.ENDC}")
 
-
-    adata.var = adata.var.drop(columns=['highly_variable_features'])
-    adata.var['highly_variable_features'] = adata.var['highly_variable']
-    adata.var = adata.var.drop(columns=['highly_variable'])
+    if not is_rust:
+        adata.var = adata.var.drop(columns=['highly_variable_features'])
+        adata.var['highly_variable_features'] = adata.var['highly_variable']
+        adata.var = adata.var.drop(columns=['highly_variable'])
+    else:
+        #adata.var = adata.var.drop(columns=['highly_variable_features'])
+        adata.var['highly_variable_features'] = adata.var['highly_variable']
+        #adata.var = adata.var.drop(columns=['highly_variable'])
     #adata.var = adata.var.rename(columns={'means':'mean', 'variances':'var'})
-    print(f'End of size normalization: {method_list[0]} and HVGs selection {method_list[1]}')
+    print(f"{EMOJI['done']} Preprocessing completed successfully.")
+    print(f"{Colors.GREEN}    Added:{Colors.ENDC}")
+    print(f"{Colors.CYAN}        'highly_variable_features', boolean vector (adata.var){Colors.ENDC}")
+    print(f"{Colors.CYAN}        'means', float vector (adata.var){Colors.ENDC}")
+    print(f"{Colors.CYAN}        'variances', float vector (adata.var){Colors.ENDC}")
+    print(f"{Colors.CYAN}        'residual_variances', float vector (adata.var){Colors.ENDC}")
+    print(f"{Colors.CYAN}        'counts', raw counts layer (adata.layers){Colors.ENDC}")
+    print(f"{Colors.BLUE}    End of size normalization: {method_list[0]} and HVGs selection {method_list[1]}{Colors.ENDC}")
 
     if 'status' not in adata.uns.keys():
         adata.uns['status'] = {}
@@ -525,7 +669,14 @@ def highly_variable_genes(adata,**kwargs):
         adata, **kwargs,
     )
 
-def scale(adata,max_value=10,layers_add='scaled'):
+@register_function(
+    aliases=["标准化", "scale", "scaling", "标准化处理"],
+    category="preprocessing",
+    description="Scale data to unit variance and zero mean",
+    examples=["ov.pp.scale(adata, max_value=10)"],
+    related=["normalize", "regress"]
+)
+def scale(adata,max_value=10,layers_add='scaled',**kwargs):
     """
     Scale the input AnnData object.
 
@@ -539,8 +690,12 @@ def scale(adata,max_value=10,layers_add='scaled'):
 
     """
     if settings.mode == 'cpu' or settings.mode == 'cpu-gpu-mixed':
-        adata_mock = sc.pp.scale(adata, copy=True,max_value=max_value)
-        adata.layers[layers_add] = adata_mock.X.copy()
+        from ._scale import scale_anndata as scale
+        adata_mock = scale(adata, copy=True,max_value=max_value,**kwargs)
+        if _is_rust_backend(adata_mock):
+            adata.layers[layers_add] = adata_mock.X[:]
+        else:
+            adata.layers[layers_add] = adata_mock.X.copy()
         del adata_mock
     else:
         import rapids_singlecell as rsc
@@ -632,6 +787,13 @@ class my_PCA:
 
         return self
 
+@register_function(
+    aliases=["主成分分析", "pca", "PCA", "降维"],
+    category="preprocessing",
+    description="Perform Principal Component Analysis for dimensionality reduction",
+    examples=["ov.pp.pca(adata, n_pcs=50)"],
+    related=["umap", "tsne", "mde"]
+)
 def pca(adata, n_pcs=50, layer='scaled',inplace=True,**kwargs):
     """
     Performs Principal Component Analysis (PCA) on the data stored in a scanpy AnnData object.
@@ -666,7 +828,9 @@ def pca(adata, n_pcs=50, layer='scaled',inplace=True,**kwargs):
             adata.uns[key + '|pca_var_ratios'] = adata_mock.uns['pca']['variance_ratio']
             adata.uns[key + '|cum_sum_eigenvalues'] = adata_mock.uns['pca']['variance']
         else:
-            sc.pp.pca(adata, layer=layer,n_comps=n_pcs,**kwargs)
+            from ._pca import pca as _pca
+            _pca(adata, layer=layer,n_comps=n_pcs,use_gpu=False,**kwargs)
+            #print(res)
             adata.obsm[key + '|X_pca'] = adata.obsm['X_pca']
             adata.varm[key + '|pca_loadings'] = adata.varm['PCs']
             adata.uns[key + '|pca_var_ratios'] = adata.uns['pca']['variance_ratio']
@@ -682,7 +846,7 @@ def pca(adata, n_pcs=50, layer='scaled',inplace=True,**kwargs):
         adata.uns[key + '|cum_sum_eigenvalues'] = adata.uns['pca']['variance']
     else:
         import rapids_singlecell as rsc
-        rsc.pp.pca(adata, layer=layer,n_comps=n_pcs)
+        rsc.pp.pca(adata, layer=layer,n_comps=n_pcs,**kwargs)
         adata.obsm[key + '|X_pca'] = adata.obsm['X_pca']
         adata.varm[key + '|pca_loadings'] = adata.varm['PCs']
         adata.uns[key + '|pca_var_ratios'] = adata.uns['pca']['variance_ratio']
@@ -731,14 +895,14 @@ def counts_store(adata,layers):
     '''
     counts store
     '''
-    adata.uns[layers] = adata.X.to_df().copy()
+    adata.uns[layers] = _safe_to_df_copy(adata.X)
 
 def counts_retrieve(adata,layers):
     '''
     counts retrieve
     '''
     cell_idx=adata.obs.index
-    adata.uns['raw_store'] = adata.X.to_df().copy()
+    adata.uns['raw_store'] = _safe_to_df_copy(adata.X)
     adata.X=adata.uns[layers].loc[cell_idx,:].values
 
 from scipy.stats import median_abs_deviation
@@ -793,6 +957,13 @@ _MetricScipySpatial = Literal[
     ]
 _Metric = Union[_MetricSparseCapable, _MetricScipySpatial]
 from types import MappingProxyType
+@register_function(
+    aliases=["计算邻居", "neighbors", "knn", "邻居图"],
+    category="preprocessing",
+    description="Compute neighborhood graph of cells",
+    examples=["ov.pp.neighbors(adata, n_neighbors=15)"],
+    related=["umap", "leiden", "louvain"]
+)
 def neighbors(
     adata: anndata.AnnData,
     n_neighbors: int = 15,
@@ -864,7 +1035,8 @@ def neighbors(
     """
     if settings.mode =='cpu' or settings.mode == 'cpu-gpu-mixed':
         print(f"{EMOJI['cpu']} Using Scanpy CPU to calculate neighbors...")
-        sc.pp.neighbors(adata,use_rep=use_rep,n_neighbors=n_neighbors, n_pcs=n_pcs,
+        from ._neighbors import neighbors as _neighbors
+        _neighbors(adata,use_rep=use_rep,n_neighbors=n_neighbors, n_pcs=n_pcs,
                          random_state=random_state,method=method,metric=metric,
                          metric_kwds=metric_kwds,
                          key_added=key_added,copy=copy)
@@ -878,6 +1050,13 @@ def neighbors(
     add_reference(adata,'scanpy','neighbors with scanpy')
 
 
+@register_function(
+    aliases=["umap", "UMAP", "非线性降维"],
+    category="preprocessing",
+    description="Compute UMAP embedding for visualization",
+    examples=["ov.pp.umap(adata)"],
+    related=["tsne", "pca", "mde", "neighbors"]
+)
 def umap(adata, **kwargs):
     """
     Run UMAP on AnnData, choosing implementation based on settings.mode,
@@ -887,7 +1066,8 @@ def umap(adata, **kwargs):
     try:
         if settings.mode == 'cpu':
             print(f"{EMOJI['cpu']} Using Scanpy CPU UMAP...")
-            sc.tl.umap(adata, **kwargs)
+            from ._umap import umap as _umap
+            _umap(adata, **kwargs)
             add_reference(adata,'umap','UMAP with scanpy')
 
         elif settings.mode == 'cpu-gpu-mixed':
@@ -926,32 +1106,86 @@ def louvain(adata, **kwargs):
         rsc.tl.louvain(adata, **kwargs)
         add_reference(adata,'louvain','Louvain clustering with RAPIDS')
 
-def leiden(adata, **kwargs):
+@register_function(
+    aliases=["莱顿聚类", "leiden", "clustering", "聚类"],
+    category="preprocessing",
+    description="Perform Leiden community detection clustering",
+    examples=["ov.pp.leiden(adata, resolution=1.0)"],
+    related=["louvain", "neighbors"]
+)
+def leiden(
+    adata, resolution=1.0, random_state=0, 
+    key_added='leiden', local_iterations=100, max_levels=10, device='cpu',**kwargs):
     '''
     leiden clustering
     '''
 
-    if settings.mode =='cpu' or settings.mode == 'cpu-gpu-mixed':
+    if settings.mode =='cpu':
         print(f"{EMOJI['cpu']} Using Scanpy CPU Leiden...")
-        sc.tl.leiden(adata, **kwargs)
+        #sc.tl.leiden(adata, **kwargs)
+        from ._leiden import leiden as _leiden
+        _leiden(adata, resolution=resolution, random_state=random_state, 
+            key_added=key_added,**kwargs)
         add_reference(adata,'leiden','Leiden clustering with scanpy')
+    elif settings.mode == 'cpu-gpu-mixed':
+        print(f"{EMOJI['mixed']} Using torch CPU/GPU mixed mode to calculate Leiden...")
+        print_gpu_usage_color()
+        from ._leiden_pyg import leiden_gpu_sparse_multilevel as _leiden
+        _leiden(
+            adata,
+            resolution=resolution,
+            random_state=random_state,
+            key_added=key_added,
+            # your API uses these names:
+            local_iterations=local_iterations,
+            max_levels=max_levels,
+            device=device,  # None -> auto-pick
+        )
+        add_reference(adata,'leiden','Leiden clustering with omicverse')
     else:
         print(f"{EMOJI['gpu']} Using RAPIDS GPU to calculate Leiden...")
         import rapids_singlecell as rsc
-        rsc.tl.leiden(adata, **kwargs)
+        rsc.tl.leiden(adata, resolution=resolution, random_state=random_state, 
+            key_added=key_added,**kwargs)
         add_reference(adata,'leiden','Leiden clustering with RAPIDS')
 
 
+@register_function(
+    aliases=["细胞周期评分", "score_genes_cell_cycle", "cell_cycle", "细胞周期", "cc_score"],
+    category="preprocessing",
+    description="Score cell cycle phases (S and G2M) using predefined gene sets",
+    examples=[
+        "# Basic cell cycle scoring for human data",
+        "ov.pp.score_genes_cell_cycle(adata, species='human')",
+        "# Mouse cell cycle scoring",
+        "ov.pp.score_genes_cell_cycle(adata, species='mouse')",
+        "# Custom gene lists",
+        "s_genes = ['MCM5', 'PCNA', 'TYMS']",
+        "g2m_genes = ['HMGB2', 'CDK1', 'NUSAP1']",
+        "ov.pp.score_genes_cell_cycle(adata, s_genes=s_genes, g2m_genes=g2m_genes)",
+        "# Visualize cell cycle phases",
+        "ov.pl.embedding(adata, basis='X_umap', color='phase')"
+    ],
+    related=["pp.preprocess", "pl.embedding", "utils.embedding"]
+)
 def score_genes_cell_cycle(adata,species='human',s_genes=None, g2m_genes=None):
-    """
-    Score cell cycle .
+    """Score cell cycle phases using predefined or custom gene sets.
 
     Arguments:
         adata: Annotated data matrix with rows for cells and columns for genes.
-        species: The species of the data. It can be either 'human' or 'mouse'.
-        s_genes: The list of genes that are specific to the S phase of the cell cycle.
-        g2m_genes: The list of genes that are specific to the G2/M phase of the cell cycle.
-    
+        species: The species of the data ('human' or 'mouse'). Default: 'human'.
+        s_genes: Custom list of S phase genes. Default: None (uses predefined).
+        g2m_genes: Custom list of G2M phase genes. Default: None (uses predefined).
+
+    Returns:
+        None: Updates adata.obs with 'S_score', 'G2M_score', and 'phase' columns.
+
+    Examples:
+        >>> import omicverse as ov
+        >>> # Score cell cycle for human data
+        >>> ov.pp.score_genes_cell_cycle(adata, species='human')
+        >>> # Check results
+        >>> print(adata.obs[['S_score', 'G2M_score', 'phase']].head())
     """
     if s_genes is None:
         if species=='human':
@@ -1037,21 +1271,44 @@ def mde(adata,embedding_dim=2,n_neighbors=15, basis='X_mde',n_pcs=None, use_rep=
     # 记录开始时间
     start_time = time.time()
 
-    print("computing neighbors")
+    print(f"{Colors.HEADER}{Colors.BOLD}{EMOJI['start']} MDE Dimensionality Reduction:{Colors.ENDC}")
+    print(f"   {Colors.CYAN}Mode: {Colors.BOLD}{settings.mode}{Colors.ENDC}")
+    print(f"   {Colors.CYAN}Embedding dimensions: {Colors.BOLD}{embedding_dim}{Colors.ENDC}")
+    print(f"   {Colors.CYAN}Neighbors: {Colors.BOLD}{n_neighbors}{Colors.ENDC}")
+    print(f"   {Colors.CYAN}Repulsive fraction: {Colors.BOLD}{repulsive_fraction}{Colors.ENDC}")
+    
     if use_rep is None:
         use_rep='X_pca'
+    print(f"   {Colors.CYAN}Using representation: {Colors.BOLD}{use_rep}{Colors.ENDC}")
     data=adata.obsm[use_rep]
     if n_pcs is None:
         n_pcs=50
+    print(f"   {Colors.CYAN}Principal components: {Colors.BOLD}{n_pcs}{Colors.ENDC}")
     data=data[:,:n_pcs]
 
+    # Determine device based on available accelerators
+    import torch
+    from .._settings import get_optimal_device, prepare_data_for_device
+    device = get_optimal_device(prefer_gpu=True, verbose=True)
+    
+    # Convert device to string for PyMDE compatibility
+    if hasattr(device, 'type'):
+        device_str = device.type
+    else:
+        device_str = str(device)
+    
+    # Prepare data for MPS compatibility (float32 requirement)
+    data = prepare_data_for_device(data, device, verbose=True)
+    
+    print(f"   {Colors.GREEN}{EMOJI['start']} Computing k-nearest neighbors graph...{Colors.ENDC}")
+    
     if constraint is None:
         _kwargs = {
         "embedding_dim": embedding_dim,
         "constraint": pymde.Standardized(),
         "repulsive_fraction": repulsive_fraction,
         "verbose": verbose,
-        "device": 'cuda',
+        "device": device_str,
         "n_neighbors": n_neighbors,
     }
     else:
@@ -1060,15 +1317,16 @@ def mde(adata,embedding_dim=2,n_neighbors=15, basis='X_mde',n_pcs=None, use_rep=
         "constraint": constraint,
         "repulsive_fraction": repulsive_fraction,
         "verbose": verbose,
-        "device": 'cuda',
+        "device": device_str,
         "n_neighbors": n_neighbors,
     }
     #_kwargs.update(kwargs)
     
     gr=pymde.preprocess.k_nearest_neighbors(data,k=n_neighbors)
-
+    
+    print(f"   {Colors.GREEN}{EMOJI['start']} Creating MDE embedding...{Colors.ENDC}")
     mde = pymde.preserve_neighbors(data, **_kwargs)
-    import torch
+    print(f"   {Colors.GREEN}{EMOJI['start']} Optimizing embedding...{Colors.ENDC}")
     emb=mde.embed(verbose=_kwargs["verbose"])
 
     if isinstance(emb, torch.Tensor):
@@ -1123,19 +1381,145 @@ def mde(adata,embedding_dim=2,n_neighbors=15, basis='X_mde',n_pcs=None, use_rep=
     adata.obsp[dists_key] = sparse_distance_matrix
     adata.obsp[conns_key] = gr.adjacency_matrix
     adata.obsm[basis]=emb
+    
     # 记录结束时间
     end_time = time.time()
     elapsed_time = end_time - start_time
+    print(f"\n")
+    print(f"{Colors.GREEN}{EMOJI['done']} MDE Dimensionality Reduction Completed Successfully!{Colors.ENDC}")
+    print(f"   {Colors.GREEN}✓ Embedding shape: {Colors.BOLD}{emb.shape[0]:,}{Colors.ENDC}{Colors.GREEN} cells × {Colors.BOLD}{emb.shape[1]}{Colors.ENDC}{Colors.GREEN} dimensions{Colors.ENDC}")
+    print(f"   {Colors.GREEN}✓ Runtime: {Colors.BOLD}{elapsed_time:.2f}s{Colors.ENDC}")
+    print(f"   {Colors.GREEN}✓ Results added to AnnData object:{Colors.ENDC}")
+    print(f"     {Colors.CYAN}• '{basis}': {Colors.BOLD}MDE coordinates{Colors.ENDC}{Colors.CYAN} (adata.obsm){Colors.ENDC}")
+    print(f"     {Colors.CYAN}• '{key_added}': {Colors.BOLD}Neighbors metadata{Colors.ENDC}{Colors.CYAN} (adata.uns){Colors.ENDC}")
+    print(f"     {Colors.CYAN}• '{dists_key}': {Colors.BOLD}Distance matrix{Colors.ENDC}{Colors.CYAN} (adata.obsp){Colors.ENDC}")
+    print(f"     {Colors.CYAN}• '{conns_key}': {Colors.BOLD}Connectivity matrix{Colors.ENDC}{Colors.CYAN} (adata.obsp){Colors.ENDC}")
 
-    # 打印结果和日志信息
-    print("    finished: added to `.uns['neighbors']`")
-    print(f"    `.obsm['{basis}']`, MDE coordinates")
-    if key_added is None:
-        print("    `.obsp['distances']`, distances for each pair of neighbors")
-        print("    `.obsp['connectivities']`, weighted adjacency matrix (0:{:02}:{:02})".format(int(elapsed_time // 60), int(elapsed_time % 60)))
-
-    else:
-        print(f"    `.obsp['{key_added}_distances']`, distances for each pair of neighbors")
-        print("    `.obsp['{}_connectivities']`, weighted adjacency matrix (0:{:02}:{:02})".format(key_added,int(elapsed_time // 60), int(elapsed_time % 60)))
     add_reference(adata,'pymde','MDE with pymde')
     #return emb
+
+
+import numpy as np
+
+def calc_mean_and_var_sparse(M, N, data, indices, indptr, axis):
+    i, j=0,0
+
+    size=0
+    value=0.0
+
+    size = N if axis == 0 else M
+    mean = np.zeros(size, dtype = np.float64)
+    var = np.zeros(size, dtype = np.float64)
+
+    mean_view = mean
+    var_view = var
+
+    for i in range(M):
+        for j in range(indptr[i], indptr[i + 1]):
+            value = data[j]
+            if axis == 0:
+                mean_view[indices[j]] += value
+                var_view[indices[j]] += value * value
+            else:
+                mean_view[i] += value
+                var_view[i] += value * value
+
+    size = M if axis == 0 else N
+    for i in range(mean_view.size):
+        mean_view[i] /= size
+        var_view[i] = (var_view[i] - size * mean_view[i] * mean_view[i]) / (size - 1)
+
+    return mean, var
+
+def calc_stat_per_batch_sparse(M, N, data, indices, indptr,nbatch, codes):
+    i, j=0,0
+
+    col=0
+    code=0
+    value=0.0
+
+    ncells = np.zeros(nbatch, dtype = np.int32)
+    means = np.zeros((N, nbatch), dtype = np.float64)
+    partial_sum = np.zeros((N, nbatch), dtype = np.float64)
+
+    ncells_view = ncells
+    means_view = means
+    ps_view = partial_sum
+
+    for i in range(M):
+        code = codes[i]
+        ncells_view[code] += 1
+        for j in range(indptr[i], indptr[i + 1]):
+            col = indices[j]
+            value = data[j]
+            means_view[col, code] += value
+            ps_view[col, code] += value * value
+
+    for j in range(nbatch):
+        if ncells_view[j] > 1:
+            for i in range(N):
+                means_view[i, j] /= ncells_view[j]
+                ps_view[i, j] = ps_view[i, j] - ncells_view[j] * means_view[i, j] * means_view[i, j]
+
+    return ncells, means, partial_sum
+
+def calc_mean_and_var_dense(M, N, X, axis):
+    i, j=0,0
+
+    size=0
+    value=0.0
+
+    size = N if axis == 0 else M
+    mean = np.zeros(size, dtype = np.float64)
+    var = np.zeros(size, dtype = np.float64)
+
+    mean_view = mean
+    var_view = var
+
+    for i in range(M):
+        for j in range(N):
+            value = X[i, j]
+            if axis == 0:
+                mean_view[j] += value
+                var_view[j] += value * value
+            else:
+                mean_view[i] += value
+                var_view[i] += value * value
+
+    size = M if axis == 0 else N
+    for i in range(mean_view.size):
+        mean_view[i] /= size
+        var_view[i] = (var_view[i] - size * mean_view[i] * mean_view[i]) / (size - 1)
+
+    return mean, var
+
+
+def calc_stat_per_batch_dense(M, N, X, nbatch, codes):
+    i, j=0,0
+
+    code, col=0,0
+    value=0.0
+
+    ncells = np.zeros(nbatch, dtype = np.int32)
+    means = np.zeros((N, nbatch), dtype = np.float64)
+    partial_sum = np.zeros((N, nbatch), dtype = np.float64)
+
+    ncells_view = ncells
+    means_view = means
+    ps_view = partial_sum
+
+    for i in range(M):
+        code = codes[i]
+        ncells_view[code] += 1
+        for j in range(N):
+            value = X[i, j]
+            means_view[j, code] += value
+            ps_view[j, code] += value * value
+
+    for j in range(nbatch):
+        if ncells_view[j] > 1:
+            for i in range(N):
+                means_view[i, j] /= ncells_view[j]
+                ps_view[i, j] = ps_view[i, j] - ncells_view[j] * means_view[i, j] * means_view[i, j]
+
+    return ncells, means, partial_sum
