@@ -14,11 +14,12 @@ from scipy import sparse
 
 from scanpy import logging as logg
 from scanpy import preprocessing as pp
-from scanpy._compat import old_positionals
+from ._compat import old_positionals
 from scanpy.get import _get_obs_rep
 from scanpy.preprocessing._scrublet import pipeline
 from scanpy.preprocessing._scrublet.core import Scrublet
-from .._settings import EMOJI
+from .._settings import EMOJI, Colors, settings
+from datetime import datetime
 
 
 from importlib.util import find_spec
@@ -29,17 +30,16 @@ import pandas as pd
 from anndata import AnnData
 from scipy import sparse
 
-from scanpy import logging as logg
-from scanpy import preprocessing as pp
-from scanpy._compat import old_positionals
-from scanpy.get import _get_obs_rep
-from scanpy.preprocessing._scrublet import pipeline
-from scanpy.preprocessing._scrublet.core import Scrublet
+
+
 
 if TYPE_CHECKING:
-    from scanpy._compat import _LegacyRandom
+    from typing import Literal
+    from ._compat import _LegacyRandom
     from scanpy.neighbors import _Metric, _MetricFn
 
+from ._normalization import normalize_total
+from ._highly_variable_genes import highly_variable_genes
 
 @old_positionals(
     "batch_key",
@@ -123,15 +123,23 @@ def scrublet(
     if copy:
         adata = adata.copy()
 
-    start = logg.info(f"Running Scrublet{EMOJI['start']}")
-
-    adata_obs = adata.copy()
+    print(f"\n{Colors.HEADER}{Colors.BOLD}{EMOJI['start']} Running Scrublet Doublet Detection:{Colors.ENDC}")
+    print(f"   {Colors.CYAN}Mode: {Colors.BOLD}{settings.mode}{Colors.ENDC}")
+    print(f"   {Colors.CYAN}Computing doublet prediction using Scrublet algorithm{Colors.ENDC}")
+    from ._qc import _is_rust_backend
+    is_rust = _is_rust_backend(adata)
+    if is_rust:
+        adata_obs = adata.to_memory()
+    else:
+        adata_obs = adata.copy()
+    #adata_obs = adata.copy()
 
     def _run_scrublet(ad_obs: AnnData, ad_sim: AnnData | None = None):
         # With no adata_sim we assume the regular use case, starting with raw
         # counts and simulating doublets
 
         if ad_sim is None:
+            print(f"   {Colors.GREEN}{EMOJI['start']} Filtering genes and cells...{Colors.ENDC}")
             pp.filter_genes(ad_obs, min_cells=3)
             pp.filter_cells(ad_obs, min_genes=3)
 
@@ -139,19 +147,21 @@ def scrublet(
             # selection of genes following normalisation and variability filtering. So
             # we need to save the raw and subset at the same time.
 
+            print(f"   {Colors.GREEN}{EMOJI['start']} Normalizing data and selecting highly variable genes...{Colors.ENDC}")
             ad_obs.layers["raw"] = ad_obs.X.copy()
-            pp.normalize_total(ad_obs)
+            normalize_total(ad_obs)
 
             # HVG process needs log'd data.
             ad_obs.layers["log1p"] = ad_obs.X.copy()
             pp.log1p(ad_obs, layer="log1p")
-            pp.highly_variable_genes(ad_obs, layer="log1p")
+            highly_variable_genes(ad_obs, layer="log1p")
             del ad_obs.layers["log1p"]
             ad_obs = ad_obs[:, ad_obs.var["highly_variable"]].copy()
 
             # Simulate the doublets based on the raw expressions from the normalised
             # and filtered object.
 
+            print(f"   {Colors.GREEN}{EMOJI['start']} Simulating synthetic doublets...{Colors.ENDC}")
             ad_sim = scrublet_simulate_doublets(
                 ad_obs,
                 layer="raw",
@@ -166,8 +176,9 @@ def scrublet(
 
             # Now normalise simulated and observed in the same way
 
-            pp.normalize_total(ad_obs, target_sum=1e6)
-            pp.normalize_total(ad_sim, target_sum=1e6)
+            print(f"   {Colors.GREEN}{EMOJI['start']} Normalizing observed and simulated data...{Colors.ENDC}")
+            normalize_total(ad_obs, target_sum=1e6)
+            normalize_total(ad_sim, target_sum=1e6)
 
         ad_obs = _scrublet_call_doublets(
             adata_obs=ad_obs,
@@ -184,6 +195,7 @@ def scrublet(
             threshold=threshold,
             random_state=random_state,
             verbose=verbose,
+            use_gpu=use_gpu,
         )
 
         return {"obs": ad_obs.obs, "uns": ad_obs.uns["scrublet"]}
@@ -234,7 +246,11 @@ def scrublet(
         adata.obs["predicted_doublet"] = scrubbed["obs"]["predicted_doublet"]
         adata.uns["scrublet"] = scrubbed["uns"]
 
-    logg.info(f"    Scrublet finished{EMOJI['done']}", time=start)
+    print(f"\n{Colors.GREEN}{EMOJI['done']} Scrublet Analysis Completed Successfully!{Colors.ENDC}")
+    print(f"   {Colors.GREEN}✓ Results added to AnnData object:{Colors.ENDC}")
+    print(f"     {Colors.CYAN}• 'doublet_score': {Colors.BOLD}Doublet scores{Colors.ENDC}{Colors.CYAN} (adata.obs){Colors.ENDC}")
+    print(f"     {Colors.CYAN}• 'predicted_doublet': {Colors.BOLD}Boolean predictions{Colors.ENDC}{Colors.CYAN} (adata.obs){Colors.ENDC}")
+    print(f"     {Colors.CYAN}• 'scrublet': {Colors.BOLD}Parameters and metadata{Colors.ENDC}{Colors.CYAN} (adata.uns){Colors.ENDC}")
 
     return adata if copy else None
 
@@ -255,7 +271,8 @@ def _scrublet_call_doublets(
     threshold: float | None = None,
     random_state: _LegacyRandom = 0,
     verbose: bool = True,
-) -> AnnData:
+    use_gpu: bool = False,
+    ) -> AnnData:
     """Core function for predicting doublets using Scrublet :cite:p:`Wolock2019`.
 
     Predict cell doublets using a nearest-neighbor classifier of observed
@@ -375,17 +392,17 @@ def _scrublet_call_doublets(
     # than trying to use Scanpy's PCA wrapper of the same functions.
 
     if mean_center:
-        logg.info("Embedding transcriptomes using PCA...")
-        pca_torch(scrub, n_prin_comps=n_prin_comps, random_state=scrub._random_state)
+        print(f"   {Colors.GREEN}{EMOJI['start']} Embedding transcriptomes using PCA...{Colors.ENDC}")
+        pca_torch(scrub, n_prin_comps=n_prin_comps, random_state=scrub._random_state,use_gpu=use_gpu)
         #pipeline.pca(scrub, n_prin_comps=n_prin_comps, random_state=scrub._random_state)
     else:
-        logg.info("Embedding transcriptomes using Truncated SVD...")
+        print(f"   {Colors.GREEN}{EMOJI['start']} Embedding transcriptomes using Truncated SVD...{Colors.ENDC}")
         pipeline.truncated_svd(
             scrub, n_prin_comps=n_prin_comps, random_state=scrub._random_state
         )
 
     # Score the doublets
-
+    print(f"   {Colors.GREEN}{EMOJI['start']} Calculating doublet scores...{Colors.ENDC}")
     scrub.calculate_doublet_scores(
         use_approx_neighbors=use_approx_neighbors,
         distance_metric=knn_dist_metric,
@@ -393,8 +410,32 @@ def _scrublet_call_doublets(
     )
 
     # Actually call doublets
-
-    scrub.call_doublets(threshold=threshold, verbose=verbose)
+    print(f"   {Colors.GREEN}{EMOJI['start']} Calling doublets with threshold detection...{Colors.ENDC}")
+    scrub.call_doublets(threshold=threshold, verbose=False)  # Suppress scrublet's output
+    
+    # Display formatted scrublet results
+    if hasattr(scrub, 'threshold_'):
+        print(f"   {Colors.CYAN}📊 Automatic threshold: {Colors.BOLD}{scrub.threshold_:.3f}{Colors.ENDC}")
+        
+        # Calculate statistics
+        n_doublets_detected = sum(scrub.predicted_doublets_)
+        detection_rate = n_doublets_detected / len(scrub.predicted_doublets_) * 100
+        print(f"   {Colors.CYAN}📈 Detected doublet rate: {Colors.BOLD}{detection_rate:.1f}%{Colors.ENDC}")
+        
+        # Show expected vs estimated
+        expected_rate = expected_doublet_rate * 100
+        if hasattr(scrub, 'doublet_scores_sim_') and len(scrub.doublet_scores_sim_) > 0:
+            # Estimate detectable fraction (simplified calculation)
+            sim_scores_above_threshold = sum(scrub.doublet_scores_sim_ > scrub.threshold_)
+            detectable_fraction = sim_scores_above_threshold / len(scrub.doublet_scores_sim_) * 100
+            estimated_rate = detection_rate / (detectable_fraction / 100) if detectable_fraction > 0 else detection_rate
+            
+            print(f"   {Colors.CYAN}🔍 Detectable doublet fraction: {Colors.BOLD}{detectable_fraction:.1f}%{Colors.ENDC}")
+            print(f"   {Colors.BLUE}📊 Overall doublet rate comparison:{Colors.ENDC}")
+            print(f"     {Colors.CYAN}• Expected: {Colors.BOLD}{expected_rate:.1f}%{Colors.ENDC}")
+            print(f"     {Colors.CYAN}• Estimated: {Colors.BOLD}{estimated_rate:.1f}%{Colors.ENDC}")
+    else:
+        print(f"   {Colors.WARNING}⚠️ Could not determine automatic threshold - manual threshold may be needed{Colors.ENDC}")
 
     # Store results in AnnData for return
 
@@ -498,17 +539,58 @@ def pca_torch(
     if use_gpu:
         if svd_solver == "auto":
             svd_solver = "gesvd"
-        try:
-            from torchdr import PCA
-        except ImportError:
-            raise ImportError("torchdr is not installed. Please install it using `pip install torchdr`.")
+        
         import torch
-        device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
+        from .._settings import get_optimal_device, prepare_data_for_device
+        device = get_optimal_device(prefer_gpu=True, verbose=True)
+        
+        # Use MLX for MPS devices (Apple Silicon optimization)
+        if device.type == 'mps':
+            try:
+                from ._pca_mlx import MLXPCA, MockPCA
+                print(f"   {Colors.GREEN}{EMOJI['gpu']} Using MLX PCA for Apple Silicon MPS acceleration in scrublet{Colors.ENDC}")
+                print(f"   {Colors.GREEN}{EMOJI['gpu']} MLX PCA backend: Apple Silicon GPU acceleration (scrublet){Colors.ENDC}")
+                
+                # Create MLX PCA instance (use "metal" for MLX)
+                mlx_pca = MLXPCA(n_components=n_prin_comps, device="metal")
+                
+                # Fit and transform
+                X_obs_transformed = mlx_pca.fit_transform(X_obs)
+                X_sim_transformed = mlx_pca.transform(X_sim)
+                
+                # Create a mock PCA object with sklearn-compatible interface
+                pca = MockPCA(mlx_pca)
+                # Set manifold directly since we already have transformed data
+                self.set_manifold(X_obs_transformed, X_sim_transformed)
+                
+            except (ImportError, Exception) as e:
+                print(f"   {EMOJI['warning']} {Colors.WARNING}MLX PCA failed in scrublet ({str(e)}), falling back to sklearn for MPS device{Colors.ENDC}")
+                print(f"   {EMOJI['warning']} {Colors.WARNING}sklearn PCA backend: MPS device fallback (scrublet){Colors.ENDC}")
+                # For MPS devices, fall back to sklearn instead of TorchDR
+                from sklearn.decomposition import PCA
+                
+                pca = PCA(
+                    n_components=n_prin_comps, random_state=random_state, svd_solver="arpack",
+                ).fit(X_obs)
+                self.set_manifold(pca.transform(X_obs), pca.transform(X_sim))
+        else:
+            # Use TorchDR for non-MPS GPU devices (CUDA, etc.)
+            print(f"   {Colors.GREEN}{EMOJI['gpu']} Using TorchDR PCA for {device.type.upper()} GPU acceleration in scrublet{Colors.ENDC}")
+            print(f"   {Colors.GREEN}{EMOJI['gpu']} TorchDR PCA backend: {device.type.upper()} GPU acceleration (scrublet){Colors.ENDC}")
+            
+            try:
+                from torchdr import PCA
+            except ImportError:
+                raise ImportError("torchdr is not installed. Please install it using `pip install torchdr`.")
+            
+            # Prepare data for GPU compatibility (float32 requirement)
+            X_obs = prepare_data_for_device(X_obs, device, verbose=True)
 
-        pca = PCA(
-            n_components=n_prin_comps, random_state=None, svd_driver=svd_solver,
-            device=device,
-        ).fit(X_obs)
+            pca = PCA(
+                n_components=n_prin_comps, random_state=None, svd_driver=svd_solver,
+                device=device,
+            ).fit(X_obs)
+            self.set_manifold(pca.transform(X_obs), pca.transform(X_sim))
     else:
         from sklearn.decomposition import PCA
         if svd_solver == "auto":
@@ -516,5 +598,5 @@ def pca_torch(
         pca = PCA(
             n_components=n_prin_comps, random_state=random_state, svd_solver=svd_solver,
         ).fit(X_obs)
-    self.set_manifold(pca.transform(X_obs), pca.transform(X_sim))
+        self.set_manifold(pca.transform(X_obs), pca.transform(X_sim))
 
