@@ -50,15 +50,31 @@ Examples:
 from ._cosg import cosg
 from ._anno import pySCSA,MetaTiME,scanpy_lazy,scanpy_cellanno_from_dict,get_celltype_marker
 from ._nocd import scnocd
-from ._mofa import pyMOFAART,pyMOFA,GLUE_pair,factor_exact,factor_correlation,get_weights,glue_pair,get_r2_from_hdf5
-from ._scdrug import autoResolution,writeGEP,Drug_Response
-from ._cpdb import (cpdb_network_cal,cpdb_plot_network,
-                    cpdb_plot_interaction,
-                    cpdb_interaction_filtered,
-                    cpdb_submeans_exacted,cpdb_exact_target,cpdb_exact_source,cellphonedb_v5,run_cellphonedb_v5)
-from ._scgsea import (geneset_aucell,pathway_aucell,pathway_aucell_enrichment,
-                      geneset_aucell_tmp,pathway_aucell_tmp,pathway_aucell_enrichment_tmp,
-                      pathway_enrichment,pathway_enrichment_plot,)
+from ._mofa import (
+    pyMOFAART,pyMOFA,GLUE_pair,
+    factor_exact,factor_correlation,
+    get_weights,glue_pair,get_r2_from_hdf5,
+    convert_r2_to_matrix,factor_group_correlation_mdata,
+    plot_factor_group_associations,plot_factor_boxplots,
+    plot_factors_violin,plot_weights,
+)
+from ._scdrug import (
+    autoResolution,writeGEP,Drug_Response
+)
+from ._cpdb import (
+    cpdb_network_cal,cpdb_plot_network,
+    cpdb_plot_interaction,
+    cpdb_interaction_filtered,
+    cpdb_submeans_exacted,cpdb_exact_target,
+    cpdb_exact_source,cellphonedb_v5,
+    run_cellphonedb_v5
+)
+
+from ._scgsea import (
+    geneset_aucell,pathway_aucell,pathway_aucell_enrichment,
+    geneset_aucell_tmp,pathway_aucell_tmp,pathway_aucell_enrichment_tmp,
+    pathway_enrichment,pathway_enrichment_plot
+)
 #from ._via import pyVIA,scRNA_hematopoiesis
 from ._simba import pySIMBA
 from ._tosica import pyTOSICA
@@ -83,15 +99,41 @@ from ._cellvote import get_cluster_celltype,CellVote
 from ._deg_ct import DCT,DEG
 from ._lazy_function import lazy
 from ._lazy_report import generate_scRNA_report
+from ._lazy_checkpoint import lazy_checkpoint, resume_from_checkpoint, list_checkpoints, cleanup_checkpoints
+from ._lazy_step_by_step import (
+    lazy_step_qc, lazy_step_preprocess, lazy_step_scale, lazy_step_pca,
+    lazy_step_cell_cycle, lazy_step_harmony, lazy_step_scvi, 
+    lazy_step_select_best_method, lazy_step_mde, lazy_step_clustering,
+    lazy_step_final_embeddings, lazy_step_by_step_guide
+)
 from ._diffusionmap import diffmap
 from ._cellmatch import CellOntologyMapper,download_cl
 from ._scenic import SCENIC,build_correlation_network_umap_layout,add_tf_regulation,plot_grn
+from ._annotation import Annotation
+from ._annotation_ref import AnnotationRef
+from ._velo import Velo,velocity_embedding
 
 
 __all__ = [
     # Core analysis functions
     'cosg',
     'lazy',
+    'lazy_checkpoint', 
+    'resume_from_checkpoint',
+    'list_checkpoints',
+    'cleanup_checkpoints',
+    'lazy_step_qc',
+    'lazy_step_preprocess',
+    'lazy_step_scale',
+    'lazy_step_pca',
+    'lazy_step_cell_cycle',
+    'lazy_step_harmony',
+    'lazy_step_scvi',
+    'lazy_step_select_best_method',
+    'lazy_step_mde',
+    'lazy_step_clustering',
+    'lazy_step_final_embeddings',
+    'lazy_step_by_step_guide',
     'aucell',
     
     # Cell type annotation
@@ -123,6 +165,12 @@ __all__ = [
     'Wrapper',
     'Integration',
     'Batch',
+    'convert_r2_to_matrix',
+    'factor_group_correlation_mdata',
+    'plot_factor_group_associations',
+    'plot_factor_boxplots',
+    'plot_factors_violin',
+    'plot_weights',
     
     # Trajectory and pseudotime analysis
     #'pyVIA',
@@ -202,5 +250,9 @@ __all__ = [
     'build_correlation_network_umap_layout',
     'add_tf_regulation',
     'plot_grn',
+    'Velo',
+    'velocity_embedding',
+    'Annotation',   # cell type annotation
+    'AnnotationRef', # cell type annotation with reference
 ]
 
