@@ -13,6 +13,7 @@ from ..pp._preprocess import scale
 from ..utils import gen_mpl_labels
 
 from .._settings import add_reference
+from ..utils.registry import register_function
 
 def global_imports(modulename,shortname = None, asfunction = False):
     if shortname is None: 
@@ -185,21 +186,27 @@ def __cell_annotate(data,
 
 def __cell_anno_print(anno):
     r"""Print the annotation result.
-    
+
     Arguments:
         anno: The annotation result
-    
+
     Returns:
         None
     """
     for i in set(anno['Cluster']):
         test=anno.loc[anno['Cluster']==i].iloc[:2]
-        if test.iloc[0]['Z-score']>test.iloc[1]['Z-score']*2:
-            print('Nice:Cluster:{}\tCell_type:{}\tZ-score:{}'.format(i,test.iloc[0]['Cell Type'],
+        if len(test) >= 2 and pd.notna(test.iloc[0]['Z-score']) and pd.notna(test.iloc[1]['Z-score']):
+            if test.iloc[0]['Z-score']>test.iloc[1]['Z-score']*2:
+                print('Nice:Cluster:{}\tCell_type:{}\tZ-score:{}'.format(i,test.iloc[0]['Cell Type'],
+                                                            np.around(test.iloc[0]['Z-score'],3)))
+            else:
+                print('Cluster:{}\tCell_type:{}\tZ-score:{}'.format(i,('|').join(test['Cell Type'].values.tolist()),
+                                                            ('|').join(np.around(test['Z-score'].values,3).astype(str).tolist())))
+        elif len(test) >= 1 and pd.notna(test.iloc[0]['Z-score']):
+            print('Cluster:{}\tCell_type:{}\tZ-score:{}'.format(i,test.iloc[0]['Cell Type'],
                                                         np.around(test.iloc[0]['Z-score'],3)))
         else:
-            print('Cluster:{}\tCell_type:{}\tZ-score:{}'.format(i,('|').join(test['Cell Type'].values.tolist()),
-                                                        ('|').join(np.around(test['Z-score'].values,3).astype(str).tolist())))
+            print('Cluster:{}\tCell_type:{}\tZ-score:{}'.format(i,'Unknown','NaN'))
 
 def scanpy_lazy(adata:anndata.AnnData,min_genes:int=200,min_cells:int=3,drop_doublet:bool=True,
                 n_genes_by_counts:int=4300,pct_counts_mt:int=25,
@@ -268,6 +275,27 @@ def scanpy_lazy(adata:anndata.AnnData,min_genes:int=200,min_cells:int=3,drop_dou
     sc.tl.umap(adata, init_pos='paga')
     return adata
 
+@register_function(
+    aliases=["字典注释", "scanpy_cellanno_from_dict", "manual_annotation", "手动注释", "字典映射注释"],
+    category="single",
+    description="Manual cell type annotation from cluster-to-celltype dictionary mapping",
+    examples=[
+        "# Basic manual annotation from dictionary",
+        "cluster2annotation = {",
+        "    '0': 'T cell', '1': 'B cell', '2': 'Monocyte',",
+        "    '3': 'NK cell', '4': 'Dendritic cell'",
+        "}",
+        "ov.single.scanpy_cellanno_from_dict(adata, anno_dict=cluster2annotation,",
+        "                                    clustertype='leiden')",
+        "# Custom annotation key",
+        "ov.single.scanpy_cellanno_from_dict(adata, anno_dict=cluster2annotation,",
+        "                                    clustertype='leiden',",
+        "                                    key_added='manual_celltype')",
+        "# Compare with automatic annotation",
+        "ov.utils.embedding(adata, color=['manual_celltype', 'scsa_celltype'])"
+    ],
+    related=["single.pySCSA", "single.get_celltype_marker", "utils.embedding"]
+)
 def scanpy_cellanno_from_dict(adata:anndata.AnnData,
                                anno_dict:dict,
                                anno_name:str='major',
@@ -288,6 +316,26 @@ def scanpy_cellanno_from_dict(adata:anndata.AnnData,
     adata.obs[anno_name+'_celltype'] = adata.obs[clustertype].map(anno_dict).astype('category')
     print('...cell type added to {}_celltype on obs of anndata'.format(anno_name))
 
+@register_function(
+    aliases=["细胞类型标记基因", "get_celltype_marker", "celltype_markers", "标记基因", "差异基因"],
+    category="single",
+    description="Extract cell type-specific marker genes from differential expression analysis",
+    examples=[
+        "# Get markers for all cell types",
+        "marker_dict = ov.single.get_celltype_marker(adata,",
+        "                                           clustertype='leiden')",
+        "print(marker_dict.keys())  # Show available cell types",
+        "# Get markers for specific cell type annotation",
+        "marker_dict = ov.single.get_celltype_marker(adata,",
+        "                                           clustertype='scsa_celltype')",
+        "# View markers for specific cell type",
+        "b_cell_markers = marker_dict['B cell']",
+        "print(b_cell_markers[:10])  # Top 10 markers",
+        "# Use with plotting",
+        "sc.pl.dotplot(adata, marker_dict, groupby='leiden')"
+    ],
+    related=["single.pySCSA", "single.scanpy_cellanno_from_dict", "pl.dotplot"]
+)
 def get_celltype_marker(adata:anndata.AnnData,
                             clustertype:str='leiden',
                             log2fc_min:int=2,scores_type='scores',
@@ -313,9 +361,10 @@ def get_celltype_marker(adata:anndata.AnnData,
         print('...get cell type marker')
         celltypes = sorted(adata.obs[clustertype].unique())
         cell_marker_dict={}
-        if rank==True and 'rank_genes_groups' not in adata.uns.keys():
-            sc.tl.rank_genes_groups(adata, clustertype, method=method)
-        elif rank==True and 'rank_genes_groups' in adata.uns.keys():
+        if rank==True and key not in adata.uns.keys():
+            sc.tl.rank_genes_groups(adata, clustertype, method=method, key_added=key)
+        elif rank==True and key in adata.uns.keys():
+            print(r"key already in adata.uns!")
             pass
         for celltype in celltypes:
             degs = sc.get.rank_genes_groups_df(adata, group=celltype, key=key, log2fc_min=log2fc_min, 
@@ -350,6 +399,28 @@ def get_celltype_marker(adata:anndata.AnnData,
 
 
 
+@register_function(
+    aliases=["单细胞注释", "pySCSA", "cell_annotation", "细胞类型注释", "自动注释"],
+    category="single",
+    description="Automated cell type annotation using SCSA (Single Cell Signature Analysis) with multiple databases",
+    examples=[
+        "# Basic SCSA annotation with CellMarker database",
+        "scsa = ov.single.pySCSA(adata, foldchange=1.5, pvalue=0.01,",
+        "                        celltype='normal', target='cellmarker')",
+        "anno = scsa.cell_anno(clustertype='leiden', cluster='all')",
+        "scsa.cell_auto_anno(adata, key='scsa_celltype_cellmarker')",
+        "# Using PanglaoDB database",
+        "scsa = ov.single.pySCSA(adata, target='panglaodb', tissue='All')",
+        "anno = scsa.cell_anno(clustertype='leiden', cluster='all')",
+        "# Cancer cell annotation",
+        "scsa = ov.single.pySCSA(adata, celltype='cancer', target='cancersea')",
+        "anno = scsa.cell_anno(clustertype='leiden', cluster='all')",
+        "# Specific tissue annotation",
+        "scsa = ov.single.pySCSA(adata, tissue='Blood', target='cellmarker')",
+        "available_tissues = scsa.get_model_tissue()"
+    ],
+    related=["single.scanpy_cellanno_from_dict", "single.get_celltype_marker", "utils.embedding"]
+)
 class pySCSA(object):
 
     def __init__(self,adata:anndata.AnnData,
@@ -496,18 +567,24 @@ class pySCSA(object):
     
     def cell_anno_print(self)->None:
         r"""Print the annotation result.
-        
+
         Returns:
             None
         """
         for i in set(self.result['Cluster']):
             test=self.result.loc[self.result['Cluster']==i].iloc[:2]
-            if test.iloc[0]['Z-score']>test.iloc[1]['Z-score']*2:
-                print('Nice:Cluster:{}\tCell_type:{}\tZ-score:{}'.format(i,test.iloc[0]['Cell Type'],
+            if len(test) >= 2 and pd.notna(test.iloc[0]['Z-score']) and pd.notna(test.iloc[1]['Z-score']):
+                if test.iloc[0]['Z-score']>test.iloc[1]['Z-score']*2:
+                    print('Nice:Cluster:{}\tCell_type:{}\tZ-score:{}'.format(i,test.iloc[0]['Cell Type'],
+                                                                np.around(test.iloc[0]['Z-score'],3)))
+                else:
+                    print('Cluster:{}\tCell_type:{}\tZ-score:{}'.format(i,('|').join(test['Cell Type'].values.tolist()),
+                                                                ('|').join(np.around(test['Z-score'].values,3).astype(str).tolist())))
+            elif len(test) >= 1 and pd.notna(test.iloc[0]['Z-score']):
+                print('Cluster:{}\tCell_type:{}\tZ-score:{}'.format(i,test.iloc[0]['Cell Type'],
                                                             np.around(test.iloc[0]['Z-score'],3)))
             else:
-                print('Cluster:{}\tCell_type:{}\tZ-score:{}'.format(i,('|').join(test['Cell Type'].values.tolist()),
-                                                            ('|').join(np.around(test['Z-score'].values,3).astype(str).tolist())))
+                print('Cluster:{}\tCell_type:{}\tZ-score:{}'.format(i,'Unknown','NaN'))
 
     def cell_auto_anno(self,adata:anndata.AnnData,
                        clustertype:str='leiden',key='scsa_celltype')->None:
