@@ -2,12 +2,13 @@ import pandas as pd
 from sklearn.linear_model import Ridge
 import numpy as np
 from sklearn.model_selection import train_test_split
-from sklearn.metrics import mean_squared_error, mean_absolute_error, r2_score
+from sklearn.metrics import mean_squared_error, root_mean_squared_error, mean_absolute_error, r2_score
 import scanpy as sc
 import anndata
 import matplotlib.pyplot as plt
 from sklearn.preprocessing import MinMaxScaler
 import torch
+from ..utils.registry import register_function
 
 class PyTorchRidge: # 保持不变， 不需要修改
     def __init__(self, alpha=1.0, device='cpu'):
@@ -62,6 +63,25 @@ def augment_gene_expression_noise(X_train, noise_std=0.01):
     noise = np.random.normal(0, noise_std, size=X_train.shape)
     return X_train + noise
 
+@register_function(
+    aliases=["细胞命运预测", "Fate", "time_fate_kernel", "TimeFateKernel", "时间命运核"],
+    category="single",
+    description="TimeFateKernel: Adaptive ridge regression model to identify timing-associated genes in single-cell pseudotime analysis",
+    examples=[
+        "# Initialize Fate analysis",
+        "fate_obj = ov.single.Fate(adata, pseudotime='palantir_pseudotime')",
+        "# Initialize model and run ATR",
+        "fate_obj.model_init()",
+        "fate_obj.ATR(stop=500)",
+        "# Fit the model",
+        "result = fate_obj.model_fit()",
+        "# Plot filtering results",
+        "fig, ax = fate_obj.plot_filtering()",
+        "# Calculate lineage scores",
+        "fate_obj.lineage_score(cluster_key='leiden', lineage=['0', '1'])"
+    ],
+    related=["pp.leiden", "utils.plot_heatmap", "pl.embedding"]
+)
 class Fate(object):
 
     def __init__(self,adata:anndata.AnnData,pseudotime:str):
@@ -133,7 +153,7 @@ class Fate(object):
 
         # 计算均方误差（MSE）
         mse = mean_squared_error(y_test, y_pred)
-        rmse = mean_squared_error(y_test, y_pred, squared=False)
+        rmse = root_mean_squared_error(y_test, y_pred)
         mae = mean_absolute_error(y_test, y_pred)
         r2 = r2_score(y_test, y_pred)
         self.raw_mse=mse
@@ -245,7 +265,7 @@ class Fate(object):
 
             # 计算均方误差（MSE）
             #mse = mean_squared_error(y_test, y_pred)
-            #rmse = mean_squared_error(y_test, y_pred, squared=False)
+            #rmse = root_mean_squared_error(y_test, y_pred)
             #mae = mean_absolute_error(y_test, y_pred)
             r2 = r2_score(y_test, y_pred)
             r2_li.append(r2)
@@ -321,7 +341,7 @@ class Fate(object):
 
         # 计算均方误差（MSE）
         mse = mean_squared_error(y_test, y_pred)
-        rmse = mean_squared_error(y_test, y_pred, squared=False)
+        rmse = root_mean_squared_error(y_test, y_pred)
         mae = mean_absolute_error(y_test, y_pred)
         self.filter_mse=mse
         self.filter_rmse=rmse
@@ -371,7 +391,7 @@ class Fate(object):
             import mellon
         except:
             print("Please install mellon package first using ``pip install mellon``")
-        from ..externel.palantir.utils import run_diffusion_maps
+        from ..external.palantir.utils import run_diffusion_maps
         run_diffusion_maps(self.adata,n_components=n_components,knn=knn,alpha=alpha,seed=seed,
                            pca_key=pca_key,kernel_key=kernel_key,sim_key=sim_key,
                            eigval_key=eigval_key,eigvec_key=eigvec_key)
@@ -389,7 +409,7 @@ class Fate(object):
                     expression_key: str = "MAGIC_imputed_data",
                     distances_key: str = "distances",
                     ):
-        from ..externel.palantir.utils import run_low_density_variability,run_local_variability
+        from ..external.palantir.utils import run_low_density_variability,run_local_variability
 
         if localvar_key not in self.adata.layers.keys():
             print("Run low_density first")
@@ -659,9 +679,9 @@ class Fate(object):
         ax.spines['bottom'].set_visible(True)
         ax.spines['left'].set_visible(True)
 
-        from sklearn.metrics import mean_squared_error, mean_absolute_error, r2_score
+        from sklearn.metrics import mean_squared_error, root_mean_squared_error, mean_absolute_error, r2_score
         mse = mean_squared_error(y_test, y_pred)
-        rmse = mean_squared_error(y_test, y_pred, squared=False)
+        rmse = root_mean_squared_error(y_test, y_pred)
         mae = mean_absolute_error(y_test, y_pred)
         r2 = r2_score(y_test, y_pred)
         ax.text(1,legend_loc[0],'$r^2={:.2}$'.format(r2),fontsize=fontsize+1,horizontalalignment='right')
@@ -1036,7 +1056,7 @@ def mellon_density(adata,
             import mellon
         except:
             print("Please install mellon package first using ``pip install mellon``")
-        from ..externel.palantir.utils import run_diffusion_maps
+        from ..external.palantir.utils import run_diffusion_maps
         run_diffusion_maps(adata,n_components=n_components,knn=knn,alpha=alpha,seed=seed,
                            pca_key=pca_key,kernel_key=kernel_key,sim_key=sim_key,
                            eigval_key=eigval_key,eigvec_key=eigvec_key)
