@@ -11,14 +11,43 @@ from tqdm import tqdm
 import sys
 from ..utils._neighboors import update_rep, eff_n_jobs,neighbors,W_from_rep
 from .._settings import add_reference
+from ..utils.registry import register_function
 
 from ._cosg import cosg
-from ..externel.palantir.plot import plot_palantir_results,plot_branch_selection,plot_gene_trends
-from ..externel.palantir.utils import run_diffusion_maps,determine_multiscale_space,run_magic_imputation
-from ..externel.palantir.core import run_palantir
-from ..externel.palantir.presults import select_branch_cells,compute_gene_trends
+from ..external.palantir.plot import plot_palantir_results,plot_branch_selection,plot_gene_trends
+from ..external.palantir.utils import run_diffusion_maps,determine_multiscale_space,run_magic_imputation
+from ..external.palantir.core import run_palantir
+from ..external.palantir.presults import select_branch_cells,compute_gene_trends
 
 
+@register_function(
+    aliases=["轨迹推断", "TrajInfer", "trajectory_inference", "轨迹分析", "发育轨迹"],
+    category="single",
+    description="Comprehensive trajectory inference for single-cell data using multiple algorithms including Palantir, diffusion maps, and Slingshot",
+    examples=[
+        "# Initialize TrajInfer",
+        "traj = ov.single.TrajInfer(adata, basis='X_umap', groupby='clusters',",
+        "                           use_rep='X_pca', n_comps=50)",
+        "# Set origin and terminal cells",
+        "traj.set_origin_cells('stem_cells')",
+        "traj.set_terminal_cells(['differentiated_A', 'differentiated_B'])",
+        "# Diffusion map trajectory inference",
+        "traj.inference(method='diffusion_map')",
+        "# Slingshot trajectory inference", 
+        "traj.inference(method='slingshot', num_epochs=1)",
+        "# Palantir trajectory inference",
+        "traj.inference(method='palantir', num_waypoints=500)",
+        "# Visualize Palantir results",
+        "traj.palantir_plot_pseudotime(embedding_basis='X_umap', cmap='RdBu_r')",
+        "# Calculate branch probabilities",
+        "traj.palantir_cal_branch(eps=0)",
+        "# Compute gene expression trends",
+        "gene_trends = traj.palantir_cal_gene_trends(layers='MAGIC_imputed_data')",
+        "# Plot gene trends along trajectories",
+        "traj.palantir_plot_gene_trends(['gene1', 'gene2', 'gene3'])"
+    ],
+    related=["utils.cal_paga", "utils.plot_paga", "pp.neighbors", "external.palantir"]
+)
 class TrajInfer(object):
     r"""Trajectory inference class for single-cell data analysis.
     
@@ -277,7 +306,7 @@ def calc_force_directed_layout(
     """
     G = construct_graph(W)
     try:
-        from ..externel.forcedirect2.forceatlas2 import ForceAtlas2
+        from ..external.forcedirect2.forceatlas2 import ForceAtlas2
         
         # 如果请求编译并且fa2util模块需要编译
         if compile_cython:
@@ -287,7 +316,7 @@ def calc_force_directed_layout(
                 import importlib.util
                 
                 # 检查是否已经编译
-                force_dir = os.path.dirname(os.path.abspath(importlib.util.find_spec("omicverse.externel.forcedirect2.forceatlas2").origin))
+                force_dir = os.path.dirname(os.path.abspath(importlib.util.find_spec("omicverse.external.forcedirect2.forceatlas2").origin))
                 compiled_module_exists = any(f.startswith('fa2util.') and (f.endswith('.so') or f.endswith('.pyd')) 
                                            for f in os.listdir(force_dir))
                 
