@@ -2,12 +2,34 @@ import scanpy as sc
 import os
 import torch
 import numpy as np
-from ..externel.gaston import neural_net,process_NN_output,dp_related,cluster_plotting
+from ..external.gaston import neural_net,process_NN_output,dp_related,cluster_plotting
 from scipy.sparse import issparse, csr_matrix
 import matplotlib.pyplot as plt
 from tqdm import tqdm
 from .._settings import add_reference
+from ..utils.registry import register_function
 
+@register_function(
+    aliases=["GASTON空间深度分析", "GASTON", "spatial_depth_analysis", "空间深度建模", "组织空间组织"],
+    category="space",
+    description="GASTON spatial depth estimation and clustering using neural networks",
+    examples=[
+        "# Basic GASTON analysis",
+        "gaston = ov.space.GASTON(adata)",
+        "counts, coords, genes = gaston.get_gaston_input()",
+        "features = gaston.get_top_pearson_residuals(num_dims=10)",
+        "gaston.load_rescale(features)",
+        "gaston.train(num_epochs=5000)",
+        "# Get results",
+        "model, features, coords = gaston.get_best_model()",
+        "isodepth, labels = gaston.cal_iso_depth(num_domains=5)",
+        "# With RGB features",
+        "counts, coords, genes, rgb = gaston.get_gaston_input(get_rgb=True)",
+        "# Plotting results",
+        "gaston.plot_gaston_scatter()"
+    ],
+    related=["space.clusters", "space.svg", "space.pySTAGATE"]
+)
 class GASTON(object):
     r"""GASTON spatial depth estimation and clustering.
     
@@ -166,7 +188,7 @@ class GASTON(object):
             sc.pp.normalize_total(adata, inplace=False)["X"]
         )
 
-        theta=np.Inf
+        theta=np.inf
         sc.experimental.pp.normalize_pearson_residuals(adata, clip=clip, theta=theta)
         sc.pp.pca(adata, n_comps=num_dims)
         A=adata.obsm['X_pca']
@@ -191,7 +213,7 @@ class GASTON(object):
             - Required before calling train()
         """
         S=self.adata.obsm['spatial']
-        from ..externel.gaston.neural_net import load_rescale_input_data
+        from ..external.gaston.neural_net import load_rescale_input_data
         S_torch, A_torch = load_rescale_input_data(S,A)
         self.S_torch=S_torch
         self.A_torch=A_torch
@@ -275,7 +297,7 @@ class GASTON(object):
         self.model=gaston_model
         self.A=A
         self.S=S
-        from ..externel.gaston import model_selection
+        from ..external.gaston import model_selection
         model_selection.plot_ll_curve(gaston_model, A, S, max_domain_num=max_domain_num, start_from=start_from)
         return gaston_model, A, S
     
@@ -343,7 +365,7 @@ class GASTON(object):
             - Colors represent depth values
             - Useful for understanding tissue organization
         """
-        from ..externel.gaston.cluster_plotting import plot_isodepth
+        from ..external.gaston.cluster_plotting import plot_isodepth
         rotate=np.radians(rotate_angle)
         plot_isodepth(self.gaston_isodepth,self.S, 
                       self.model, figsize=figsize, streamlines=show_streamlines, 
@@ -485,7 +507,7 @@ class GASTON(object):
         """
         rotate = np.radians(rotate_angle)
 
-        from ..externel.gaston.restrict_spots import restrict_spots
+        from ..external.gaston.restrict_spots import restrict_spots
         counts_mat_restrict, coords_mat_restrict, gaston_isodepth_restrict, gaston_labels_restrict, S_restrict=restrict_spots(
                                                                     self.adata.X, 
                                                                     self.adata.obsm['spatial'], 
@@ -531,13 +553,13 @@ class GASTON(object):
             - Results can be used for downstream analysis
         """
         self.umi_thresh=umi_thresh
-        from ..externel.gaston.filter_genes import filter_genes
+        from ..external.gaston.filter_genes import filter_genes
         if issparse(self.counts_mat_restrict):
             counts_mat_restrict=self.counts_mat_restrict.toarray()
         else:
             counts_mat_restrict=self.counts_mat_restrict
         idx_kept, gene_labels_idx = filter_genes(counts_mat_restrict, self.adata.var.index.to_numpy(), 
-                                                umi_thresh = umi_thresh,exclude_prefix=exclude_prefix)
+                                                umi_threshold = umi_thresh,exclude_prefix=exclude_prefix)
         self.gene_labels_idx=gene_labels_idx
         self.idx_kept=idx_kept
         return idx_kept, gene_labels_idx
@@ -567,7 +589,7 @@ class GASTON(object):
             - Identifies expression breakpoints
             - Useful for finding spatial transition points
         """
-        from ..externel.gaston.segmented_fit import pw_linear_fit
+        from ..external.gaston.segmented_fit import pw_linear_fit
         if issparse(self.counts_mat_restrict):
             counts_mat_restrict=self.counts_mat_restrict.toarray()
         else:
@@ -611,8 +633,8 @@ class GASTON(object):
             - Can incorporate cell type information
             - Useful for trajectory analysis
         """
-        from ..externel.gaston.binning_and_plotting import bin_data
-        from ..externel.gaston.spatial_gene_classification import get_discont_genes, get_cont_genes
+        from ..external.gaston.binning_and_plotting import bin_data
+        from ..external.gaston.spatial_gene_classification import get_discont_genes, get_cont_genes
         if issparse(self.counts_mat_restrict):
             counts_mat_restrict=self.counts_mat_restrict.toarray()
         else:
@@ -661,22 +683,41 @@ class GASTON(object):
         slope_mat, intercept_mat, _, _ = self.pw_fit_dict['all_cell_types']
 
         gene_list = list(self.binning_output['gene_labels_idx']) # 获取基因列表
-        adata2=adata2[:,gene_list]
+        
+        # Filter gene_list to only include genes that are actually in adata2 and gene_labels_idx
+        valid_genes = []
+        for gene_name in gene_list:
+            if gene_name in adata2.var_names and gene_name in self.gene_labels_idx:
+                valid_genes.append(gene_name)
+        
+        # Update adata2 to only include valid genes
+        adata2=adata2[:,valid_genes]
         all_gene_outputs = []
 
-        for gene_name in tqdm(gene_list):
-            if gene_name in self.binning_output['gene_labels_idx']:
-                gene_index = np.where(self.gene_labels_idx == gene_name)[0]
+        for gene_name in tqdm(valid_genes):
+            gene_index = np.where(self.gene_labels_idx == gene_name)[0]
+            
+            if len(gene_index) == 0:
+                # This shouldn't happen with our validation above, but keep as safety check
+                continue
+                
+            outputs = np.zeros(self.gaston_isodepth_restrict.shape[0])
+            for i in range(self.gaston_isodepth_restrict.shape[0]):
+                dom = int(self.gaston_labels_restrict[i])
+                slope = slope_mat[gene_index, dom]
+                intercept = intercept_mat[gene_index, dom]
+                outputs[i] = np.log(offset) + intercept + slope * self.gaston_isodepth_restrict[i]
 
-                outputs = np.zeros(self.gaston_isodepth_restrict.shape[0])
-                for i in range(self.gaston_isodepth_restrict.shape[0]):
-                    dom = int(self.gaston_labels_restrict[i])
-                    slope = slope_mat[gene_index, dom]
-                    intercept = intercept_mat[gene_index, dom]
-                    outputs[i] = np.log(offset) + intercept + slope * self.gaston_isodepth_restrict[i]
-
-                all_gene_outputs.append(outputs)
+            all_gene_outputs.append(outputs)
 
+        # Final validation
+        if len(all_gene_outputs) == 0:
+            raise ValueError("No valid gene outputs generated. Check gene filtering and binning steps.")
+            
+        if len(all_gene_outputs) != adata2.n_vars:
+            raise ValueError(f"Gene output count ({len(all_gene_outputs)}) doesn't match adata2 gene count ({adata2.n_vars}). "
+                           f"Valid genes found: {len(valid_genes)}")
+            
         sparse_output_matrix = csr_matrix(all_gene_outputs)
         adata2.layers['GASTON_ReX']=sparse_output_matrix.T
         return adata2
@@ -728,7 +769,7 @@ class GASTON(object):
 
         # display log CPM (if you want to do CP500, set offset=500)
         #offset=10**6
-        from ..externel.gaston.binning_and_plotting import plot_gene_pwlinear
+        from ..external.gaston.binning_and_plotting import plot_gene_pwlinear
         plot_gene_pwlinear(gene_name, self.pw_fit_dict, self.gaston_labels_restrict, self.gaston_isodepth_restrict, 
                            self.binning_output, cell_type_list=cell_type_list, pt_size=pt_size, colors=domain_colors, 
                            linear_fit=linear_fit, ticksize=ticksize, figsize=figsize, offset=offset, lw=lw,
@@ -766,7 +807,7 @@ class GASTON(object):
             - Useful for examining expression patterns
         """
         rotate=np.radians(rotate_angle)
-        from ..externel.gaston.binning_and_plotting import plot_gene_raw
+        from ..external.gaston.binning_and_plotting import plot_gene_raw
         if issparse(self.counts_mat_restrict):
             counts_mat_restrict=self.counts_mat_restrict.toarray()
         else:
@@ -805,7 +846,7 @@ class GASTON(object):
             - Useful for detailed pattern analysis
         """
         rotate=np.radians(rotate_angle)
-        from ..externel.gaston.binning_and_plotting import plot_gene_function
+        from ..external.gaston.binning_and_plotting import plot_gene_function
 
         plot_gene_function(gene_name, self.S_restrict, self.pw_fit_dict, 
                            self.gaston_labels_restrict, self.gaston_isodepth_restrict, 
