@@ -1,11 +1,34 @@
 r"""Module providing encapsulation of STT for spatial transition tensor analysis."""
-from ..externel.STT import tl,pl
+from ..external.STT import tl,pl
 from typing import Any
 import scanpy as sc
 import numpy as np
 import pandas as pd
 from .._settings import add_reference
+from ..utils.registry import register_function
 
+@register_function(
+    aliases=["STT空间转换张量", "STT", "spatial_transition_tensor", "空间转换分析", "空间动力学"],
+    category="space",
+    description="Spatial Transition Tensor analysis for modeling spatial dynamics and cell state transitions",
+    examples=[
+        "# Basic STT analysis",
+        "stt = ov.space.STT(adata, spatial_loc='spatial',",
+        "                   region='tissue_region')",
+        "# Stage estimation",
+        "stt.stage_estimate()",
+        "# Train STT model",
+        "stt.train(n_states=10)",
+        "# Vector field analysis",
+        "stt.vector_field()",
+        "# Spatial dynamics",
+        "stt.cal_pseudotime()",
+        "# Custom parameters",
+        "stt = ov.space.STT(adata, spatial_loc='xy_loc',",
+        "                   region='Region')"
+    ],
+    related=["space.svg", "space.clusters", "external.scvelo"]
+)
 class STT(object):
     r"""Spatial Transition Tensor (STT) analysis class.
     
@@ -180,7 +203,7 @@ class STT(object):
         self.adata_aggr=adata_aggr
         add_reference(self.adata,'STT','spatial transition tensor with STT')
 
-    def compute_pathway(self,pathway_dict):
+    def compute_pathway(self,pathway_dict,n_components=10):
         r"""Compute spatial pathways for cell transitions.
         
         This method identifies and computes transition pathways between cell states
@@ -206,7 +229,7 @@ class STT(object):
             - Results can be visualized using plot_pathway()
             - Considers both spatial and transcriptional information
         """
-        return tl.compute_pathway(self.adata,self.adata_aggr,pathway_dict)
+        return tl.compute_pathway(self.adata,self.adata_aggr,pathway_dict,n_components=n_components)
 
     def plot_pathway(self,label_fontsize=20,**kwargs):
         r"""Plot spatial transition pathways.
