@@ -8,7 +8,28 @@ from matplotlib.cm import ScalarMappable
 import scanpy as sc
 import torch
 import torch.nn.functional as F
+from ..utils.registry import register_function
 
+@register_function(
+    aliases=["裁剪空间数据", "crop_space_visium", "crop_visium", "空间数据裁剪", "Visium裁剪"],
+    category="space",
+    description="Crop Visium spatial transcriptomics data to focus on specific region of interest",
+    examples=[
+        "# Basic cropping",
+        "adata_cropped = ov.space.crop_space_visium(",
+        "    adata, crop_loc=(500, 500), crop_area=(1000, 1000),",
+        "    library_id='V1_Human_Brain', scale=1.0)",
+        "# Small region cropping",
+        "adata_roi = ov.space.crop_space_visium(",
+        "    adata, crop_loc=(0, 0), crop_area=(800, 600),",
+        "    library_id=list(adata.uns['spatial'].keys())[0], scale=1)",
+        "# High resolution cropping",
+        "adata_hires = ov.space.crop_space_visium(",
+        "    adata, crop_loc=(200, 200), crop_area=(1500, 1500),",
+        "    library_id='sample_1', scale=2, res='hires')"
+    ],
+    related=["space.rotate_space_visium", "space.map_spatial_auto"]
+)
 def crop_space_visium(adata, crop_loc, crop_area,
                      library_id, scale, spatial_key='spatial', res='hires'):
     """
@@ -57,6 +78,9 @@ def crop_space_visium(adata, crop_loc, crop_area,
         ...     scale=1.0
         ... )
     """
+    # Configure dask to use query-planning before importing squidpy
+    import dask
+    dask.config.set({"dataframe.query-planning": True})
     import squidpy as sq
     adata1 = adata.copy()
     img = sq.im.ImageContainer(
@@ -75,6 +99,27 @@ def crop_space_visium(adata, crop_loc, crop_area,
 import numpy as np
 import scipy.ndimage  # 导入 scipy.ndimage 库用于图像旋转
 
+@register_function(
+    aliases=["旋转空间数据", "rotate_space_visium", "rotate_visium", "空间数据旋转", "Visium旋转"],
+    category="space",
+    description="Rotate Visium spatial data image and coordinates by specified angle",
+    examples=[
+        "# Basic rotation",
+        "adata_rotated = ov.space.rotate_space_visium(",
+        "    adata, angle=45, library_id='V1_Human_Brain')",
+        "# Custom center rotation",
+        "adata_rotated = ov.space.rotate_space_visium(",
+        "    adata, angle=90, center=(100, 100),",
+        "    library_id='sample_1', interpolation_order=3)",
+        "# Precise rotation with high-quality interpolation",
+        "adata_rotated = ov.space.rotate_space_visium(",
+        "    adata, angle=30, res='hires',",
+        "    library_id=library_id, interpolation_order=1)",
+        "# Apply spatial mapping after rotation",
+        "ov.space.map_spatial_auto(adata_rotated, method='phase')"
+    ],
+    related=["space.crop_space_visium", "space.map_spatial_auto", "space.map_spatial_manual"]
+)
 def rotate_space_visium(
     adata,
     angle,
@@ -533,6 +578,23 @@ def _map_spatial_img(adata_rotated, color=None,
     return adata_rotated
 
 
+@register_function(
+    aliases=["自动空间映射", "map_spatial_auto", "auto_spatial_mapping", "空间自动映射", "自动对齐"],
+    category="space",
+    description="Automatically map and align spatial transcriptomics data with tissue image",
+    examples=[
+        "# Basic auto mapping with phase correlation",
+        "ov.space.map_spatial_auto(adata_rotated, method='phase')",
+        "# Feature-based alignment",
+        "ov.space.map_spatial_auto(adata_rotated, method='feature')",
+        "# Hybrid alignment approach",
+        "ov.space.map_spatial_auto(adata_rotated, method='hybrid')",
+        "# After rotation, apply auto mapping",
+        "adata_rotated = ov.space.rotate_space_visium(adata, angle=45)",
+        "ov.space.map_spatial_auto(adata_rotated)"
+    ],
+    related=["space.map_spatial_manual", "space.rotate_space_visium", "space.crop_space_visium"]
+)
 def map_spatial_auto(adata_rotated, method='phase'):
     """
     Automatically map and align spatial transcriptomics data.
@@ -667,6 +729,22 @@ def map_spatial_auto(adata_rotated, method='phase'):
     )
     return adata_rotated
 
+@register_function(
+    aliases=["手动空间映射", "map_spatial_manual", "manual_spatial_mapping", "空间手动映射", "手动对齐"],
+    category="space",
+    description="Manually adjust spatial transcriptomics data alignment with specified offset",
+    examples=[
+        "# Apply manual offset",
+        "ov.space.map_spatial_manual(adata_rotated, offset=(-1500, 1000))",
+        "# Fine-tune alignment",
+        "ov.space.map_spatial_manual(adata_rotated, offset=(50, -30))",
+        "# Large adjustment",
+        "ov.space.map_spatial_manual(adata_rotated, offset=(-500, 200))",
+        "# Access manually aligned coordinates",
+        "manual_coords = adata_rotated.obsm['spatial1']"
+    ],
+    related=["space.map_spatial_auto", "space.rotate_space_visium", "space.crop_space_visium"]
+)
 def map_spatial_manual(
         adata_rotated,
         offset,
@@ -711,5 +789,231 @@ def map_spatial_manual(
             adata_rotated.obsm['spatial'][:, 1].mean() / (adata_rotated.obsm['spatial'][:, 1].max())
     )
     return adata_rotated
-    
 
+
+@register_function(
+    aliases=["读取Visium数据", "read_visium_10x", "load_visium", "Visium数据读取", "空间数据载入"],
+    category="space",
+    description="Read and process 10x Visium spatial transcriptomics data",
+    examples=[
+        "# Basic Visium data loading",
+        "adata = ov.space.read_visium_10x(adata_path)",
+        "# With custom parameters",
+        "adata = ov.space.read_visium_10x(adata_path, genome='GRCh38')",
+        "# Load with filtering",
+        "adata = ov.space.read_visium_10x(path, min_counts=100)"
+    ],
+    related=["space.svg", "space.crop_space_visium"]
+)
+def read_visium_10x(
+        adata,
+        **kwargs,
+):
+    from ..external.bin2cell import read_visium
+    adata = read_visium(adata, **kwargs)
+    adata.var_names_make_unique()
+    return adata
+
+
+@register_function(
+    aliases=["Visium细胞分割HE", "visium_10x_hd_cellpose_he", "cellpose_he", "HE图像分割", "细胞核分割"],
+    category="space",
+    description="Cell segmentation on H&E images for high-resolution Visium data using CellPose",
+    examples=[
+        "# Basic H&E segmentation",
+        "adata = ov.space.visium_10x_hd_cellpose_he(adata, mpp=0.3)",
+        "# Custom parameters",
+        "adata = ov.space.visium_10x_hd_cellpose_he(adata, mpp=0.5,",
+        "                                           prob_thresh=0.1,",
+        "                                           flow_threshold=0.3)",
+        "# GPU acceleration",
+        "adata = ov.space.visium_10x_hd_cellpose_he(adata, gpu=True,",
+        "                                           buffer=200)"
+    ],
+    related=["space.visium_10x_hd_cellpose_gex", "space.bin2cell"]
+)
+def visium_10x_hd_cellpose_he(
+        adata,
+        mpp=0.3,
+        he_save_path="stardist/he_colon.tiff",
+        prob_thresh=0,
+        flow_threshold=0.2,
+        gpu=True,
+        buffer=150,
+        backend='tifffile',
+        **kwargs,
+):
+    """
+    Convert Visium 10x data to cell-level data.
+    
+    """
+    from ..external.bin2cell import destripe, scaled_he_image, stardist, insert_labels
+
+    if not os.path.exists(he_save_path):    
+        destripe(adata)
+        scaled_he_image(adata, mpp=mpp, buffer=buffer, save_path=he_save_path,
+                        backend=backend)
+    else:
+        print(f"he_save_path {he_save_path} already exists, skipping destripe and scaled_he_image")
+    stardist(image_path=he_save_path    , 
+             labels_npz_path=he_save_path.replace(".tiff", ".npz"), 
+             stardist_model="2D_versatile_he", 
+             prob_thresh=prob_thresh,
+             flow_threshold=flow_threshold,
+             gpu=gpu,
+             **kwargs,
+            )
+    insert_labels(adata, 
+                  labels_npz_path=he_save_path.replace(".tiff", ".npz"), 
+                  basis="spatial", 
+                  spatial_key=f"spatial_cropped_{buffer}_buffer",
+                  mpp=mpp, 
+                  labels_key="labels_he"
+                 )
+    
+@register_function(
+    aliases=["Visium细胞扩展", "visium_10x_hd_cellpose_expand", "cellpose_expand", "细胞标签扩展", "空间扩展"],
+    category="space",
+    description="Expand cell segmentation labels to nearby bins for improved coverage",
+    examples=[
+        "# Basic label expansion",
+        "adata = ov.space.visium_10x_hd_cellpose_expand(adata,",
+        "                                               max_bin_distance=4)",
+        "# Custom parameters",
+        "adata = ov.space.visium_10x_hd_cellpose_expand(adata,",
+        "                                               max_bin_distance=6,",
+        "                                               labels_key='cellpose_labels')",
+        "# Different expansion keys",
+        "adata = ov.space.visium_10x_hd_cellpose_expand(adata,",
+        "                                               expanded_labels_key='expanded_cells')"
+    ],
+    related=["space.visium_10x_hd_cellpose_he", "space.visium_10x_hd_cellpose_gex"]
+)
+def visium_10x_hd_cellpose_expand(
+        adata,
+        max_bin_distance=4,
+        labels_key="labels_he",
+        expanded_labels_key="labels_he_expanded",
+        **kwargs,
+):
+    from ..external.bin2cell import expand_labels
+    expand_labels(adata, 
+                  labels_key=labels_key, 
+                  expanded_labels_key=expanded_labels_key,
+                  max_bin_distance=max_bin_distance,
+                  **kwargs,
+                 )
+    
+@register_function(
+    aliases=["Visium细胞基因表达", "visium_10x_hd_cellpose_gex", "cellpose_gex", "细胞基因表达映射", "细胞水平表达"],
+    category="space",
+    description="Map gene expression to cell level using CellPose segmentation",
+    examples=[
+        "# Basic gene expression mapping",
+        "adata = ov.space.visium_10x_hd_cellpose_gex(adata)",
+        "# Custom parameters",
+        "adata = ov.space.visium_10x_hd_cellpose_gex(adata,",
+        "                                            obs_key='total_counts',",
+        "                                            mpp=0.5, sigma=3)",
+        "# With log transformation",
+        "adata = ov.space.visium_10x_hd_cellpose_gex(adata, log1p=True,",
+        "                                            prob_thresh=0.1)"
+    ],
+    related=["space.visium_10x_hd_cellpose_he", "space.bin2cell"]
+)
+def visium_10x_hd_cellpose_gex(
+        adata,
+        obs_key="n_counts_adjusted",
+        log1p=False,
+        mpp=0.3,
+        sigma=5,
+        gex_save_path="stardist/gex_colon.tiff",
+        prob_thresh=0.01,
+        nms_thresh=0.1,
+        gpu=True,
+        buffer=150,
+        **kwargs,
+):
+    from ..external.bin2cell import grid_image, stardist, insert_labels,destripe
+    #if gex_save_path's file exist, jump grid_image to stardist
+    if obs_key not in adata.obs.keys():
+        destripe(adata)
+    if not os.path.exists(gex_save_path):
+        grid_image(adata, obs_key, log1p=log1p,
+                mpp=mpp, sigma=sigma, save_path=gex_save_path)
+    else:
+        print(f"gex_save_path {gex_save_path} already exists, skipping grid_image")
+    stardist(image_path=gex_save_path, 
+             labels_npz_path=gex_save_path.replace(".tiff", ".npz"), 
+             stardist_model="2D_versatile_fluo", 
+             prob_thresh=prob_thresh, 
+             nms_thresh=nms_thresh,gpu=gpu,
+             **kwargs,
+            )
+    insert_labels(adata, 
+                  labels_npz_path=gex_save_path.replace(".tiff", ".npz"), 
+                  basis="spatial", 
+                  spatial_key=f"spatial_cropped_{buffer}_buffer",
+                  mpp=mpp, 
+                  labels_key="labels_gex"
+                 )
+    
+@register_function(
+    aliases=["挂失次级标签", "salvage_secondary_labels", "rescue_labels", "标签救救", "次级标签恢复"],
+    category="space",
+    description="Salvage secondary labels by combining primary and secondary segmentation results",
+    examples=[
+        "# Basic label salvaging",
+        "ov.space.salvage_secondary_labels(adata, primary_label='labels_he',",
+        "                                   secondary_label='labels_gex')",
+        "# Custom label keys",
+        "ov.space.salvage_secondary_labels(adata, primary_label='cellpose_he',",
+        "                                   secondary_label='cellpose_gex',",
+        "                                   labels_key='combined_labels')",
+        "# Access combined labels",
+        "joint_labels = adata.obs['labels_joint']"
+    ],
+    related=["space.visium_10x_hd_cellpose_he", "space.visium_10x_hd_cellpose_gex", "space.bin2cell"]
+)
+def salvage_secondary_labels(
+        adata,
+        primary_label="labels_he", 
+        secondary_label="labels_gex",
+        labels_key="labels_joint",
+        **kwargs,
+):
+    from ..external.bin2cell import salvage_secondary_labels
+    salvage_secondary_labels(adata, primary_label=primary_label,
+                             secondary_label=secondary_label,
+                             labels_key=labels_key)  
+    
+    
+    
+@register_function(
+    aliases=["空间段到细胞", "bin2cell", "bin_to_cell", "空间细胞化", "段级到细胞级"],
+    category="space",
+    description="Convert spatial bins to single-cell resolution using segmentation labels",
+    examples=[
+        "# Basic bin to cell conversion",
+        "adata_cell = ov.space.bin2cell(adata, labels_key='labels_joint')",
+        "# Custom spatial keys",
+        "adata_cell = ov.space.bin2cell(adata, labels_key='segmentation',",
+        "                               spatial_keys=['spatial'])",
+        "# With diameter scaling",
+        "adata_cell = ov.space.bin2cell(adata, labels_key='cellpose_labels',",
+        "                               diameter_scale_factor=1.5)"
+    ],
+    related=["space.visium_10x_hd_cellpose_he", "space.visium_10x_hd_cellpose_gex"]
+)
+def bin2cell(
+        adata,
+        labels_key="labels_joint",
+        spatial_keys=["spatial"],
+        diameter_scale_factor=None,
+):
+    from ..external.bin2cell import bin_to_cell
+    return bin_to_cell(adata, labels_key=labels_key, 
+                spatial_keys=spatial_keys,diameter_scale_factor=diameter_scale_factor)
+    
+    
+    
