@@ -179,15 +179,37 @@ def get_sparse_from_igraph(graph, weight_attr=None):
     if weight_attr is None:
         weights = [1] * len(edges)
     else:
-        weights = graph.es[weight_attr]
+        # Convert to list to ensure proper extend behavior
+        try:
+            weights = list(graph.es[weight_attr])
+        except (KeyError, AttributeError):
+            # If weight attribute doesn't exist or is malformed, use default weights
+            weights = [1] * len(edges)
+
     if not graph.is_directed():
+        # Create a copy of the current weights before extending
+        current_weights = weights.copy() if hasattr(weights, 'copy') else list(weights)
         edges.extend([(v, u) for u, v in edges])
-        weights.extend(weights)
+        weights.extend(current_weights)
+
     shape = graph.vcount()
     shape = (shape, shape)
-    if len(edges) > 0:
-        return csr_matrix((weights, zip(*edges)), shape=shape)
+
+    # Check that we have valid edges and matching weights
+    if len(edges) > 0 and len(weights) == len(edges):
+        # Unpack edges into separate row and column index lists
+        try:
+            row_indices, col_indices = zip(*edges)
+            # Convert to lists to ensure compatibility with scipy
+            row_indices = list(row_indices)
+            col_indices = list(col_indices)
+            return csr_matrix((weights, (row_indices, col_indices)), shape=shape)
+        except (ValueError, TypeError) as e:
+            # If unpacking fails, return empty sparse matrix
+            print(f"Warning: Failed to unpack edges: {e}. Returning empty sparse matrix.")
+            return csr_matrix(shape)
     else:
+        # Return empty sparse matrix with the correct shape
         return csr_matrix(shape)
 
 
@@ -393,6 +415,23 @@ def default_basis(adata, **kwargs):
         raise ValueError("No basis specified.")
     return keys[-1] if len(keys) > 0 else None
 
+def _safe_categorical_to_str(adata, color_key):
+    """
+    Safely convert categorical values to string format for plotting.
+    
+    This function addresses pandas compatibility issues where direct assignment
+    to cat.categories is no longer allowed in newer pandas versions.
+    """
+    if color_key in adata.obs.columns and hasattr(adata.obs[color_key], 'cat'):
+        # Create a copy to avoid modifying the original data
+        adata_copy = adata.copy()
+        # Convert categorical values to string using cat.rename_categories
+        if hasattr(adata_copy.obs[color_key].cat, 'categories'):
+            str_categories = adata_copy.obs[color_key].cat.categories.astype(str)
+            adata_copy.obs[color_key] = adata_copy.obs[color_key].cat.rename_categories(str_categories)
+        return adata_copy
+    return adata
+
 def plot_paga(adata,
     basis=None,
     vkey="velocity",
@@ -441,8 +480,14 @@ def plot_paga(adata,
     if layout is not None:
         basis = None
     
+    # Fix for pandas compatibility: when using basis with categorical colors,
+    # scvelo may fail due to read-only categories in newer pandas versions
+    adata_processed = adata
+    if basis is not None and color is not None:
+        adata_processed = _safe_categorical_to_str(adata, color)
+    
     import scvelo as scv
-    return scv.pl.paga(adata,
+    return scv.pl.paga(adata_processed,
     basis=basis,
     vkey=vkey,
     color=color,
