@@ -4,6 +4,7 @@ import matplotlib.pyplot as plt
 import matplotlib as mpl
 import matplotlib.font_manager as fm
 import matplotlib.patches as mpatches
+from matplotlib import rcParams
 import random
 import scanpy as sc
 import networkx as nx
@@ -13,70 +14,85 @@ from sklearn.cluster import KMeans
 from scipy.spatial import ConvexHull
 import seaborn as sns
 from datetime import datetime,timedelta
-import pkg_resources
-from pkg_resources import DistributionNotFound, VersionConflict
 import tomli
 import os
+from typing import Union
 
 
 from datetime import datetime, timedelta
 import warnings
 import platform
 import os
+from .registry import register_function
 try:
     import torch  # Optional, used for GPU information
 except ImportError:  # pragma: no cover - optional dependency
     torch = None
 
-sc_color=[
- '#1F577B', '#A56BA7', '#E0A7C8', '#E069A6', '#941456', '#FCBC10', '#EF7B77', '#279AD7','#F0EEF0',
- '#EAEFC5', '#7CBB5F','#368650','#A499CC','#5E4D9A','#78C2ED','#866017', '#9F987F','#E0DFED',
- '#01A0A7', '#75C8CC', '#F0D7BC', '#D5B26C', '#D5DA48', '#B6B812', '#9DC3C3', '#A89C92', '#FEE00C', '#FEF2A1']
+# Global variable to control vector-friendly rasterization
+_vector_friendly = True
 
-red_color=['#F0C3C3','#E07370','#CB3E35','#A22E2A','#5A1713','#D3396D','#DBC3DC','#85539B','#5C2B80','#5C4694']
-green_color=['#91C79D','#8FC155','#56AB56','#2D5C33','#BBCD91','#6E944A','#A5C953','#3B4A25','#010000']
-orange_color=['#EFBD49','#D48F3E','#AC8A3E','#7D7237','#745228','#E1C085','#CEBC49','#EBE3A1','#6C6331','#8C9A48','#D7DE61']
-blue_color=['#347862','#6BBBA0','#81C0DD','#3E8CB1','#88C8D2','#52B3AD','#265B58','#B2B0D4','#5860A7','#312C6C']
-purple_color=['#823d86','#825b94','#bb98c6','#c69bc6','#a69ac9','#c5a6cc','#caadc4','#d1c3d4']
+sc_color=[
+ '#1F577B', '#A56BA7', '#E0A7C8', '#E069A6', '#941456', 
+ '#FCBC10', '#EF7B77', '#279AD7','#F0EEF0',
+ '#EAEFC5', '#7CBB5F','#368650','#A499CC','#5E4D9A',
+ '#78C2ED','#866017', '#9F987F','#E0DFED',
+ '#01A0A7', '#75C8CC', '#F0D7BC', '#D5B26C', '#D5DA48',
+ '#B6B812', '#9DC3C3', '#A89C92', '#FEE00C', '#FEF2A1']
+
+red_color=['#F0C3C3','#E07370','#CB3E35','#A22E2A','#5A1713',
+           '#D3396D','#8B0000', '#A52A2A', '#CD5C5C', '#DC143C' ]
+
+green_color=['#91C79D','#8FC155','#56AB56','#2D5C33','#BBCD91',
+             '#6E944A','#A5C953','#3B4A25','#010000']
+
+orange_color=['#EFBD49','#D48F3E','#AC8A3E','#7D7237','#745228',
+              '#E1C085','#CEBC49','#EBE3A1','#6C6331','#8C9A48','#D7DE61']
+
+blue_color=['#1F577B', '#279AD7', '#78C2ED', '#01A0A7', '#75C8CC', '#9DC3C3',
+            '#3E8CB1', '#52B3AD', '#265B58', '#5860A7', '#312C6C', '#4CC9F0']
+
+purple_color=['#823d86','#825b94','#bb98c6','#c69bc6','#a69ac9',
+              '#c5a6cc','#caadc4','#d1c3d4']
 
 #more beautiful colors
 # 28-color palettes with distinct neighboring colors
-palette_28 = [
-    '#E63946', '#1D3557', '#FFB703', '#2A9D8F', '#9B2226', '#4361EE', '#FF9F1C', '#560BAD',
-    '#06D6A0', '#BC4749', '#4895EF', '#F4D03F', '#7209B7', '#52B788', '#D00000', '#4CC9F0',
-    '#F7B801', '#3C096C', '#40916C', '#DC2F02', '#48CAE4', '#F9C74F', '#240046', '#081C15',
-    '#9D0208', '#90E0EF', '#F3722C', '#14213D'
-]
-
+palette_28 = sc_color[:28]
 # 56-color palette with clear transitions
-palette_56 = [
-    '#001219', '#94D2BD', '#AE2012', '#3A0CA3', '#FF7B00', '#006466', '#E63946', '#1D3557',
-    '#FFB703', '#2A9D8F', '#9B2226', '#4361EE', '#FF9F1C', '#560BAD', '#06D6A0', '#BC4749',
-    '#4895EF', '#F4D03F', '#7209B7', '#52B788', '#D00000', '#4CC9F0', '#F7B801', '#3C096C',
-    '#40916C', '#DC2F02', '#48CAE4', '#F9C74F', '#240046', '#081C15', '#9D0208', '#90E0EF',
-    '#2D00F7', '#E76F51', '#006400', '#FF4D6D', '#073B4C', '#FF9E00', '#440154', '#55A630',
-    '#7B2CBF', '#FF4800', '#0077B6', '#F72585', '#3D405B', '#588157', '#6A040F', '#023E8A',
-    '#FF006E', '#2B9348', '#8338EC', '#F94144', '#0F4C5C', '#E85D04', '#540B0E', '#1A759F'
-]
 
 # 112-color palette with distinct transitions
-palette_112 = [
-    '#001219', '#94D2BD', '#AE2012', '#3A0CA3', '#FF7B00', '#006466', '#E63946', '#1D3557',
-    '#FFB703', '#2A9D8F', '#9B2226', '#4361EE', '#FF9F1C', '#560BAD', '#06D6A0', '#BC4749',
-    '#4895EF', '#F4D03F', '#7209B7', '#52B788', '#D00000', '#4CC9F0', '#F7B801', '#3C096C',
-    '#40916C', '#DC2F02', '#48CAE4', '#F9C74F', '#240046', '#081C15', '#9D0208', '#90E0EF',
-    '#2D00F7', '#E76F51', '#006400', '#FF4D6D', '#073B4C', '#FF9E00', '#440154', '#55A630',
-    '#7B2CBF', '#FF4800', '#0077B6', '#F72585', '#3D405B', '#588157', '#6A040F', '#023E8A',
-    '#FF006E', '#2B9348', '#8338EC', '#F94144', '#0F4C5C', '#E85D04', '#540B0E', '#1A759F',
-    '#FF0A54', '#2B9348', '#5E60CE', '#F8961E', '#073B3A', '#FF6B6B', '#2D3047', '#70E000',
-    '#5A189A', '#FF7900', '#0096C7', '#FF48B0', '#344E41', '#606C38', '#641220', '#03045E',
-    '#FF0075', '#386641', '#7400B8', '#F3722C', '#27474E', '#F48C06', '#3C1518', '#0077B6',
-    '#FF758F', '#40916C', '#6930C3', '#F9844A', '#1B4965', '#FAA307', '#582F0E', '#0096C7',
-    '#FF477E', '#2D6A4F', '#5E60CE', '#F9C74F', '#004E89', '#FF9500', '#50514F', '#70E000',
-    '#9D4EDD', '#FF6D00', '#219EBC', '#FF5C8A', '#344E41', '#606C38', '#800E13', '#0353A4',
-    '#FF0A54', '#387D44', '#480CA8', '#F3622C', '#1A5653', '#FF9F1C', '#4A4E69', '#55A630'
+cet_g_bw = [
+ '#d60000', '#8c3bff', '#018700', '#00acc6', '#97ff00', '#ff7ed1', '#6b004f', '#ffa52f', '#00009c', '#857067',
+ '#004942', '#4f2a00', '#00fdcf', '#bcb6ff', '#95b379', '#bf03b8', '#2466a1', '#280041', '#dbb3af', '#fdf490',
+ '#4f445b', '#a37c00', '#ff7066', '#3f806e', '#82000c', '#a37bb3', '#344d00', '#9ae4ff', '#eb0077', '#2d000a',
+ '#5d90ff', '#00c61f', '#5701aa', '#001d00', '#9a4600', '#959ea5', '#9a425b', '#001f31', '#c8c300', '#ffcfff',
+ '#00bd9a', '#3615ff', '#2d2424', '#df57ff', '#bde6bf', '#7e4497', '#524f3b', '#d86600', '#647438', '#c17287',
+ '#6e7489', '#809c03', '#bd8a64', '#623338', '#cacdda', '#6beb82', '#213f69', '#a17eff', '#fd03ca', '#75bcfd',
+ '#d8c382', '#cda3cd', '#6d4f00', '#006974', '#469e5d', '#93c6bf', '#f9ff00', '#bf5444', '#00643b', '#5b4fa8',
+ '#521f64', '#4f5eff', '#7e8e77', '#b808f9', '#8a91c3', '#b30034', '#87607e', '#9e0075', '#ffddc3', '#500800',
+ '#1a0800', '#4b89b5', '#00dfdf', '#c8fff9', '#2f3415', '#ff2646', '#ff97aa', '#03001a', '#c860b1', '#c3a136',
+ '#7c4f3a', '#f99e77', '#566464', '#d193ff', '#2d1f69', '#411a34', '#af9397', '#629e99', '#bcdd7b', '#ff5d93',
+ '#0f2823', '#b8bdac', '#743b64', '#0f000c', '#7e6ebc', '#9e6b3b', '#ff4600', '#7e0087', '#ffcd3d', '#2f3b42',
+ '#fda5ff', '#89013d', '#752b01', '#0a8995', '#050052', '#8ed631', '#52c372', '#465970', '#570121', '#a52101',
+ '#90934b', '#00421d', '#8000d1', '#2f263f', '#bf3883', '#f4ffd4', '#00d3ff', '#6900f7', '#9cbad1', '#79d8aa',
+ '#69565d', '#006905', '#36369c', '#018246', '#441d18', '#07a5ef', '#ff802f', '#a754b8', '#675982', '#72ffff',
+ '#d88701', '#bad3ff', '#8e362f', '#a7a080', '#007ce2', '#8e7e8e', '#994487', '#00f034', '#aeaac8', '#a06062',
+ '#4b3a77', '#6b8282', '#f0dde6', '#ffbad3', '#38a523', '#b3ffa8', '#0c1107', '#d6526e', '#959efd', '#7c7e00',
+ '#759eb8', '#db877e', '#111318', '#d482d4', '#9e00bf', '#dbefff', '#8eaa9a', '#706442', '#493b3d', '#084d5e',
+ '#9cb844', '#d8ddd4', '#caff6b', '#b364eb', '#465d33', '#009e7c', '#c14100', '#4fbcba', '#d88ab1', '#5b72b5',
+ '#4b4101', '#95825d', '#49748a', '#ff72ff', '#82691c', '#dbcfff', '#7e6bfd', '#627560', '#ffc191', '#595d00',
+ '#e408e6', '#b8b1b6', '#d32d41', '#314236', '#d8a362', '#5b8a33', '#2f1f00', '#97e6d6', '#2a6256', '#cd724d',
+ '#5d3d28', '#0059d8', '#ac93d6', '#6b1d93', '#b3015d', '#410046', '#9cffcf', '#e4489c', '#e2e246', '#dbe2a5',
+ '#002859', '#aa5b82', '#0000db', '#4b4d50', '#dabfd4', '#004d99', '#87649e', '#691d1c', '#8e52c4', '#b8dadf',
+ '#ddb3fd', '#7b4854', '#4b7200', '#440077', '#b15e00', '#91d185', '#54334b', '#69af85', '#aa93af', '#e65442',
+ '#8e8c89', '#70ac50', '#aa7c74', '#00343b', '#240f13', '#e6af00', '#79ccdb', '#18133a', '#9c5238', '#ba7b31',
+ '#b6ca93', '#310800', '#a39505', '#00daba', '#74a0dd', '#623b72', '#ffda8e', '#77b800', '#3f2f1c', '#578759',
+ '#2d0021', '#f4a1d4', '#da00aa', '#752849', '#bce400', '#c3c15d'
 ]
 
+palette_112 = cet_g_bw[:112]
+palette_56 = cet_g_bw[:56]
+
 # Vibrant palette with clear distinctions (24 colors)
 vibrant_palette = [
     '#FF0000', '#00FF00', '#0000FF', '#FFFF00', '#FF00FF', '#00FFFF', 
@@ -110,74 +126,6 @@ sc_color_cmap = LinearSegmentedColormap.from_list('Custom', sc_color, len(sc_col
 
 
 
-omics="""
-   ____            _     _    __                  
-  / __ \____ ___  (_)___| |  / /__  _____________ 
- / / / / __ `__ \/ / ___/ | / / _ \/ ___/ ___/ _ \ 
-/ /_/ / / / / / / / /__ | |/ /  __/ /  (__  )  __/ 
-\____/_/ /_/ /_/_/\___/ |___/\___/_/  /____/\___/                                              
-"""
-days_christmas="""
-      .
-   __/ \__
-   \     /
-   /.'o'.\
-    .o.'.         Merry Christmas!
-   .'.'o'.
-  o'.o.'.o.
- .'.o.'.'.o.       ____ 
-.o.'.o.'.o.'.     / __ \____ ___  (_)___| |  / /__  _____________ 
-   [_____]       / / / / __ `__ \/ / ___/ | / / _ \/ ___/ ___/ _ \ 
-    \___/       / /_/ / / / / / / / /__ | |/ /  __/ /  (__  )  __/ 
-                \____/_/ /_/ /_/_/\___/ |___/\___/_/  /____/\___/
-"""
-#Tua Xiong
-days_chinese_new_year="""
-                                        ,   ,
-                                        $,  $,     ,
-                                        "ss.$ss. .s'
-                                ,     .ss$$$$$$$$$$s,
-                                $. s$$$$$$$$$$$$$$`$$Ss
-                                "$$$$$$$$$$$$$$$$$$o$$$       ,
-                               s$$$$$$$$$$$$$$$$$$$$$$$$s,  ,s
-                              s$$$$$$$$$"$$$$$$ssss$$$$$$"$$$$$,
-                              s$$$$$$$$$$sss$$$$ssssss"$$$$$$$$"
-                             s$$$$$$$$$$'         `\"\"\"ss"$"$s\"\"
-                             s$$$$$$$$$$,              `\"\"\"\"\"$  .s$$s
-                             s$$$$$$$$$$$$s,...               `s$$'  `
-                         `ssss$$$$$$$$$$$$$$$$$$$$####s.     .$$"$.   , s-
-                           `""\""$$$$$$$$$$$$$$$$$$$$#####$$$$$$"     $.$'
-                                 "$$$$$$$$$$$$$$$$$$$$$####s""     .$$$|
-                                  "$$$$$$$$$$$$$$$$$$$$$$$$##s    .$$" $
-                                   $$""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"   `
-                                  $$"  "$"$$$$$$$$$$$$$$$$$$$$S""\""'
-                             ,   ,"     '  $$$$$$$$$$$$$$$$####s
-                             $.          .s$$$$$$$$$$$$$$$$$####"
-                 ,           "$s.   ..ssS$$$$$$$$$$$$$$$$$$$####"
-                 $           .$$$S$$$$$$$$$$$$$$$$$$$$$$$$#####"
-                 Ss     ..sS$$$$$$$$$$$$$$$$$$$$$$$$$$$######""
-                  "$$sS$$$$$$$$$$$$$$$$$$$$$$$$$$$########"
-           ,      s$$$$$$$$$$$$$$$$$$$$$$$$#########""'
-           $    s$$$$$$$$$$$$$$$$$$$$$#######""'      s'         ,
-           $$..$$$$$$$$$$$$$$$$$$######"'       ....,$$....    ,$
-            "$$$$$$$$$$$$$$$######"' ,     .sS$$$$$$$$$$$$$$$$s$$
-              $$$$$$$$$$$$#####"     $, .s$$$$$$$$$$$$$$$$$$$$$$$$s.
-   )          $$$$$$$$$$$#####'      `$$$$$$$$$###########$$$$$$$$$$$.
-  ((          $$$$$$$$$$$#####       $$$$$$$$###"       "####$$$$$$$$$$
-  ) \         $$$$$$$$$$$$####.     $$$$$$###"             "###$$$$$$$$$   s'
- (   )        $$$$$$$$$$$$$####.   $$$$$###"                ####$$$$$$$$s$$'
- )  ( (       $$"$$$$$$$$$$$#####.$$$$$###' -OmicVerse     .###$$$$$$$$$$"
- (  )  )   _,$"   $$$$$$$$$$$$######.$$##'                .###$$$$$$$$$$
- ) (  ( \.         "$$$$$$$$$$$$$#######,,,.          ..####$$$$$$$$$$$"
-(   )$ )  )        ,$$$$$$$$$$$$$$$$$$####################$$$$$$$$$$$"
-(   ($$  ( \     _sS"  `"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$S$$,
- )  )$$$s ) )  .      .   `$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"'  `$$
-  (   $$$Ss/  .$,    .$,,s$$$$$$##S$$$$$$$$$$$$$$$$$$$$$$$$S""        '
-    \)_$$$$$$$$$$$$$$$$$$$$$$$##"  $$        `$$.        `$$.
-        `"S$$$$$$$$$$$$$$$$$#"      $          `$          `$
-            `\"""\""\""\""\""\""'         '           '           '
-"""
-
 spring_festival = { 
     2022: datetime(2022, 2, 1), 
     2023: datetime(2023, 1, 22), 
@@ -216,21 +164,48 @@ EMOJI = {
 }
 
 
+@register_function(
+    aliases=["绘图设置", "plot_set", "ov_plot_set", "plotset", "设置绘图"],
+    category="utils",
+    description="Configure plotting settings for OmicVerse including matplotlib, scanpy, and GPU detection",
+    examples=[
+        "ov.utils.ov_plot_set()",
+        "ov.utils.plot_set(dpi=100, figsize=6)",
+        "ov.utils.plot_set(scanpy=False, fontsize=12)",
+        "ov.utils.plot_set(vector_friendly=True)"
+    ],
+    related=["pl.embedding", "pl.volcano", "utils.palette"]
+)
 def plot_set(verbosity: int = 3, dpi: int = 80, 
-             facecolor: str = 'white', font_path: str = None):
+             facecolor: str = 'white', 
+             font_path: str = None,
+             ipython_format: str  = "retina",
+             dpi_save: int = 300,
+             transparent: bool = None,
+             scanpy: bool = True,
+             fontsize: int = 14,
+             color_map: Union[str, None] = None,
+             figsize: Union[int, None] = None,
+             vector_friendly: bool = True,
+             ):
     r"""Configure plotting settings for OmicVerse.
     
-    Sets up scanpy verbosity, matplotlib parameters, suppresses warnings,
-    detects GPU devices, and displays the OmicVerse logo.
-    
     Arguments:
-        verbosity: Scanpy verbosity level (3)
-        dpi: Resolution for matplotlib figures (80) 
-        facecolor: Background color for figures ('white')
-        font_path: Path to font for custom fonts (None)
+        verbosity: Scanpy verbosity level. Default: 3.
+        dpi: Resolution for matplotlib figures. Default: 80.
+        facecolor: Background color for figures. Default: 'white'.
+        font_path: Path to font for custom fonts. Default: None.
+        ipython_format: IPython display format. Default: 'retina'.
+        dpi_save: Resolution for saved figures. Default: 300.
+        transparent: Whether to use transparent background. Default: None.
+        scanpy: Whether to apply scanpy settings. Default: True.
+        fontsize: Default font size for plots. Default: 14.
+        color_map: Default color map for plots. Default: None.
+        figsize: Default figure size. Default: None.
+        vector_friendly: Control rasterization for vector-friendly plots. Default: True.
         
     Returns:
-        None
+        None: The function configures global plotting settings and displays initialization information.
     """
     global _has_printed_logo
 
@@ -244,7 +219,31 @@ def plot_set(verbosity: int = 3, dpi: int = 80,
     # 2) scanpy verbosity & figure params
     #print(f"{EMOJI['settings']} Applying plotting settings (verbosity={verbosity}, dpi={dpi})")
     sc.settings.verbosity = verbosity
-    sc.settings.set_figure_params(dpi=dpi, facecolor=facecolor)
+    import builtins
+    is_ipython = getattr(builtins, "__IPYTHON__", False)
+    if is_ipython:
+        from matplotlib_inline.backend_inline import set_matplotlib_formats
+        ipython_format = [ipython_format]
+        set_matplotlib_formats(*ipython_format)
+    
+    from matplotlib import rcParams
+    if dpi is not None:
+        rcParams["figure.dpi"] = dpi
+    if dpi_save is not None:
+        rcParams["savefig.dpi"] = dpi_save
+    if transparent is not None:
+        rcParams["savefig.transparent"] = transparent
+    if facecolor is not None:
+        rcParams["figure.facecolor"] = facecolor
+        rcParams["axes.facecolor"] = facecolor
+    if scanpy:
+        set_rcParams_scanpy(fontsize=fontsize, color_map=color_map)
+    if figsize is not None:
+        rcParams["figure.figsize"] = figsize
+    
+    # Set global vector_friendly setting
+    global _vector_friendly
+    _vector_friendly = vector_friendly
     #print(f"{EMOJI['done']} Settings applied")
 
     # 3) Custom font setup
@@ -312,18 +311,56 @@ def plot_set(verbosity: int = 3, dpi: int = 80,
     #print(f"{EMOJI['done']} Warnings suppressed")
 
     # 5) GPU detection
-    print(f"{EMOJI['gpu']} Detecting CUDA devices…")
-    if torch is None or not torch.cuda.is_available():
-        print(f"{EMOJI['warnings']} No CUDA devices found")
-    else:
+    print(f"{EMOJI['gpu']} Detecting GPU devices…")
+    gpu_found = False
+    
+    # Check CUDA devices
+    if torch is not None and torch.cuda.is_available():
         try:
-            for idx in range(torch.cuda.device_count()):
+            cuda_count = torch.cuda.device_count()
+            print(f"{EMOJI['done']} NVIDIA CUDA GPUs detected: {cuda_count}")
+            for idx in range(cuda_count):
                 props = torch.cuda.get_device_properties(idx)
-                print(f"{EMOJI['done']} [GPU {idx}] {props.name}")
-                print(f"    • Total memory: {props.total_memory/1024**3:.1f} GB")
-                print(f"    • Compute capability: {props.major}.{props.minor}")
+                print(f"    • [CUDA {idx}] {props.name}")
+                print(f"      Memory: {props.total_memory/1024**3:.1f} GB | Compute: {props.major}.{props.minor}")
+            gpu_found = True
         except Exception as e:
-            print(f"{EMOJI['warnings']} GPU detection failed: {e}")
+            print(f"{EMOJI['warnings']} CUDA detection failed: {e}")
+    
+    # Check Apple Silicon MPS
+    if torch is not None and hasattr(torch.backends, 'mps') and torch.backends.mps.is_available():
+        try:
+            print(f"{EMOJI['done']} Apple Silicon MPS detected")
+            print(f"    • [MPS] Apple Silicon GPU - Metal Performance Shaders available")
+            gpu_found = True
+        except Exception as e:
+            print(f"{EMOJI['warnings']} MPS detection failed: {e}")
+    
+    # Check AMD ROCm
+    if torch is not None and hasattr(torch.version, 'hip') and torch.version.hip is not None:
+        try:
+            print(f"{EMOJI['done']} AMD ROCm GPU detected")
+            print(f"    • [ROCm] AMD GPU - HIP version: {torch.version.hip}")
+            gpu_found = True
+        except Exception as e:
+            print(f"{EMOJI['warnings']} ROCm detection failed: {e}")
+    
+    # Check Intel XPU
+    if torch is not None and hasattr(torch, 'xpu') and torch.xpu.is_available():
+        try:
+            xpu_count = torch.xpu.device_count()
+            print(f"{EMOJI['done']} Intel XPU detected: {xpu_count}")
+            for idx in range(xpu_count):
+                print(f"    • [XPU {idx}] Intel GPU")
+            gpu_found = True
+        except Exception as e:
+            print(f"{EMOJI['warnings']} Intel XPU detection failed: {e}")
+    
+    if not gpu_found:
+        if torch is None:
+            print(f"{EMOJI['warnings']} PyTorch not available - GPU detection skipped")
+        else:
+            print(f"{EMOJI['warnings']} No GPU devices found (CUDA/MPS/ROCm/XPU)")
 
     # 6) print logo & version only once
     if not _has_printed_logo:
@@ -339,6 +376,9 @@ def plot_set(verbosity: int = 3, dpi: int = 80,
         print(f"🔖 Version: {__version__}   📚 Tutorials: https://omicverse.readthedocs.io/")
         _has_printed_logo = True
 
+    import matplotlib.pyplot as plt
+    plt.rcParams['axes.grid'] = False
+
     print(f"{EMOJI['done']} plot_set complete.\n")
 
 
@@ -346,6 +386,9 @@ def plot_set(verbosity: int = 3, dpi: int = 80,
 plotset = plot_set
 ov_plot_set = plot_set
 
+
+
+
 def pyomic_palette()->list:
     r"""Returns the default OmicVerse color palette.
     
@@ -807,10 +850,10 @@ def plot_embedding_celltype(adata:anndata.AnnData,figsize:tuple=(6,4),basis:str=
                         adata.obs[celltype_key].cat.categories):
         ax2.scatter(100,
                 cell,c=cell_color_dict[cell],s=50)
-        ax2.plot((100,cell_num_pd.loc[cell,celltype_key]),(idx,idx),
+        ax2.plot((100,cell_num_pd.loc[cell,'count']),(idx,idx),
                 c=cell_color_dict[cell],lw=4)
         ax2.text(100,idx+0.2,
-                cell+'('+str("{:,}".format(cell_num_pd.loc[cell,celltype_key]))+')',fontsize=11)
+                cell+'('+str("{:,}".format(cell_num_pd.loc[cell,'count']))+')',fontsize=11)
     ax2.set_xlim(xlim,cell_num_pd.iloc[1].values[0]) 
     ax2.text(xlim,idx+1,title,fontsize=12)
     ax2.grid(False)
@@ -822,19 +865,39 @@ def plot_embedding_celltype(adata:anndata.AnnData,figsize:tuple=(6,4),basis:str=
 
     return fig,[ax1,ax2]
 
+@register_function(
+    aliases=["标签生成", "gen_mpl_labels", "cluster_labels", "添加标签", "embedding_labels"],
+    category="utils",
+    description="Add cluster labels at median positions in embedding plots with automatic text positioning",
+    examples=[
+        "# Basic cluster labeling",
+        "fig, ax = plt.subplots()",
+        "ov.utils.embedding(adata, basis='X_umap', color='leiden', ax=ax)",
+        "ov.utils.gen_mpl_labels(adata, groupby='leiden', ax=ax)",
+        "# Custom basis and text styling",
+        "ov.utils.gen_mpl_labels(adata, groupby='celltype', basis='X_tsne',",
+        "                        text_kwargs={'fontsize': 12, 'weight': 'bold'})",
+        "# Exclude specific clusters",
+        "ov.utils.gen_mpl_labels(adata, groupby='leiden', exclude=['0', '1'])",
+        "# With automatic text adjustment",
+        "ov.utils.gen_mpl_labels(adata, groupby='leiden', ax=ax,", 
+        "                        adjust_kwargs={'arrowprops': {'arrowstyle': '->'}})"
+    ],
+    related=["utils.embedding", "utils.plot_ConvexHull", "pl.umap"]
+)
 def gen_mpl_labels(
     adata, groupby, exclude=(), 
     basis='X_umap',ax=None, adjust_kwargs=None, text_kwargs=None
 ):
-    r"""Add cluster labels at median positions in embedding plots.
+    """Add cluster labels at median positions in embedding plots with automatic text positioning.
     
     Arguments:
-        adata: AnnData object
-        groupby: Column name for grouping
-        exclude: Groups to exclude from labeling (())
-        basis: Embedding basis name ('X_umap')
-        ax: Matplotlib axes object (None)
-        adjust_kwargs: Parameters for text adjustment (None)
+        adata: AnnData object containing single-cell data.
+        groupby: Column name for grouping in adata.obs.
+        exclude: Groups to exclude from labeling. Default: ().
+        basis: Embedding basis name in adata.obsm. Default: 'X_umap'.
+        ax: Matplotlib axes object. Default: None.
+        adjust_kwargs: Parameters for adjustText text adjustment. Default: None.
         text_kwargs: Parameters for text styling (None)
         
     Returns:
@@ -916,6 +979,26 @@ def normalize_to_minus_one_to_one(arr):
     
     return normalized_arr
 
+@register_function(
+    aliases=["堆叠火山图", "stacking_vol", "multiple_volcano", "火山图组合", "比较火山图"],
+    category="utils", 
+    description="Create stacked volcano plots for comparing differential expression across multiple datasets",
+    examples=[
+        "# Basic stacked volcano plot",
+        "data_dict = {'Dataset1': deg_df1, 'Dataset2': deg_df2}",
+        "color_dict = {'Dataset1': 'red', 'Dataset2': 'blue'}",
+        "fig, axes = ov.utils.stacking_vol(data_dict, color_dict)",
+        "# Custom thresholds and styling",
+        "fig, axes = ov.utils.stacking_vol(data_dict, color_dict,",
+        "                                 pval_threshold=0.05, log2fc_threshold=1.5,",
+        "                                 figsize=(10,6), plot_genes_num=15)",
+        "# Three-way comparison",
+        "data_dict = {'Control': deg1, 'Treatment1': deg2, 'Treatment2': deg3}",
+        "color_dict = {'Control': '#1f77b4', 'Treatment1': '#ff7f0e', 'Treatment2': '#2ca02c'}",
+        "fig, axes = ov.utils.stacking_vol(data_dict, color_dict)"
+    ],
+    related=["pl.volcano", "bulk.get_deg", "single.cosg"]
+)
 def stacking_vol(data_dict:dict,color_dict:dict,
                  pval_threshold:float=0.01,
                  log2fc_threshold:int=2,
@@ -925,10 +1008,10 @@ def stacking_vol(data_dict:dict,color_dict:dict,
                  plot_genes_num:int=10,
                  plot_genes_fontsize:int=8,
                 plot_genes_weight:str='bold')->tuple:
-    r"""Create stacked volcano plots for comparing multiple datasets.
+    """Create stacked volcano plots for comparing differential expression across multiple datasets.
     
     Arguments:
-        data_dict: Dictionary with DataFrames containing 'logfoldchanges', 'pvals_adj', 'names'
+        data_dict: Dictionary with DataFrames containing 'logfoldchanges', 'pvals_adj', 'names' columns.
         color_dict: Dictionary mapping dataset names to colors
         pval_threshold: P-value significance threshold (0.01)
         log2fc_threshold: Log2 fold change threshold (2)
@@ -1012,21 +1095,50 @@ def stacking_vol(data_dict:dict,color_dict:dict,
     return fig,axes
 
 
+@register_function(
+    aliases=["凸包绘制", "plot_ConvexHull", "convex_hull", "凸包轮廓", "cluster_hull"],
+    category="utils",
+    description="Add convex hull outline for a specific cluster in embedding plot",
+    examples=[
+        "# Basic convex hull for a cluster",
+        "fig, ax = plt.subplots()",
+        "ov.utils.embedding(adata, basis='X_umap', color='leiden', ax=ax)",
+        "ov.utils.plot_ConvexHull(adata, basis='X_umap', cluster_key='leiden',",
+        "                         hull_cluster='0', ax=ax)",
+        "# Custom color and transparency",
+        "ov.utils.plot_ConvexHull(adata, basis='X_tsne', cluster_key='celltype',",
+        "                         hull_cluster='T cells', ax=ax, color='red', alpha=0.3)",
+        "# Multiple cluster hulls",
+        "for cluster in ['0', '1', '2']:",
+        "    ov.utils.plot_ConvexHull(adata, basis='X_umap', cluster_key='leiden',",
+        "                             hull_cluster=cluster, ax=ax)"
+    ],
+    related=["utils.embedding", "pl.umap", "pl.tsne"]
+)
 def plot_ConvexHull(adata:anndata.AnnData,basis:str,cluster_key:str,
                     hull_cluster:str,ax,color=None,alpha:float=0.2):
-    r"""Add convex hull outline for a specific cluster in embedding plot.
+    """Add convex hull outline for a specific cluster in embedding plot.
     
     Arguments:
-        adata: AnnData object
-        basis: Embedding basis name in adata.obsm
-        cluster_key: Column name for cluster assignments
-        hull_cluster: Specific cluster to outline
-        ax: Matplotlib axes object
-        color: Hull color (None for automatic)
-        alpha: Hull transparency (0.2)
+        adata: AnnData object containing single-cell data.
+        basis: Embedding basis name in adata.obsm (e.g., 'X_umap', 'X_tsne').
+        cluster_key: Column name for cluster assignments in adata.obs.
+        hull_cluster: Specific cluster identifier to outline.
+        ax: Matplotlib axes object to draw on.
+        color: Hull color. Default: None (automatic).
+        alpha: Hull transparency level. Default: 0.2.
         
     Returns:
-        Modified matplotlib axes object
+        ax: Modified matplotlib axes object with convex hull added.
+
+    Examples:
+        >>> import omicverse as ov
+        >>> import matplotlib.pyplot as plt
+        >>> # Create embedding plot with convex hull
+        >>> fig, ax = plt.subplots()
+        >>> ov.utils.embedding(adata, basis='X_umap', color='leiden', ax=ax)
+        >>> ov.utils.plot_ConvexHull(adata, basis='X_umap', cluster_key='leiden',
+        ...                          hull_cluster='0', ax=ax)
     """
     
     adata.obs[cluster_key]=adata.obs[cluster_key].astype('category')
@@ -1238,6 +1350,22 @@ class geneset_wordcloud(object):
 
 from scanpy.plotting._anndata import ranking
 from scanpy.plotting._utils import savefig_or_show
+@register_function(
+    aliases=["主成分方差比", "plot_pca_variance_ratio", "pca_variance", "PCA方差", "主成分分析方差"],
+    category="utils",
+    description="Plot PCA variance ratio to determine optimal number of principal components",
+    examples=[
+        "# Basic PCA variance ratio plot",
+        "ov.pp.pca(adata, n_pcs=50)",
+        "ov.utils.plot_pca_variance_ratio(adata, n_pcs=30)",
+        "# Custom variance ratios with log scale",
+        "ov.utils.plot_pca_variance_ratio(adata, n_pcs=50, log=True)",
+        "# Check different representation",
+        "ov.utils.plot_pca_variance_ratio(adata, use_rep='scaled|original|pca_var_ratios',",
+        "                                 n_pcs=20)"
+    ],
+    related=["pp.pca", "pp.scale", "utils.cluster"]
+)
 def plot_pca_variance_ratio(
     adata,
     use_rep='scaled|original|pca_var_ratios',
@@ -1246,6 +1374,27 @@ def plot_pca_variance_ratio(
     show=None,
     save=None,
 ):
+    r"""Plot PCA variance ratio to determine optimal number of principal components.
+
+    Arguments:
+        adata: AnnData object containing PCA results.
+        use_rep: Key in adata.uns for variance ratios. Default: 'scaled|original|pca_var_ratios'.
+        n_pcs: Number of principal components to plot. Default: 30.
+        log: Whether to use logarithmic scale. Default: False.
+        show: Show the figure. Default: None.
+        save: Save the figure to file. Default: None.
+
+    Returns:
+        None: Displays or saves the PCA variance ratio plot.
+
+    Examples:
+        >>> import omicverse as ov
+        >>> # Basic PCA variance ratio plot
+        >>> ov.pp.pca(adata, n_pcs=50)
+        >>> ov.utils.plot_pca_variance_ratio(adata, n_pcs=30)
+        >>> # Custom variance ratios with log scale
+        >>> ov.utils.plot_pca_variance_ratio(adata, n_pcs=50, log=True)
+    """
     ranking(
         adata,
         "uns",
@@ -1300,8 +1449,112 @@ def check_dependencies(dependencies=None, check_full=False):
             print(f"Warning: Could not read dependencies from pyproject.toml: {e}")
             return
 
-    try:
-        pkg_resources.require(dependencies)
+    # try:
+    #     pkg_resources.require(dependencies)
+    #     print("All dependencies are satisfied.")
+    # except (DistributionNotFound, VersionConflict) as e:
+    #     print(f"Dependency error: {e}")
+    
+    import importlib.metadata as importlib_metadata
+    for req in dependencies:
+        try:
+            importlib_metadata.distribution(req)  # Validate package installation and version constraints
+        except importlib_metadata.PackageNotFoundError as e:
+            print(f"Missing dependency: {req!r}: {e}")
+        except Exception as e:
+            # Handle version conflicts or other issues
+            print(f"Dependency error for {req!r}: {e}")
+    else:
         print("All dependencies are satisfied.")
-    except (DistributionNotFound, VersionConflict) as e:
-        print(f"Dependency error: {e}")
\ No newline at end of file
+
+
+
+
+def set_rcParams_scanpy(fontsize=14, color_map=None):
+    """Set matplotlib.rcParams to Scanpy defaults.
+
+    Call this through :func:`scanpy.set_figure_params`.
+    """
+    # figure
+    import matplotlib as mpl
+    from cycler import cycler
+    
+    rcParams["figure.figsize"] = (4, 4)
+    rcParams["figure.subplot.left"] = 0.18
+    rcParams["figure.subplot.right"] = 0.96
+    rcParams["figure.subplot.bottom"] = 0.15
+    rcParams["figure.subplot.top"] = 0.91
+
+    rcParams["lines.linewidth"] = 1.5  # the line width of the frame
+    rcParams["lines.markersize"] = 6
+    rcParams["lines.markeredgewidth"] = 1
+
+    # font
+    rcParams["font.sans-serif"] = [
+        "Arial",
+        "Helvetica",
+        "DejaVu Sans",
+        "Bitstream Vera Sans",
+        "sans-serif",
+    ]
+    rcParams["font.size"] = fontsize
+    rcParams["legend.fontsize"] = 0.92 * fontsize
+    rcParams["axes.titlesize"] = fontsize
+    rcParams["axes.labelsize"] = fontsize
+
+    # legend
+    rcParams["legend.numpoints"] = 1
+    rcParams["legend.scatterpoints"] = 1
+    rcParams["legend.handlelength"] = 0.5
+    rcParams["legend.handletextpad"] = 0.4
+
+    # color cycles
+    rcParams["axes.prop_cycle"] = cycler(color=sc_color)
+
+    # lines
+    rcParams["axes.linewidth"] = 0.8
+    rcParams["axes.edgecolor"] = "black"
+    rcParams["axes.facecolor"] = "white"
+
+    # ticks
+    rcParams["xtick.color"] = "k"
+    rcParams["ytick.color"] = "k"
+    rcParams["xtick.labelsize"] = fontsize
+    rcParams["ytick.labelsize"] = fontsize
+
+    # axes grid
+    rcParams["axes.grid"] = True
+    rcParams["grid.color"] = ".8"
+
+    # color map
+    rcParams["image.cmap"] = rcParams["image.cmap"] if color_map is None else color_map
+
+
+def set_rcParams_defaults():
+    """Reset `matplotlib.rcParams` to defaults."""
+    rcParams.update(mpl.rcParamsDefault)
+
+
+omics=r"""
+   ____            _     _    __                  
+  / __ \____ ___  (_)___| |  / /__  _____________ 
+ / / / / __ `__ \/ / ___/ | / / _ \/ ___/ ___/ _ \ 
+/ /_/ / / / / / / / /__ | |/ /  __/ /  (__  )  __/ 
+\____/_/ /_/ /_/_/\___/ |___/\___/_/  /____/\___/                                              
+"""
+days_christmas=r"""
+      .
+   __/ \__
+   \     /
+   /.'o'.\
+    .o.'.         Merry Christmas!
+   .'.'o'.
+  o'.o.'.o.
+ .'.o.'.'.o.       ____ 
+.o.'.o.'.o.'.     / __ \____ ___  (_)___| |  / /__  _____________ 
+   [_____]       / / / / __ `__ \/ / ___/ | / / _ \/ ___/ ___/ _ \ 
+    \___/       / /_/ / / / / / / / /__ | |/ /  __/ /  (__  )  __/ 
+                \____/_/ /_/ /_/_/\___/ |___/\___/_/  /____/\___/
+"""
+#Tua Xiong
+days_chinese_new_year=""""""
\ No newline at end of file
